/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001, Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
  [my current email address is on the web servers]

                                  -----
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_priv.h"
#ifdef LCRZODEF_SYSTEM_Linux
 #define _GNU_SOURCE
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <signal.h>
 #include <termios.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>  
 #include <sys/wait.h>
 #include <netinet/in.h>    
#elif defined LCRZODEF_SYSTEM_FreeBSD
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <signal.h>
 #include <termios.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>      
 #include <sys/wait.h>
 #include <netinet/in.h>    
#elif defined LCRZODEF_SYSTEM_Solaris
 #define __EXTENSIONS__
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <signal.h>
 #include <termios.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/stat.h>      
 #include <sys/wait.h>
 #include <netinet/in.h>    
#else
 #error "Traiter le cas de LCRZODEF_SYSTEM"
#endif

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_and(lcrzo_etha a, lcrzo_etha b, lcrzo_etha resu)
{ int i;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  if (b==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(a[i]&b[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_or(lcrzo_etha a, lcrzo_etha b, lcrzo_etha resu)
{ int i;
  
  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  if (b==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(a[i]|b[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_xor(lcrzo_etha a, lcrzo_etha b, lcrzo_etha resu)
{ int i;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  if (b==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(a[i]^b[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_not(lcrzo_etha a, lcrzo_etha resu)
{ int i;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { resu[i]=(lcrzo_uint8)(~a[i]);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_inc(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=1;
    for ( i=LCRZO_ETHA_MAXBYTES-1 ; i>=0 ; i-- )
    { if ( a[i]==0xFF && retenue ) /*il va y avoir debordement*/
      { resu[i]=0;
      }
      else
      { resu[i]=(lcrzo_uint8)(a[i]+retenue);
        retenue=0; /*plus de retenue a propager*/
      }
    }
  }
  /*on ignore le debordement de ff:ff:ff:ff:ff:ff*/
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_dec(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=1;
    for ( i=LCRZO_ETHA_MAXBYTES-1 ; i>=0 ; i-- )
    { if ( a[i]==0 && retenue ) /*il va y avoir debordement*/
      { resu[i]=0xFF;
      }
      else
      { resu[i]=(lcrzo_uint8)(a[i]-retenue);
        retenue=0; /*plus de retenue a propager*/
      }
    }
  }
  /*on ignore le debordement de 0:0:0:0:0:0*/
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_shl(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue, r;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=0;
    for ( i=LCRZO_ETHA_MAXBYTES-1 ; i>=0 ; i-- )
    { r=(lcrzo_uint8)(a[i]&0x80);
      resu[i]=(lcrzo_uint8)((a[i]<<1)|retenue);
      retenue=(lcrzo_uint8)(r>>7);
    }
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_shr(lcrzo_etha a, lcrzo_etha resu)
{ int i;
  lcrzo_uint8 retenue, r;

  /*parameters verification*/
  if (a==NULL) return(LCRZO_ERR_PANULLPTR);
  
  if (resu!=NULL)
  { retenue=0;
    for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
    { r=(lcrzo_uint8)(a[i]&1);
      resu[i]=(lcrzo_uint8)((a[i]>>1)|(retenue<<7));
      retenue=r;
    }
  }
  return(LCRZO_ERR_OK);
}


/*---------------------------------------------------------------*/
int lcrzo_priv_hdrleth_equal_addr(lcrzo_hdrleth hdrleth,
				  const lcrzo_etha ethasrc,
				  const lcrzo_etha ethadst)
{ return(lcrzo_etha_equal(hdrleth.src, ethasrc) &&
	 lcrzo_etha_equal(hdrleth.dst, ethadst));
}

/*---------------------------------------------------------------*/
int lcrzo_priv_hdrlip_equal_addr(lcrzo_hdrlip hdrlip,
				 const lcrzo_ipl iplsrc,
				 const lcrzo_ipl ipldst)
{ return(lcrzo_ipl_equal(hdrlip.saddr, iplsrc) &&
	 lcrzo_ipl_equal(hdrlip.daddr, ipldst));
}

/*---------------------------------------------------------------*/
int lcrzo_priv_hdrludp_equal_port(lcrzo_hdrludp hdrludp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst)
{ return( (hdrludp.sport==portsrc) && (hdrludp.dport==portdst) );
}

/*---------------------------------------------------------------*/
int lcrzo_priv_hdrltcp_equal_port(lcrzo_hdrltcp hdrltcp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst)
{ return( (hdrltcp.sport==portsrc) && (hdrltcp.dport==portdst) );
}

/*-------------------------------------------------------------*/
int lcrzo_priv_stdin(lcrzo_string *puserstring,
		     lcrzo_int32 *puserstringsize)
{ char *ptr;
  lcrzo_int32 fgetstep, ptrallocsize, userstringsize;
  int c;

  fgetstep=3;
  ptrallocsize=fgetstep;
  lcrzo_er(lcrzo_string_alloc(ptrallocsize, &ptr));  

  lcrzo_stdin_flush();

  userstringsize=0;
  c=getc(stdin);  
  while ( c!=0x0D && c!=0x0A && c!=0x00 && c!=EOF )
  { ptr[userstringsize]=(char)c;
    userstringsize++;
    c=getc(stdin);
    if (userstringsize==ptrallocsize)
    { ptrallocsize+=fgetstep;
      lcrzo_er(lcrzo_string_realloc(ptrallocsize, &ptr));  
    }
  }
  ptr[userstringsize]='\0';

  if (puserstring!=NULL)
  { *puserstring=ptr;
  }
  else
  { lcrzo_string_free(ptr);
  }
  if (puserstringsize!=NULL) *puserstringsize=userstringsize;

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_passwd(lcrzo_string *puserstring,
		      lcrzo_int32 *puserstringsize)
{ char *ptr;
  lcrzo_int32 fgetstep, ptrallocsize, userstringsize;
  int c;
  struct termios tios;

  fgetstep=3;
  ptrallocsize=fgetstep;
  lcrzo_er(lcrzo_string_alloc(ptrallocsize, &ptr));  

  lcrzo_stdin_flush();

  if (tcgetattr(STDIN_FILENO, &tios)) return(LCRZO_ERR_FUTCGETATTR);
  tios.c_lflag &= ~ECHO;
  if (tcsetattr(STDIN_FILENO, TCSANOW,  &tios)) return(LCRZO_ERR_FUTCSETATTR);
  userstringsize=0;
  c=getc(stdin);  
  while ( c!=0x0D && c!=0x0A && c!=0x00 && c!=EOF )
  { ptr[userstringsize]=(char)c;
    userstringsize++;
    c=getc(stdin);
    if (userstringsize==ptrallocsize)
    { ptrallocsize+=fgetstep;
      lcrzo_er(lcrzo_string_realloc(ptrallocsize, &ptr));  
    }
  }
  ptr[userstringsize]='\0';
  tios.c_lflag |= ECHO;
  if (tcsetattr(STDIN_FILENO, TCSANOW,  &tios)) return(LCRZO_ERR_FUTCSETATTR);
  puts("");

  if (puserstring!=NULL)
  { *puserstring=ptr;
  }
  else
  { lcrzo_string_free(ptr);
  }
  if (puserstringsize!=NULL) *puserstringsize=userstringsize;

  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_ie(lcrzo_conststring functionname,
	     lcrzo_conststring frexplain,
	     lcrzo_conststring usexplain)
{ lcrzo_string string;

  lcrzo_string_initm_text("", &string);
  if ( functionname!=NULL )
  { lcrzo_string_appendm_text(functionname, &string);
    if ( frexplain!=NULL || usexplain!=NULL )
      lcrzo_string_appendm_text(" : " , &string);
  }
  if (lcrzo_global.language==LCRZO_GLOBAL_FRLANG)
  { if ( frexplain!=NULL ) lcrzo_string_appendm_text(frexplain, &string);
    else if ( usexplain!=NULL ) lcrzo_string_appendm_text(usexplain, &string);
  }
  else
  { if ( usexplain!=NULL ) lcrzo_string_appendm_text(usexplain, &string);
    else if ( frexplain!=NULL ) lcrzo_string_appendm_text(frexplain, &string);
  }

  if ( functionname!=NULL || frexplain!=NULL || usexplain!=NULL )
  { lcrzo_string_appendm_text("\n" , &string);
  }

  lcrzo_string_init_text(string,
			 LCRZO_GLOERRMSG_MAXBYTES, lcrzo_global.errmsglcrzo);

  if (lcrzo_global.language==LCRZO_GLOBAL_FRLANG)
  { puts("Erreur interne : contactez Laurent");
  }
  else
  { puts("Internal error : contact Laurent");
  }
  puts(string);
  lcrzo_string_free(string);

  /* this function always return ie */
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*---------------------------------------------------------------*/
int lcrzo_ieuv(lcrzo_conststring functionname,
	       lcrzo_conststring varname)
{ lcrzo_string frstr, usstr;

  lcrzo_string_initm_text("valeur inconnue", &frstr);
  lcrzo_string_initm_text("unknown value", &usstr);
  if (varname!=NULL)
  { lcrzo_string_appendm_text(" pour ", &frstr);
    lcrzo_string_appendm_text(varname, &frstr);
    lcrzo_string_appendm_text(" for ", &usstr);
    lcrzo_string_appendm_text(varname, &usstr);
  }

  lcrzo_ie(functionname, frstr, usstr);
  
  lcrzo_string_free(frstr);
  lcrzo_string_free(usstr);

  /* this function always return ie */
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*---------------------------------------------------------------*/
int lcrzo_ierp(lcrzo_conststring functionname)
{ return(lcrzo_ie(functionname, 
		  "ce point n'aurait pas du etre atteint",
		  "we should not have reached this point"));
}


/*-------------------------------------------------------------*/
int lcrzo_priv_manageelempos(lcrzo_int32 elemcount,
			     lcrzo_int32 position,
			     lcrzo_bool wanttoinsertelem,
			     lcrzo_int32 *pposnegative,
			     lcrzo_int32 *ppospositive,
			     int *pdirection, 
			     lcrzo_int32 *pdistance)
{ int sens;
  lcrzo_int32 distance, pospositif, posnegatif;

  /*verification des parametres*/
  lcrzo_bool_verifbof(wanttoinsertelem);

  /*verification et calcul de la position positivee*/
  if (position==0)
  { return(LCRZO_ERR_PONOZERO);
  }
  else if (position>0)
  { if ( position > elemcount+(wanttoinsertelem?1:0) )
    { return(LCRZO_ERR_POTOOBIG);
    }
    posnegatif=position-elemcount-(wanttoinsertelem?2:1);
    pospositif=position;
  }
  else
  { if ( position < -(elemcount+(wanttoinsertelem?1:0)) )
    { return(LCRZO_ERR_POTOOLOW);
    }
    posnegatif=position;
    pospositif=elemcount+(wanttoinsertelem?2:1)+position;
  }

  /*regarde quel sens prendre*/
  if ( pospositif <= -posnegatif )
  { sens=LCRZO_PRIV_INCREASE;
    distance=pospositif;
  }
  else
  { sens=LCRZO_PRIV_DECREASE;
    distance=-posnegatif;
  }

  if (pdirection!=NULL) *pdirection=sens;
  if (pdistance!=NULL) *pdistance=distance;
  if (pposnegative!=NULL) *pposnegative=posnegatif;
  if (ppospositive!=NULL) *ppospositive=pospositif;

  /*  printf("cnt=%ld pos=%ld ins=%d ==> dir %d, distance %ld [%ld,%ld]\n",
	 elemcount, position, wanttoinsertelem,
	 sens, distance, posnegatif, pospositif);*/
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_manageelemrange(lcrzo_int32 elemcount,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance)
{ lcrzo_int32 infpospositive, suppospositive, infsupdistance;
  int infsupdirection;

  lcrzo_er(lcrzo_priv_manageelempos(elemcount, infposition, 0,
				    pinfposnegative, &infpospositive,
				    pinfdirection, pinfdistance));
  lcrzo_er(lcrzo_priv_manageelempos(elemcount, supposition, 0,
				    psupposnegative, &suppospositive,
				    NULL, NULL));

  /*on calcule le nombre d'elements a voir et le sens*/
  if (suppospositive>=infpospositive)
  { infsupdirection=LCRZO_PRIV_INCREASE;
    infsupdistance=suppospositive-infpospositive;
  }
  else
  { infsupdirection=LCRZO_PRIV_DECREASE;
    infsupdistance=infpospositive-suppospositive;
  }

  if (pinfpospositive!=NULL) *pinfpospositive=infpospositive;
  if (psuppospositive!=NULL) *psuppospositive=suppospositive;
  if (pinfsupdirection!=NULL) *pinfsupdirection=infsupdirection;
  if (pinfsupdistance!=NULL) *pinfsupdistance=infsupdistance;

  /*  printf("pos=%ld*%ld[%ld*%ld] ==> dir %d, distance %ld\n",
	 infposition, supposition, infpospositive, suppospositive, 
	 infsupdirection, infsupdistance);
  */
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_manageelemrangefast(lcrzo_int32 elemcount,
				   lcrzo_int32 infposition,
				   lcrzo_int32 supposition,
				   lcrzo_int32 *pinfposnegative,
				   lcrzo_int32 *pinfpospositive,
				   lcrzo_int32 *psupposnegative,
				   lcrzo_int32 *psuppospositive,
				   int *pinfdirection, 
				   lcrzo_int32 *pinfdistance,
				   int *pinfsupdirection, 
				   lcrzo_int32 *pinfsupdistance)
{ lcrzo_int32 infdist, supdist;

  lcrzo_er(lcrzo_priv_manageelempos(elemcount, infposition, 0, NULL, NULL,
				    NULL, &infdist));
  lcrzo_er(lcrzo_priv_manageelempos(elemcount, supposition, 0, NULL, NULL,
				    NULL, &supdist));

  if (infdist<=supdist)
  { lcrzo_er(lcrzo_priv_manageelemrange(elemcount,
					infposition, supposition,
					pinfposnegative, pinfpospositive,
					psupposnegative, psuppospositive,
					pinfdirection, pinfdistance,
					pinfsupdirection, pinfsupdistance));
  }
  else
  { lcrzo_er(lcrzo_priv_manageelemrange(elemcount,
					supposition, infposition,
					pinfposnegative, pinfpospositive,
					psupposnegative, psuppospositive,
					pinfdirection, pinfdistance,
					pinfsupdirection, pinfsupdistance));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_managedataposneg(lcrzo_int32 datasize,
				lcrzo_int32 negative,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive)
{
  if (negative==0) return(LCRZO_ERR_PONOZERO);
  if (negative<datasize-1)return(LCRZO_ERR_POTOOBIG);
  
  if (pnegative!=NULL) *pnegative=negative;
  if (ppositive!=NULL) *ppositive=negative+datasize+1;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_managedatapospos(lcrzo_int32 datasize,
				lcrzo_int32 positive,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive)
{
  if (positive==0) return(LCRZO_ERR_PONOZERO);
  if (positive>datasize+1)return(LCRZO_ERR_POTOOBIG);
  
  if (pnegative!=NULL) *pnegative=positive-datasize-1;
  if (ppositive!=NULL) *ppositive=positive;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_managedatarange(lcrzo_int32 datasize,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance)
{ lcrzo_int32 infposnegative, infpospositive, supposnegative;
  lcrzo_int32 suppospositive, infdistance, infsupdistance;
  int infdirection, infsupdirection;

  if (infposition==0) return(LCRZO_ERR_PONOZERO);
  if (supposition==0) return(LCRZO_ERR_PONOZERO);

  /*check infposition*/
  if (infposition>0)
  { if (infposition>datasize+1)return(LCRZO_ERR_POTOOBIG);
    infposnegative=infposition-datasize-2;
    infpospositive=infposition;
  }
  else
  { if (-infposition>datasize+1)return(LCRZO_ERR_POTOOLOW);
    infposnegative=infposition;
    infpospositive=datasize+2+infposition;
  }

  /*check supposition*/
  if (supposition>0)
  { if (supposition>datasize+1)return(LCRZO_ERR_POTOOBIG);
    supposnegative=supposition-datasize-2;
    suppospositive=supposition;
  }
  else
  { if (-supposition>datasize+1)return(LCRZO_ERR_POTOOLOW);
    supposnegative=supposition;
    suppospositive=datasize+2+supposition;
  }

  /*calculate infdirection and distance*/
  if (infpospositive>(datasize+1)/2)
  { infdirection=LCRZO_PRIV_DECREASE;
    infdistance=-infposnegative-1;
  }
  else
  { infdirection=LCRZO_PRIV_INCREASE;
    infdistance=infpospositive-1;
  }

  /*on calcule le nombre d'elements a voir et le sens*/
  if (suppospositive>=infpospositive)
  { infsupdirection=LCRZO_PRIV_INCREASE;
    infsupdistance=suppospositive-infpospositive;
  }
  else
  { infsupdirection=LCRZO_PRIV_DECREASE;
    infsupdistance=infpospositive-suppospositive;
  }

  if (pinfposnegative!=NULL) *pinfposnegative=infposnegative;
  if (psupposnegative!=NULL) *psupposnegative=supposnegative;
  if (pinfpospositive!=NULL) *pinfpospositive=infpospositive;
  if (psuppospositive!=NULL) *psuppospositive=suppospositive;
  if (pinfdirection!=NULL) *pinfdirection=infdirection;
  if (pinfdistance!=NULL) *pinfdistance=infdistance;
  if (pinfsupdirection!=NULL) *pinfsupdirection=infsupdirection;
  if (pinfsupdistance!=NULL) *pinfsupdistance=infsupdistance;

  /*
  printf("%ld pos=%ld*%ld[%ld*%ld][%ld*%ld] ==> %d_%ld %d_%ld\n", datasize,
	 infposition, supposition, 
	 infposnegative, supposnegative, infpospositive, suppospositive, 
	 infdirection, infdistance, infsupdirection, infsupdistance);
  */
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_init_data(lcrzo_constdata datain,
				  lcrzo_uint16 datainsize,
				  lcrzo_uint16 dataoutmaxsize,
				  lcrzo_data dataout,
				  lcrzo_uint16 *pdataoutsize)
{ return(lcrzo_priv_data_u16_append_data(datain, datainsize, 0,
				    dataoutmaxsize, dataout, pdataoutsize));
}

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_initm_data(lcrzo_constdata datain,
				   lcrzo_uint16 datainsize,
				   lcrzo_data *pdataout,
				   lcrzo_uint16 *pdataoutsize)
{ lcrzo_data ptr;

  ptr=NULL;
  lcrzo_er(lcrzo_priv_data_u16_appendm_data(datain, datainsize, 0,
					    &ptr, pdataoutsize));
  if (pdataout!=NULL) *pdataout=ptr;
  else lcrzo_data_free(ptr);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_append_data(lcrzo_constdata datain,
				    lcrzo_uint16 datainsize,
				    lcrzo_uint16 dataoutcurrentsize,
				    lcrzo_uint16 dataoutmaxsize,
				    lcrzo_data dataout,
				    lcrzo_uint16 *pdataoutsize)
{
  /*parameters verification*/
  if (datain==NULL && datainsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (dataoutcurrentsize>dataoutmaxsize) return(LCRZO_ERR_PADATACURSUPMAX);

  if (dataoutcurrentsize+datainsize<=dataoutmaxsize)
  { if (dataout!=NULL)
    { memcpy(dataout+dataoutcurrentsize, datain, datainsize);
    }
    if (pdataoutsize!=NULL) 
    { *pdataoutsize=(lcrzo_uint16)(dataoutcurrentsize+datainsize);
    }
    return(LCRZO_ERR_OK);
  }
  else
  { if (dataout!=NULL)
    { memcpy(dataout+dataoutcurrentsize, datain,
             dataoutmaxsize-dataoutcurrentsize);
    }
    if (pdataoutsize!=NULL) *pdataoutsize=dataoutmaxsize;
    return(LCRZO_ERR_OKDATATRUNCATED);
  }
 
  /*should never be here*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_appendm_data(lcrzo_constdata datain,
				     lcrzo_uint16 datainsize,
				     lcrzo_uint16 dataoutcurrentsize,
				     lcrzo_data *pdataout,
				     lcrzo_uint16 *pdataoutsize)
{ lcrzo_uint16 dataoutsize;

  /*parameters verification*/
  if (datain==NULL && datainsize) return(LCRZO_ERR_SPNULLPTRSIZE);

  /*compute the output size*/
  dataoutsize=(lcrzo_uint16)(dataoutcurrentsize+datainsize);
  if (pdataoutsize!=NULL) 
  { *pdataoutsize=(lcrzo_uint16)dataoutsize;
  }

  /*init pdataout*/
  if (pdataout!=NULL)
  { lcrzo_er(lcrzo_data_realloc(dataoutsize, pdataout));
    memcpy(*pdataout+dataoutcurrentsize, datain, datainsize);
  }

  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_priv_fd_myread(int fd,
			 lcrzo_int32 maxdatasizetoread,
			 lcrzo_data *pdataout,
			 lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 nboct;
  lcrzo_data ptr;

  /* parameter verification */
  if (maxdatasizetoread<0) return(LCRZO_ERR_PATOOLOW);

  /* special case */
  if (maxdatasizetoread==0)
  { lcrzo_er(lcrzo_data_alloc(0, pdataout));
    if (pdataoutsize!=NULL) *pdataoutsize=0;
    return(LCRZO_ERR_OK);
  }

  /*read*/
  lcrzo_er(lcrzo_data_alloc(maxdatasizetoread, &ptr));
  nboct=read(fd, ptr, maxdatasizetoread);
  /*une erreur s'est produite*/
  if (nboct==-1)
  { lcrzo_data_free(ptr);
    if (errno==EAGAIN)
    { /*annule l'erreur, c'est simplement que rien n'etait disponible*/
      errno=0;
      return(LCRZO_ERR_OKTEMPDATAEND);
    }
    else if (errno==EBADF)
    { errno=0;
      return(LCRZO_ERR_OKDATAEND);
    }
    else
    { /*c'etait une erreur fatale*/
      return(LCRZO_ERR_FUREAD);
    }
  }

  /*on a lu quelque chose*/
  if (pdataoutsize!=NULL) *pdataoutsize=nboct;

  /*if we read 0, we are at end of file*/
  if (nboct==0)
  { lcrzo_data_free(ptr);
    return(LCRZO_ERR_OKDATAEND);
  }

  if (pdataout!=NULL)
  { lcrzo_er(lcrzo_data_realloc(nboct, &ptr));
    *pdataout=ptr; 
  }
  else
  { lcrzo_data_free(ptr);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_stream_myread(LCRZOFILE *pstream,
			     lcrzo_int32 maxdatasizetoread,
			     lcrzo_data *pdataout,
			     lcrzo_int32 *pdataoutsize)
{ lcrzo_bool beblocking;
  int ret;

  /* parameter verification */
  if (pstream==NULL) return(LCRZO_ERR_PANULLPTR);
  if (maxdatasizetoread<0) return(LCRZO_ERR_PATOOLOW);

  /* special case */
  if (maxdatasizetoread==0)
  { lcrzo_er(lcrzo_data_alloc(0, pdataout));
    if (pdataoutsize!=NULL) *pdataoutsize=0;
    return(LCRZO_ERR_OK);
  }
  
  /*end of file*/
  lcrzo_er(lcrzo_fd_block_get(fileno((FILE*)pstream), &beblocking));
  if (feof((FILE*)pstream)) return(LCRZO_ERR_OKDATAEND);

  /*get a char*/
  ret=fgetc((FILE*)pstream);
  if (ret==EOF)
  { if (beblocking) return(LCRZO_ERR_OKDATAEND);
    else return(LCRZO_ERR_OKTEMPDATAEND);
  }

  /*set values*/
  lcrzo_er(lcrzo_data_initm_char(ret, 1, pdataout, pdataoutsize));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_ipc_readtlv(lcrzo_ipc *pipc,
			   lcrzo_bool beblocking,
			   lcrzo_uint16 *ptype,
			   lcrzo_data *pdataout,
			   lcrzo_int32 *pdataoutsize);
int lcrzo_priv_ipc_myread(lcrzo_ipc *pci, 
			  lcrzo_bool beblocking,
			  lcrzo_int32 maxdatasizetoread,
			  lcrzo_data *pdataout,
			  lcrzo_int32 *pdataoutsize)
{ lcrzo_uint16 type;
  lcrzo_int32 leftsize;
  lcrzo_data p;

  /* parameters verification */
  if (pci==NULL) return(LCRZO_ERR_PANULLPTR);
  if (maxdatasizetoread<0) return(LCRZO_ERR_PATOOLOW);

  /* special case */
  if (maxdatasizetoread==0)
  { lcrzo_er(lcrzo_data_alloc(0, pdataout));
    if (pdataoutsize!=NULL) *pdataoutsize=0;
    return(LCRZO_ERR_OK);
  }

  /* if storeddata is empty, we read the next tlv */
  if ( pci->storeddatasize==0 )
  { lcrzo_er(lcrzo_priv_ipc_readtlv(pci, beblocking, &type,
				    &(pci->storeddata),  
				    &(pci->storeddatasize)));
    /* if we were not blocking, we never reach this point */
    /* we check the received tlv */
    if (type!=LCRZO_PRIV_IPCTYPE_MSG)
    { return(LCRZO_ERR_BUIPCUNWANTEDTYPE);
    }
  }

  /*printf("IPC stored : %ld\n", pci->storeddatasize);
  lcrzo_data_print(pci->storeddata, pci->storeddatasize, LCRZO_PRINTTYPE_DUMP);
  */

  /* if read data is < to maxdatasizetoread, we put everything in dataout */
  if ( pci->storeddatasize<=maxdatasizetoread )
  { if (pdataoutsize!=NULL) *pdataoutsize=pci->storeddatasize;
    if (pdataout==NULL)
    { lcrzo_er(lcrzo_data_free(pci->storeddata));
    }
    else *pdataout=pci->storeddata;
    /* now there is nothing left which is stored */
    pci->storeddatasize=0;
  }
  /* else, we only use an extract */
  else
  { lcrzo_er(lcrzo_data_initm_range(pci->storeddata, pci->storeddatasize,
				    +1, 1+maxdatasizetoread,
				    pdataout, pdataoutsize));
    /* we left the end of datastored */
    leftsize=pci->storeddatasize-maxdatasizetoread;
    lcrzo_er(lcrzo_data_alloc(leftsize, &p));
    memcpy(p, pci->storeddata+maxdatasizetoread, leftsize);
    lcrzo_er(lcrzo_data_free(pci->storeddata));
    pci->storeddata=p;
    pci->storeddatasize=leftsize;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_myrecvfrom(lcrzo_sock *psock,
			       lcrzo_int32 maxdatasizetoread,
			       lcrzo_data *pdataout,
			       lcrzo_int32 *pdataoutsize)
{ lcrzo_int32 leftsize;
  lcrzo_data p;
  struct sockaddr_in sai;
  int sailength, ret;

  /* parameters verification */
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  if (maxdatasizetoread<0) return(LCRZO_ERR_PATOOLOW);

  /* special case */
  if (maxdatasizetoread==0)
  { lcrzo_er(lcrzo_data_alloc(0, pdataout));
    if (pdataoutsize!=NULL) *pdataoutsize=0;
    return(LCRZO_ERR_OK);
  }

  /* if storeddata is empty, we read the next msg */
  if ( psock->storeddatasize==0 )
  { lcrzo_er(lcrzo_data_alloc(LCRZO_PRIV_READMAXMSGSIZE,
			      &(psock->storeddata)));
    sailength=sizeof(struct sockaddr_in);
    psock->storeddatasize=recvfrom(psock->socknum, psock->storeddata, 
				   LCRZO_PRIV_READMAXMSGSIZE, 
				   0, (struct sockaddr *)&sai, &sailength);
    /* an error occured */
    if (psock->storeddatasize==-1)
    { psock->storeddatasize=0;
      lcrzo_data_free(psock->storeddata);
      if (errno==EAGAIN)
      { /*annule l'erreur, c'est simplement que rien n'etait disponible*/
        errno=0;
        return(LCRZO_ERR_OKTEMPDATAEND);
      }
      else if (errno==EBADF)
      { errno=0;
        return(LCRZO_ERR_OKDATAEND);
      }
      else
      { /*c'etait une erreur fatale*/
        return(LCRZO_ERR_FURECVFROM);
      }
    }

    /* we update client/server information */
    psock->there_ipl=lcrzo_ntohl(sai.sin_addr.s_addr);
    psock->there_port=lcrzo_ntohs(sai.sin_port);

    /* on the first read on an UDP serveur, we connect back to the client*/
    if ( psock->canwrite==0 && psock->type==LCRZO_PRIV_TYPESOCK_REALSERUDP )
    { /*on se connecte maintenant sur le client afin de pouvoir y ecrire*/
      ret=connect(psock->socknum, (struct sockaddr *)&sai, sizeof(sai));
      if (ret<0) return(LCRZO_ERR_FUCONNECT);
      /*maintenant, comme on connait le client, on pourra ecrire*/
      psock->canwrite=1;
    }
  }

  /*printf("Socket stored : %ld\n", psock->storeddatasize);*/

  /* if read data is < to maxdatasizetoread, we put everything in dataout */
  if ( psock->storeddatasize<=maxdatasizetoread )
  { if (pdataoutsize!=NULL) *pdataoutsize=psock->storeddatasize;
    if (pdataout==NULL)
    { lcrzo_er(lcrzo_data_free(psock->storeddata));
    }
    else *pdataout=psock->storeddata;
    /* now there is nothing left which is stored */
    psock->storeddatasize=0;
  }
  /* else, we only use an extract */
  else
  { lcrzo_er(lcrzo_data_initm_range(psock->storeddata, psock->storeddatasize,
				    +1, 1+maxdatasizetoread,
				    pdataout, pdataoutsize));
    /* we left the end of datastored */
    leftsize=psock->storeddatasize-maxdatasizetoread;
    lcrzo_er(lcrzo_data_alloc(leftsize, &p));
    memcpy(p, psock->storeddata+maxdatasizetoread, leftsize);
    lcrzo_er(lcrzo_data_free(psock->storeddata));
    psock->storeddata=p;
    psock->storeddatasize=leftsize;
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_myipcvsum(lcrzo_sock *psock,
			      lcrzo_bool beblocking,
			      lcrzo_int32 maxdatasizetoread,
			      lcrzo_data *pdataout,
			      lcrzo_int32 *pdataoutsize)
{ lcrzo_uint16 type;
  lcrzo_int32 leftsize;
  lcrzo_data p;

  /* parameters verification */
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  if (maxdatasizetoread<0) return(LCRZO_ERR_PATOOLOW);

  /* special case */
  if (maxdatasizetoread==0)
  { lcrzo_er(lcrzo_data_alloc(0, pdataout));
    if (pdataoutsize!=NULL) *pdataoutsize=0;
    return(LCRZO_ERR_OK);
  }

  /* if storeddata is empty, we read the next tlv */
  if ( psock->cisniff.storeddatasize==0 )
  { lcrzo_er(lcrzo_priv_ipc_readtlv(&(psock->cisniff), beblocking, &type,
				    &(psock->cisniff.storeddata),  
				    &(psock->cisniff.storeddatasize)));
    /* if we were not blocking, we never reach this point */
    /* we check the received tlv */
    if (type!=LCRZO_PRIV_IPCTYPE_MSG)
    { return(LCRZO_ERR_BUIPCUNWANTEDTYPE);
    }
    /* set the current client informations */
    lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), LCRZO_TRUE,
					psock->there_etha,
					&(psock->there_ipl),
					&(psock->there_port)));
  }

  /*printf("IPC stored : %ld\n", pci->storeddatasize);*/

  /* if read data is < to maxdatasizetoread, we put everything in dataout */
  if ( psock->cisniff.storeddatasize<=maxdatasizetoread )
  { if (pdataoutsize!=NULL) *pdataoutsize=psock->cisniff.storeddatasize;
    if (pdataout==NULL)
    { lcrzo_er(lcrzo_data_free(psock->cisniff.storeddata));
    }
    else *pdataout=psock->cisniff.storeddata;
    /* now there is nothing left which is stored */
    psock->cisniff.storeddatasize=0;
  }
  /* else, we only use an extract */
  else
  { lcrzo_er(lcrzo_data_initm_range(psock->cisniff.storeddata,
				    psock->cisniff.storeddatasize,
				    +1, 1+maxdatasizetoread,
				    pdataout, pdataoutsize));
    /* we left the end of datastored */
    leftsize=psock->cisniff.storeddatasize-maxdatasizetoread;
    lcrzo_er(lcrzo_data_alloc(leftsize, &p));
    memcpy(p, psock->cisniff.storeddata+maxdatasizetoread, leftsize);
    lcrzo_er(lcrzo_data_free(psock->cisniff.storeddata));
    psock->cisniff.storeddata=p;
    psock->cisniff.storeddatasize=leftsize;
  }

  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_priv_multi_read(int type,
			  void *psrc,
			  lcrzo_bool beblocking,
			  lcrzo_int32 dataoutmaxsize,
			  lcrzo_data dataout,
			  lcrzo_int32 *pdataoutsize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int *pfd;
  FILE *pstream;
  lcrzo_ipc *pipc;
  lcrzo_sock *psock;
  int ret;

  /* parameter verification */  
  if (psrc==NULL) return (LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);
  if (dataoutmaxsize<0) return(LCRZO_ERR_PATOOLOW);

  /* special case */
  if (dataoutmaxsize==0)
  { if (pdataoutsize!=NULL) *pdataoutsize=0;
    return(LCRZO_ERR_OK);
  }

  /* read */
  switch(type)
  { case LCRZO_PRIV_MULTI_READ_FD:
      pfd=(int *)psrc;
      lcrzo_er(lcrzo_fd_block_set(*pfd, beblocking));
      ret=lcrzo_priv_fd_myread(*pfd, dataoutmaxsize, &data, &datasize);
      break;
    case LCRZO_PRIV_MULTI_READ_STREAM:
      pstream=(FILE *)psrc;
      lcrzo_er(lcrzo_fd_block_set(fileno(pstream), beblocking));
      ret=lcrzo_priv_stream_myread(pstream, dataoutmaxsize, &data, &datasize);
      break;
    case LCRZO_PRIV_MULTI_READ_IPC:
      pipc=(lcrzo_ipc *)psrc;
      ret=lcrzo_priv_ipc_myread(pipc, beblocking,
				dataoutmaxsize, &data, &datasize);
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKR:
      psock=(lcrzo_sock *)psrc;
      lcrzo_er(lcrzo_fd_block_set(psock->socknum, beblocking));
      ret=lcrzo_priv_sock_myrecvfrom(psock, dataoutmaxsize, &data, &datasize);
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKVSUM:
      psock=(lcrzo_sock *)psrc;
      ret=lcrzo_priv_sock_myipcvsum(psock, beblocking,
				    dataoutmaxsize, &data, &datasize);
      break;
    default:
      return(lcrzo_ieuv("lcrzo_priv_multi_read", "type"));
  }

  /* set output */
  if (ret==LCRZO_ERR_OK)
  { if (dataout!=NULL) memcpy(dataout, data, datasize);
    lcrzo_data_free(data);
    if (pdataoutsize!=NULL) *pdataoutsize=datasize;
  }

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_multi_readm(int type,
			   void *psrc,
			   lcrzo_bool beblocking,
			   lcrzo_data *pdataout,
			   lcrzo_int32 *pdataoutsize)
{ int *pfd;
  FILE *pstream;
  lcrzo_ipc *pipc;
  lcrzo_sock *psock;
 
  /* parameter verification */  
  if (psrc==NULL) return (LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);

  /* read */
  switch(type)
  { case LCRZO_PRIV_MULTI_READ_FD:
      pfd=(int *)psrc;
      lcrzo_er(lcrzo_fd_block_set(*pfd, beblocking));
      lcrzo_er(lcrzo_priv_fd_myread(*pfd, LCRZO_PRIV_READMAXMSGSIZE,
				    pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_MULTI_READ_STREAM:
      pstream=(FILE *)psrc;
      lcrzo_er(lcrzo_fd_block_set(fileno(pstream), beblocking));
      lcrzo_er(lcrzo_priv_stream_myread(pstream, LCRZO_PRIV_READMAXMSGSIZE,
					pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_MULTI_READ_IPC:
      pipc=(lcrzo_ipc *)psrc;
      lcrzo_er(lcrzo_priv_ipc_myread(pipc, beblocking, 0x7FFFFFFF,
				     pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKR:
      psock=(lcrzo_sock *)psrc;
      lcrzo_er(lcrzo_fd_block_set(psock->socknum, beblocking));
      lcrzo_er(lcrzo_priv_sock_myrecvfrom(psock, LCRZO_PRIV_READMAXMSGSIZE,
					  pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKVSUM:
      psock=(lcrzo_sock *)psrc;
      lcrzo_er(lcrzo_priv_sock_myipcvsum(psock, beblocking, 0x7FFFFFFF,
					 pdataout, pdataoutsize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_priv_multi_readm", "type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_multi_read_fixed(int type,
				void *psrc,
				lcrzo_bool beblocking,
				lcrzo_int32 datasizetoread,
				lcrzo_data dataout)
{ lcrzo_data data;

  /* get data */
  lcrzo_er(lcrzo_priv_multi_readm_fixed(type, psrc, beblocking, 
					datasizetoread, &data));
      
  /*copy it*/
  memcpy(dataout, data, datasizetoread);

  /*free*/
  lcrzo_er(lcrzo_data_free(data));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_multi_readm_fixed(int type,
				 void *psrc,
				 lcrzo_bool beblocking,
				 lcrzo_int32 datasizetoread,
				 lcrzo_data *pdataout)
{ lcrzo_data ptr, tmpdata;
  lcrzo_int32 tmpdatasize, totalread, tmpdatasizetoread;
  int *pfd=NULL;
  FILE *pstream=NULL;
  lcrzo_ipc *pipc=NULL;
  lcrzo_sock *psock=NULL;
  int ret;

  /* parameter verification */  
  if (psrc==NULL) return (LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);
  if (datasizetoread<0) return(LCRZO_ERR_PATOOLOW);

  /* special case */
  if (datasizetoread==0)
  { lcrzo_er(lcrzo_data_alloc(0, pdataout));
    return(LCRZO_ERR_OK);
  }

  /* eventually set blocking */
  switch(type)
  { case LCRZO_PRIV_MULTI_READ_FD:
      pfd=(int *)psrc;
      lcrzo_er(lcrzo_fd_block_set(*pfd, beblocking));
      break;
    case LCRZO_PRIV_MULTI_READ_STREAM:
      pstream=(FILE *)psrc;
      lcrzo_er(lcrzo_fd_block_set(fileno(pstream), beblocking));
      break;
    case LCRZO_PRIV_MULTI_READ_IPC:
      pipc=(lcrzo_ipc *)psrc;
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKR:
      psock=(lcrzo_sock *)psrc;
      lcrzo_er(lcrzo_fd_block_set(psock->socknum, beblocking));
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKVSUM:
      psock=(lcrzo_sock *)psrc;
      break;
    default:
      return(lcrzo_ieuv("lcrzo_priv_multi_readm_fixed", "type"));
  }

  /* read data */
  lcrzo_er(lcrzo_data_alloc(datasizetoread, &ptr));
  totalread=0;  
  while(1)
  { tmpdatasizetoread=datasizetoread-totalread;
    if (tmpdatasizetoread>500) tmpdatasizetoread=500;
    switch(type)
    { case LCRZO_PRIV_MULTI_READ_FD:
        ret=lcrzo_priv_fd_myread(*pfd, tmpdatasizetoread,
				 &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_STREAM:
        ret=lcrzo_priv_stream_myread(pstream, tmpdatasizetoread,
				     &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_IPC:
        ret=lcrzo_priv_ipc_myread(pipc, beblocking, tmpdatasizetoread,
				  &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_SOCKR:
        ret=lcrzo_priv_sock_myrecvfrom(psock, tmpdatasizetoread,
				       &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_SOCKVSUM:
        ret=lcrzo_priv_sock_myipcvsum(psock, beblocking, tmpdatasizetoread,
				      &tmpdata, &tmpdatasize);
        break;
      default:
	return(lcrzo_ieuv("lcrzo_priv_multi_readm_fixed", "type"));
    }
    if (ret==LCRZO_ERR_OK)
    { /*concatene*/
      memcpy(ptr+totalread, tmpdata, tmpdatasize);
      lcrzo_data_free(tmpdata);
      totalread+=tmpdatasize;
      /*eventually quit*/
      if ( totalread==datasizetoread )
      { if (pdataout!=NULL) *pdataout=ptr;
        else lcrzo_data_free(ptr);
        return(LCRZO_ERR_OK);
      }
    }
    else if (ret==LCRZO_ERR_OKDATAEND)
    { /*no more data, so it's an error*/
      lcrzo_data_free(ptr);
      return(ret);
    }
    else if (ret==LCRZO_ERR_OKTEMPDATAEND)
    { if (!totalread)
      { /*we didn't read anathing, so we can quit now*/
	lcrzo_data_free(ptr);
        return(ret);
      }
    }
    else
    { lcrzo_data_free(ptr);
      return(ret);
    }
  }

  /*on ne doit jamais atteindre ce return*/
  return(LCRZO_ERR_IEINTERNALERROR);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_multi_read_line(int type,
			       void *psrc,
			       lcrzo_bool beblocking,
			       lcrzo_int32 dataoutmaxsize,
			       lcrzo_data dataout,
			       lcrzo_int32 *pdataoutsize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* special case */
  if (dataoutmaxsize==0)
  { if (pdataoutsize!=NULL) *pdataoutsize=0;
    return(LCRZO_ERR_OK);
  }

  /* get data */
  lcrzo_er(lcrzo_priv_multi_readm_line(type, psrc, beblocking, 
				       &data, &datasize));
      
  /*copy it*/
  ret=lcrzo_data_init_data(data, datasize,
			   dataoutmaxsize, dataout, pdataoutsize);
  lcrzo_data_free(data);
  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_multi_readm_line(int type,
				void *psrc,
				lcrzo_bool beblocking,
				lcrzo_data *pdataout,
				lcrzo_int32 *pdataoutsize)
{ lcrzo_data data, tmpdata;
  lcrzo_uint8 block[500];
  lcrzo_int32 datasize, tmpdatasize, blocksize;
  int *pfd=NULL;
  FILE *pstream=NULL;
  lcrzo_ipc *pipc=NULL;
  lcrzo_sock *psock=NULL;
  int ret, c;
  
  /* parameter verification */  
  if (psrc==NULL) return (LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);

  /* eventually set blocking */
  switch(type)
  { case LCRZO_PRIV_MULTI_READ_FD:
      pfd=(int *)psrc;
      lcrzo_er(lcrzo_fd_block_set(*pfd, beblocking));
      break;
    case LCRZO_PRIV_MULTI_READ_STREAM:
      pstream=(FILE *)psrc;
      lcrzo_er(lcrzo_fd_block_set(fileno(pstream), beblocking));
      break;
    case LCRZO_PRIV_MULTI_READ_IPC:
      pipc=(lcrzo_ipc *)psrc;
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKR:
      psock=(lcrzo_sock *)psrc;
      lcrzo_er(lcrzo_fd_block_set(psock->socknum, beblocking));
      break;
    case LCRZO_PRIV_MULTI_READ_SOCKVSUM:
      psock=(lcrzo_sock *)psrc;
      break;
    default:
      return(lcrzo_ieuv("lcrzo_priv_multi_readm_line", "type"));
  }

  /* get data */
  datasize=0;
  lcrzo_er(lcrzo_data_alloc(datasize, &data));
  blocksize=0;
  while (1)
  { 
    switch(type)
    { case LCRZO_PRIV_MULTI_READ_FD:
        ret=lcrzo_priv_fd_myread(*pfd, 1, &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_STREAM:
        ret=lcrzo_priv_stream_myread(pstream, 1, &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_IPC:
        ret=lcrzo_priv_ipc_myread(pipc, beblocking, 1, &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_SOCKR:
        ret=lcrzo_priv_sock_myrecvfrom(psock, 1, &tmpdata, &tmpdatasize);
        break;
      case LCRZO_PRIV_MULTI_READ_SOCKVSUM:
        ret=lcrzo_priv_sock_myipcvsum(psock, beblocking, 1, 
				      &tmpdata, &tmpdatasize);
        break;
      default:
	return(lcrzo_ieuv("lcrzo_priv_multi_readm_line", "type"));
    }
    if (ret==LCRZO_ERR_OK)
    { if (tmpdatasize)
      { c=tmpdata[0];
        lcrzo_er(lcrzo_data_free(tmpdata));
        /*ignore 0D*/
        if ( c == 0x0D ) {}
        else if ( c == 0x0A ) { break; }
        else
        { block[blocksize]=c;
          blocksize++;
          if (blocksize>499)
          { lcrzo_er(lcrzo_data_appendm_data(block, blocksize, 
					     datasize, &data, &datasize));
  	    blocksize=0;
	  }
        }
      }
      else
      { /*we read nothing, continue*/
	lcrzo_er(lcrzo_data_free(tmpdata));
      }
    }
    else if (ret==LCRZO_ERR_OKTEMPDATAEND)
    { /*exit if we read nothing*/
      if (!datasize)
      { lcrzo_data_free(data);
        return(ret);
      }
    }
    else if (ret==LCRZO_ERR_OKDATAEND)
    { /*exit if we read nothing*/
      if (!datasize)
      { lcrzo_data_free(data);
        return(ret);
      }
      /* end of file is like end of line */
      break;
    }
    else
    { lcrzo_data_free(data);
      return(ret);
    }
  }
  /*append the end*/
  lcrzo_er(lcrzo_data_appendm_data(block, blocksize, 
				   datasize, &data, &datasize));

  /*on a lu quelque chose*/
  if (pdataout!=NULL)
  { *pdataout=data; 
  }
  else
  { lcrzo_data_free(data);
  }
  if (pdataoutsize!=NULL) *pdataoutsize=datasize;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_killwait(int pid)
{ int ret, status;

  ret=kill(pid, SIGTERM);
  if (ret==-1)
  { /* under FreeBSD, killing a Zombie is an error */
    if (errno!=ESRCH) return(LCRZO_ERR_FUKILL);
    /* ignore error */
    errno=0;
  }
  ret=waitpid(pid, &status, 0);
  if (ret==-1) return(LCRZO_ERR_FUWAITPID);
  return(LCRZO_ERR_OK);
}






