/*us    Laurent Constantin's network library (lcrzo)
                 lcrzo_priv module

  Functions herein are private.
*/
/*fr    Bibliotheque reseau de Laurent Constantin (lcrzo)
                 Module lcrzo_priv

  Les fonctions presentes dans ce module sont privees a la bibliotheque.
*/


#include "lcrzo.h"

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_PRINTTYPE_COUNT    35
#define LCRZO_PRIV_PRINTPROFILE_COUNT  8

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_WRITEMAXMSGSIZE      500
#define LCRZO_PRIV_READMAXMSGSIZE   0x1FFFF

/*-------------------------------------------------------------*/
#define LCRZO_PRIV_IPCTYPE_MSG      1 
#define LCRZO_PRIV_IPCTYPE_UINT8   10
#define LCRZO_PRIV_IPCTYPE_UINT16  11
#define LCRZO_PRIV_IPCTYPE_UINT32  12
#define LCRZO_PRIV_IPCTYPE_HS      20
#define LCRZO_PRIV_IPCTYPE_IPS     21
#define LCRZO_PRIV_IPCTYPE_IPA     22
#define LCRZO_PRIV_IPCTYPE_IPL     23
#define LCRZO_PRIV_IPCTYPE_ETHS    24
#define LCRZO_PRIV_IPCTYPE_ETHA    25
#define LCRZO_PRIV_IPCTYPE_DEVICE  26

/*-------------------------------------------------------------*/
#define LCTSUDP   0x0001
#define LCTSTCP   0x0002
#define LCTSCLI   0x0004
#define LCTSSER   0x0008
#define LCTSREAL  0x0010
#define LCTSVIRT  0x0020
#define LCTSMONO  0x0040
#define LCTSMULT  0x0080

#define LCRZO_PRIV_TYPESOCK_REALCLIUDP      (LCTSUDP|LCTSCLI|LCTSREAL|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_REALCLITCP      (LCTSTCP|LCTSCLI|LCTSREAL|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_VIRTCLIUDP      (LCTSUDP|LCTSCLI|LCTSVIRT|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_VIRTCLITCP      (LCTSTCP|LCTSCLI|LCTSVIRT|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_REALSERUDP      (LCTSUDP|LCTSSER|LCTSREAL|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_REALSERTCP      (LCTSTCP|LCTSSER|LCTSREAL|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_VIRTSERUDP      (LCTSUDP|LCTSSER|LCTSVIRT|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_VIRTSERTCP      (LCTSTCP|LCTSSER|LCTSVIRT|LCTSMONO)
#define LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI (LCTSUDP|LCTSSER|LCTSREAL|LCTSMULT)
#define LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI (LCTSUDP|LCTSSER|LCTSVIRT|LCTSMULT)
#define LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI (LCTSTCP|LCTSSER|LCTSREAL|LCTSMULT)

/*---------------------------------------------------------------*/
int lcrzo_priv_etha_and(lcrzo_etha a, lcrzo_etha b, lcrzo_etha result);
int lcrzo_priv_etha_or(lcrzo_etha a, lcrzo_etha b, lcrzo_etha result);
int lcrzo_priv_etha_xor(lcrzo_etha a, lcrzo_etha b, lcrzo_etha result);
int lcrzo_priv_etha_not(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_inc(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_dec(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_shl(lcrzo_etha a, lcrzo_etha result);
int lcrzo_priv_etha_shr(lcrzo_etha a, lcrzo_etha result);

/*-------------------------------------------------------------*/
int lcrzo_priv_hdrleth_equal_addr(lcrzo_hdrleth hdrleth,
				  const lcrzo_etha ethasrc,
				  const lcrzo_etha ethadst);
int lcrzo_priv_hdrlip_equal_addr(lcrzo_hdrlip hdrlip,
				 const lcrzo_ipl iplsrc,
				 const lcrzo_ipl ipldst);
int lcrzo_priv_hdrludp_equal_port(lcrzo_hdrludp hdrludp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst);
int lcrzo_priv_hdrltcp_equal_port(lcrzo_hdrltcp hdrltcp,
				  lcrzo_uint16 portsrc,
				  lcrzo_uint16 portdst);

/*-------------------------------------------------------------*/
int lcrzo_priv_stdin(lcrzo_string *puserstring,
		     lcrzo_int32 *puserstringsize);
int lcrzo_priv_passwd(lcrzo_string *puserstring,
		      lcrzo_int32 *puserstringsize);

/*---------------------------------------------------------------*/
int lcrzo_ie(lcrzo_conststring functionname,
	     lcrzo_conststring frexplain,
	     lcrzo_conststring usexplain);
int lcrzo_ieuv(lcrzo_conststring functionname,
	       lcrzo_conststring varname);
int lcrzo_ierp(lcrzo_conststring functionname);

/*---------------------------------------------------------------*/
#define LCRZO_PRIV_INCREASE 1
#define LCRZO_PRIV_DECREASE 2

/*-------------------------------------------------------------*/
int lcrzo_priv_manageelempos(lcrzo_int32 elemcount,
			     lcrzo_int32 position,
			     lcrzo_bool wanttoinsertelem,
			     lcrzo_int32 *pposnegative,
			     lcrzo_int32 *ppospositive,
			     int *pdirection, 
			     lcrzo_int32 *pdistance);
int lcrzo_priv_manageelemrange(lcrzo_int32 elemcount,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance);
int lcrzo_priv_manageelemrangefast(lcrzo_int32 elemcount,
				   lcrzo_int32 infposition,
				   lcrzo_int32 supposition,
				   lcrzo_int32 *pinfposnegative,
				   lcrzo_int32 *pinfpospositive,
				   lcrzo_int32 *psupposnegative,
				   lcrzo_int32 *psuppospositive,
				   int *pinfdirection, 
				   lcrzo_int32 *pinfdistance,
				   int *pinfsupdirection, 
				   lcrzo_int32 *pinfsupdistance);

/*-------------------------------------------------------------*/
int lcrzo_priv_managedataposneg(lcrzo_int32 datasize,
				lcrzo_int32 negative,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive);
int lcrzo_priv_managedatapospos(lcrzo_int32 datasize,
				lcrzo_int32 positive,
				lcrzo_int32 *pnegative,
				lcrzo_int32 *ppositive);
int lcrzo_priv_managedatarange(lcrzo_int32 datasize,
			       lcrzo_int32 infposition,
			       lcrzo_int32 supposition,
			       lcrzo_int32 *pinfposnegative,
			       lcrzo_int32 *pinfpospositive,
			       lcrzo_int32 *psupposnegative,
			       lcrzo_int32 *psuppospositive,
			       int *pinfdirection, 
			       lcrzo_int32 *pinfdistance,
			       int *pinfsupdirection, 
			       lcrzo_int32 *pinfsupdistance);


/*-------------------------------------------------------------*/
int lcrzo_priv_data_u16_init_data(lcrzo_constdata datain,
				  lcrzo_uint16 datainsize,
				  lcrzo_uint16 dataoutmaxsize,
				  lcrzo_data dataout,
				  lcrzo_uint16 *pdataoutsize);
int lcrzo_priv_data_u16_initm_data(lcrzo_constdata datain,
				   lcrzo_uint16 datainsize,
				   lcrzo_data *pdataout,
				   lcrzo_uint16 *pdataoutsize);
int lcrzo_priv_data_u16_append_data(lcrzo_constdata datain,
				    lcrzo_uint16 datainsize,
				    lcrzo_uint16 dataoutcurrentsize,
				    lcrzo_uint16 dataoutmaxsize,
				    lcrzo_data dataout,
				    lcrzo_uint16 *pdataoutsize);
int lcrzo_priv_data_u16_appendm_data(lcrzo_constdata datain,
				     lcrzo_uint16 datainsize,
				     lcrzo_uint16 dataoutcurrentsize,
				     lcrzo_data *pdataout,
				     lcrzo_uint16 *pdataoutsize);


/*-------------------------------------------------------------*/
int lcrzo_priv_fd_myread(int fd,
			 lcrzo_int32 maxdatasizetoread,
			 lcrzo_data *pdataout,
			 lcrzo_int32 *pdataoutsize);
int lcrzo_priv_stream_myread(LCRZOFILE *pstream,
			     lcrzo_int32 maxdatasizetoread,
			     lcrzo_data *pdataout,
			     lcrzo_int32 *pdataoutsize);
int lcrzo_priv_ipc_myread(lcrzo_ipc *pci,
			  lcrzo_bool beblocking,
			  lcrzo_int32 maxdatasizetoread,
			  lcrzo_data *pdataout,
			  lcrzo_int32 *pdataoutsize);
int lcrzo_priv_sock_myrecvfrom(lcrzo_sock *psock,
			       lcrzo_int32 maxdatasizetoread,
			       lcrzo_data *pdataout,
			       lcrzo_int32 *pdataoutsize);
int lcrzo_priv_sock_myipcvsum(lcrzo_sock *psock,
			      lcrzo_bool beblocking,
			      lcrzo_int32 maxdatasizetoread,
			      lcrzo_data *pdataout,
			      lcrzo_int32 *pdataoutsize);

#define LCRZO_PRIV_MULTI_READ_FD       1
#define LCRZO_PRIV_MULTI_READ_STREAM   2
#define LCRZO_PRIV_MULTI_READ_IPC      3
#define LCRZO_PRIV_MULTI_READ_SOCKR    4
#define LCRZO_PRIV_MULTI_READ_SOCKVSUM 5

int lcrzo_priv_multi_read(int type,
			  void *psrc,
			  lcrzo_bool beblocking,
			  lcrzo_int32 dataoutmaxsize,
			  lcrzo_data dataout,
			  lcrzo_int32 *pdataoutsize);
int lcrzo_priv_multi_readm(int type,
			   void *psrc,
			   lcrzo_bool beblocking,
			   lcrzo_data *pdataout,
			   lcrzo_int32 *pdataoutsize);
int lcrzo_priv_multi_read_fixed(int type,
				void *psrc,
				lcrzo_bool beblocking,
				lcrzo_int32 datasizetoread,
				lcrzo_data dataout);
int lcrzo_priv_multi_readm_fixed(int type,
				 void *psrc,
				 lcrzo_bool beblocking,
				 lcrzo_int32 datasizetoread,
				 lcrzo_data *pdataout);
int lcrzo_priv_multi_read_line(int type,
			       void *psrc,
			       lcrzo_bool beblocking,
			       lcrzo_int32 dataoutmaxsize,
			       lcrzo_data dataout,
			       lcrzo_int32 *pdataoutsize);
int lcrzo_priv_multi_readm_line(int type,
				void *psrc,
				lcrzo_bool beblocking,
				lcrzo_data *pdataout,
				lcrzo_int32 *pdataoutsize);

/*-------------------------------------------------------------*/
int lcrzo_priv_killwait(int pid);

/*-------------------------------------------------------------*/
/*us system dependant functions */
/*fr fonctions dependant du systeme */
#include "lcrzo_sysdep.h"







