/*
		                  LCRZO
                             Network library
             Copyright(c) 1999,2000,2001, Laurent Constantin
                                  -----

  Main server    : http://www.laurentconstantin.com/
  Backup servers : http://go.to/laurentconstantin/
                   http://laurentconstantin.est-la.com/
  [my current email address is on the web servers]

                                  -----
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details (http://www.gnu.org).

------------------------------------------------------------------------
*/

/* -- Note about comments and variable names --
   In versions 1.00 up to 2.08, lcrzo was only written in French (my
   mother tongue). Since version 3.00, lcrzo is written in English, but
   most of comments and variable names are still in French.
   In future versions, I'll try to translate everything into English,
   but as you may guess, that's a quite big work...
   So, for the moment, here are some word translation, if you want to
   be able to understand what's going on in the functions :
     afficher - print     ;  ajouter - add         ;  copie - copy
     dernier - last       ;  donnee - data         ;  ecrire - write
     entete - header      ;  envoye - sent         ;  ferme - close
     fichier - file       ;  lire - read           ;  masque - mask 
     nboct - byte number  ;  ouvre, ouvrir - open  ;  paq, paquet - packet
     plage - range        ;  premier - first       ;  recu - received
     reseau - network     ;  retour - return       ;  taille - size
     tunnel - pipe

*/

#include "lcrzo_priv.h"
#ifdef LCRZODEF_SYSTEM_Linux
 #define _GNU_SOURCE
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
#elif defined LCRZODEF_SYSTEM_FreeBSD
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
#elif defined LCRZODEF_SYSTEM_Solaris
 #define __EXTENSIONS__
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
#else
 #error "Traiter le cas de LCRZODEF_SYSTEM"
#endif

/*   usage of control channels :
udp client :
  cicontrole1 :  error code of one of the child (we terminate when
                 one of the childs terminates)
  cicontrole2 :  not used

udp server :
  cicontrole1 :  error code of one of the child (we terminate when
                 one of the childs terminates)
  cicontrole2 :  who is the client (one is used by spoof process, and
                 the other by the first read)

udp multiclient server :
  cicontrole1 :  error code of the child
  cicontrole2 :  who is the client (one for each received packet)

tcp client : 
  cicontrole1 :  end of handshake
                 error code of child
  cicontrole2 :  father want to close connexion

tcp server :
  cicontrole1 :  end of handshake, if OK, who is the client
		 error code of child
  cicontrole2 :  father want to close connexion
 */

/***************************************************************/
/* Private functions */

/*-------------------------------------------------------------*/
/*-- error code functions --*/
/*-------------------------------------------------------------*/
int lcrzo_priv_sock_eechild(lcrzo_sock *psock, int ret);
int lcrzo_priv_sock_eechild(lcrzo_sock *psock, int ret)
{ 
  /*printf("pid %d : %d\n", getpid(), ret);*/
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_ipc_write_uint32(&(psock->cicontrole1), ret);
    _exit(ret);
    /* we should never reach this point */
    return(lcrzo_ierp("lcrzo_priv_sock_eechild"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_okchild(lcrzo_sock *psock);
int lcrzo_priv_sock_okchild(lcrzo_sock *psock)
{ lcrzo_er(lcrzo_ipc_write_uint32(&(psock->cicontrole1), LCRZO_ERR_OK));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_analyzeexitchild(lcrzo_sock *psock,
				     lcrzo_bool beblocking);
int lcrzo_priv_sock_analyzeexitchild(lcrzo_sock *psock,
				     lcrzo_bool beblocking)
{ lcrzo_uint32 childret;
  int ret;

  ret=lcrzo_ipc_read_uint32(&(psock->cicontrole1), beblocking, &childret);
  /*printf("ret %d %lu", ret, childret);*/
  if ( ret==LCRZO_ERR_OK )
  { /* the child sent something */
    psock->connectionclosed=LCRZO_TRUE;
    return(childret);
  }
  else if ( ret==LCRZO_ERR_OKTEMPDATAEND && !beblocking )
  { return(LCRZO_ERR_OK);
  }
  else if ( ret==LCRZO_ERR_OKDATAEND )
  { psock->connectionclosed=LCRZO_TRUE;
    return(LCRZO_ERR_OK);
  }
  else
  { return(ret);
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-- init and close functions --*/
/*-------------------------------------------------------------*/
int lcrzo_priv_sock_ipc_init(lcrzo_sock *psock);
int lcrzo_priv_sock_ipc_init(lcrzo_sock *psock)
{ int retour;
  
  lcrzo_er(lcrzo_ipc_init(&(psock->cisniff)));
  retour=lcrzo_ipc_init(&(psock->cispoof));
  if ( retour!=LCRZO_ERR_OK )
  { lcrzo_er(lcrzo_ipc_close(&(psock->cisniff)));
    return(retour);
  }
  retour=lcrzo_ipc_init(&(psock->cioptions));
  if ( retour!=LCRZO_ERR_OK )
  { lcrzo_er(lcrzo_ipc_close(&(psock->cispoof)));
    lcrzo_er(lcrzo_ipc_close(&(psock->cisniff)));
    return(retour);
  }
  retour=lcrzo_ipc_init(&(psock->cicontrole1));
  if ( retour!=LCRZO_ERR_OK )
  { lcrzo_er(lcrzo_ipc_close(&(psock->cioptions)));
    lcrzo_er(lcrzo_ipc_close(&(psock->cispoof)));
    lcrzo_er(lcrzo_ipc_close(&(psock->cisniff)));
    return(retour);
  }
  retour=lcrzo_ipc_init(&(psock->cicontrole2));
  if ( retour!=LCRZO_ERR_OK )
  { lcrzo_er(lcrzo_ipc_close(&(psock->cicontrole1)));
    lcrzo_er(lcrzo_ipc_close(&(psock->cioptions)));
    lcrzo_er(lcrzo_ipc_close(&(psock->cispoof)));
    lcrzo_er(lcrzo_ipc_close(&(psock->cisniff)));
    return(retour);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_ipc_close(lcrzo_sock *psock);
int lcrzo_priv_sock_ipc_close(lcrzo_sock *psock)
{ 
  lcrzo_ipc_close(&(psock->cioptions));
  lcrzo_ipc_close(&(psock->cicontrole2));
  lcrzo_ipc_close(&(psock->cicontrole1));
  lcrzo_ipc_close(&(psock->cisniff));
  lcrzo_ipc_close(&(psock->cispoof));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sniff_init_dev(const lcrzo_device device,
			      lcrzo_sniff *psniff);
int lcrzo_priv_sniff_init_dev(const lcrzo_device device,
			      lcrzo_sniff *psniff)
{ int retour;
  lcrzo_uint32 mtu;

  retour=lcrzo_mtu_init_device(device, &mtu);
  if ( retour!=LCRZO_ERR_OK )
  { mtu=1500;
  }
  lcrzo_er(lcrzo_sniff_init(device, mtu, "", psniff));
    
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-- generic functions --*/
/*-------------------------------------------------------------*/
int lcrzo_priv_sock_fork(lcrzo_sock *psock, int *ppid);
int lcrzo_priv_sock_fork(lcrzo_sock *psock, int *ppid)
{ 
  *ppid=fork();
  if (*ppid<0)
  { lcrzo_priv_sock_ipc_close(psock);
    return(LCRZO_ERR_FUFORK);
  }
  /*Attention : on ne doit jamais quitter avec "return()" (ou lcrzo_er),
    mais avec "_exit()", sinon le processus retourne dans la fonction
    appelante du pere*/

  /*on initialise le generateur aleatoire du fils*/
  if (*ppid==0)
  { lcrzo_priv_sock_eechild(psock, lcrzo_rand_init(0));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_majinfos(const lcrzo_device here_device,
			     const lcrzo_etha here_etha,
			     lcrzo_ipl here_ipl,
			     lcrzo_uint16 here_port,
			     const lcrzo_etha there_etha,
			     lcrzo_ipl there_ipl,
			     lcrzo_uint16 there_port,
			     lcrzo_sock *psock);
int lcrzo_priv_sock_majinfos(const lcrzo_device here_device,
			     const lcrzo_etha here_etha,
			     lcrzo_ipl here_ipl,
			     lcrzo_uint16 here_port,
			     const lcrzo_etha there_etha,
			     lcrzo_ipl there_ipl,
			     lcrzo_uint16 there_port,
			     lcrzo_sock *psock)
{
  lcrzo_er(lcrzo_device_init(here_device, psock->here_device));
  memcpy(psock->here_etha, here_etha, LCRZO_ETHA_MAXBYTES);
  psock->here_ipl=here_ipl;
  psock->here_port=here_port;
  if (there_etha!=NULL)
    memcpy(psock->there_etha, there_etha, LCRZO_ETHA_MAXBYTES);
  psock->there_ipl=there_ipl;
  psock->there_port=there_port;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
/*-- udp functions --*/
/*-------------------------------------------------------------*/
int lcrzo_priv_sock_spiou(lcrzo_sock *psock,
			  lcrzo_data udpdata,  
			  lcrzo_uint16 udpdatasize);
int lcrzo_priv_sock_spiou(lcrzo_sock *psock,
			  lcrzo_data udpdata,  
			  lcrzo_uint16 udpdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip  hdrlip;
  lcrzo_hdrludp hdrludp;
  int ret;
  lcrzo_data ipopt;
  lcrzo_int32 ipoptsize;
  lcrzo_spoof spoof;

  /*initialisation de l'entete ETH*/
  lcrzo_er(lcrzo_hdrleth_initdefault(&hdrleth));
  memcpy(hdrleth.src, psock->here_etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrleth.dst, psock->there_etha, LCRZO_ETHA_MAXBYTES);
  /*initialisation de l'entete IP*/
  lcrzo_er(lcrzo_hdrlip_initdefault(&hdrlip));
  hdrlip.saddr=psock->here_ipl;
  hdrlip.daddr=psock->there_ipl;
  /*initialisation de l'entete UDP*/
  lcrzo_er(lcrzo_hdrludp_initdefault(&hdrludp));
  hdrludp.sport=psock->here_port;
  hdrludp.dport=psock->there_port;

  /*mise a jour eventuelle des options IP*/
  ret=lcrzo_ipc_readm(&(psock->cioptions), LCRZO_FALSE, &ipopt, &ipoptsize);
  if ( ret==LCRZO_ERR_OK )
  { if (ipoptsize>LCRZO_IPOPT_MAXBYTES)
      return(lcrzo_ie("lcrzo_priv_sock_spiou",
		      "ipoptsize lu depuis l'IPC est > LCRZO_IPOPT_MAXBYTES",
		      "ipoptsize read from IPC is > LCRZO_IPOPT_MAXBYTES"));
    memcpy(psock->ipopt, ipopt, ipoptsize);
    psock->ipoptsize=(lcrzo_uint8)(ipoptsize);
    lcrzo_data_free(ipopt);
  }

  /*envoi d'un packet avec donnees*/
  lcrzo_er(lcrzo_spoof_init(&spoof));
  lcrzo_er(lcrzo_spoof_ethipoptudpdata(&spoof, psock->here_device,
				       hdrleth, hdrlip,
				       psock->ipopt, psock->ipoptsize,
				       hdrludp, udpdata, udpdatasize));
  lcrzo_er(lcrzo_spoof_close(&spoof));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_udp_sniffprocess(lcrzo_sock *psock);
int lcrzo_priv_udp_sniffprocess(lcrzo_sock *psock)
{ lcrzo_sniff sniff;
  lcrzo_data sniffdata, data;
  lcrzo_int32 sniffdatasize;
  lcrzo_uint16 datasize;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_bool givepackettouser;
  int ret;

  /* initialize sniff */
  lcrzo_er(lcrzo_priv_sniff_init_dev(psock->here_device, &sniff));

  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_IPREAS,
			&sniffdata, &sniffdatasize);
  while(ret==LCRZO_ERR_OK)
  { /* answer for the virtual computer */
    if (lcrzo_global.sock_virt_answeralive)
    { lcrzo_er(lcrzo_virtual_answer_alive(sniffdata, sniffdatasize,
					  psock->here_device, 
					  psock->here_etha, 
					  psock->here_ipl));
    }
    
    /* send data to father */
    ret=lcrzo_packet_decodem_ethipoptudpdata(sniffdata, sniffdatasize,
					     &hdrleth, &hdrlip,
					     NULL, NULL, &hdrludp,
					     &data, &datasize);
    lcrzo_data_free(sniffdata);

    /* if udp was sniffed */
    if (ret==LCRZO_ERR_OK)
    { givepackettouser=LCRZO_FALSE;
      if ( lcrzo_etha_equal(hdrleth.dst, psock->here_etha) &&
	   hdrlip.daddr==psock->here_ipl &&
	   hdrludp.dport==psock->here_port )
      { /* packet is for the virtual computer */
	switch (psock->type) {
	case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
	  /* verify also server */
	  if ( lcrzo_etha_equal(hdrleth.src, psock->there_etha) &&
	       hdrlip.saddr==psock->there_ipl &&
	       hdrludp.sport==psock->there_port )
	  { givepackettouser=LCRZO_TRUE;
	  }
	  break;
	case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
	  /* if we do not know the client, set it */
	  if (psock->there_ipl==0)
	  { memcpy(psock->there_etha, hdrleth.src, LCRZO_ETHA_MAXBYTES);
	    psock->there_ipl=hdrlip.saddr;
	    psock->there_port=hdrludp.sport;
	    /* we write it twice because there are two readers */
	    lcrzo_er(lcrzo_ipc_write_ethaiplport(&(psock->cicontrole2),
						 hdrleth.src, hdrlip.saddr,
						 hdrludp.sport));
	    lcrzo_er(lcrzo_ipc_write_ethaiplport(&(psock->cicontrole2),
						 hdrleth.src, hdrlip.saddr,
						 hdrludp.sport));
	    /* this packet is sent to user */
	    givepackettouser=LCRZO_TRUE;
	  }
	  /* else verify the client */
	  else if ( lcrzo_etha_equal(hdrleth.src, psock->there_etha) &&
		    hdrlip.saddr==psock->there_ipl &&
		    hdrludp.sport==psock->there_port )
	  { givepackettouser=LCRZO_TRUE;
	  }
	  break;
	case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
	  lcrzo_er(lcrzo_ipc_write_ethaiplport(&(psock->cicontrole2),
					       hdrleth.src, hdrlip.saddr,
					       hdrludp.sport));
	  givepackettouser=LCRZO_TRUE;
	  break;
	default:
	  return(lcrzo_ieuv("lcrzo_priv_udp_sniffprocess", "psock->type"));
	  break;
	}
      }

      /* eventually give packet to user */
      if (givepackettouser)
      { lcrzo_er(lcrzo_ipc_write_msg(&(psock->cisniff), data, datasize));
      }
      lcrzo_data_free(data);
    }
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_IPREAS, 
			  &sniffdata, &sniffdatasize);
  }

  /* close sniff */
  lcrzo_er(lcrzo_sniff_close(&sniff));

  return(ret);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_udp_spoofprocess(lcrzo_sock *psock);
int lcrzo_priv_udp_spoofprocess(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  switch (psock->type) {
  case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    /* nothing to do */  
    break;
  case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
    /* we have to know who is the client */
    lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), LCRZO_TRUE,
					psock->there_etha,
					&(psock->there_ipl),
					&(psock->there_port)));
    break;
  default:
    return(lcrzo_ieuv("lcrzo_priv_udp_spoofprocess", "psock->type"));
    break;
  }

  /* main loop */
  while(1)  
  { ret=lcrzo_ipc_readm(&(psock->cispoof), LCRZO_TRUE, &data, &datasize);
    if (ret==LCRZO_ERR_OK)
    { lcrzo_er(lcrzo_priv_sock_spiou(psock, data, (lcrzo_uint16)datasize));
      lcrzo_er(lcrzo_data_free(data));
    }
    else if (ret==LCRZO_ERR_OKDATAEND)
    { /* parent process closed the pipe : he wants to end */
      return(LCRZO_ERR_OK);
    }
    else
    { return(ret);
    }
  }

  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
/*-- tcp functions --*/
/*-------------------------------------------------------------*/
#define LCRZO_PRIV_SOCK_TCPSTATE_LISTEN            1
#define LCRZO_PRIV_SOCK_TCPSTATE_SYNSENT           2
#define LCRZO_PRIV_SOCK_TCPSTATE_SYNRECEIVED       3
#define LCRZO_PRIV_SOCK_TCPSTATE_ESTABLISHED       4
#define LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT1          5
#define LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT2          6
#define LCRZO_PRIV_SOCK_TCPSTATE_CLOSEWAIT         7
#define LCRZO_PRIV_SOCK_TCPSTATE_CLOSING           8
#define LCRZO_PRIV_SOCK_TCPSTATE_LASTACK           9
#define LCRZO_PRIV_SOCK_TCPSTATE_TIMEWAIT         10
#define LCRZO_PRIV_SOCK_TCPSTATE_CLOSED           11

#define LCRZO_PRIV_SOCK_TCPSTATE_LISTEN_INIT      21
#define LCRZO_PRIV_SOCK_TCPSTATE_SYNSENT_INIT     22
#define LCRZO_PRIV_SOCK_TCPSTATE_SYNRECEIVED_INIT 23
#define LCRZO_PRIV_SOCK_TCPSTATE_ESTABLISHED_INIT 24
#define LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT1_INIT    25
#define LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT2_INIT    26
#define LCRZO_PRIV_SOCK_TCPSTATE_CLOSEWAIT_INIT   27
#define LCRZO_PRIV_SOCK_TCPSTATE_CLOSING_INIT     28
#define LCRZO_PRIV_SOCK_TCPSTATE_LASTACK_INIT     29
#define LCRZO_PRIV_SOCK_TCPSTATE_TIMEWAIT_INIT    30
#define LCRZO_PRIV_SOCK_TCPSTATE_CLOSED_INIT      31

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_waitendhandshake(lcrzo_sock *psock);
int lcrzo_priv_sock_waitendhandshake(lcrzo_sock *psock)
{ lcrzo_uint32 childret;

  lcrzo_er(lcrzo_ipc_read_uint32(&(psock->cicontrole1), LCRZO_TRUE,
				 &childret));
  if ( childret!=LCRZO_ERR_OK )
  { /* kill the child */
    lcrzo_er(lcrzo_priv_killwait(psock->pidsniff));
    /* close ipc */
    lcrzo_priv_sock_ipc_close(psock);
    /* return the error of the child */
    return(childret);
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_priv_sock_spiot(lcrzo_sock *psock,
			  lcrzo_bool bitsyn, lcrzo_bool bitack, 
			  lcrzo_bool bitfin, lcrzo_bool bitrst,
			  lcrzo_uint32 seqnum,
			  lcrzo_uint32 acknum,
			  lcrzo_uint16 window,
			  lcrzo_data tcpdata,  
			  lcrzo_uint16 tcpdatasize);
int lcrzo_priv_sock_spiot(lcrzo_sock *psock,
			  lcrzo_bool bitsyn, lcrzo_bool bitack, 
			  lcrzo_bool bitfin, lcrzo_bool bitrst,
			  lcrzo_uint32 seqnum,
			  lcrzo_uint32 acknum,
			  lcrzo_uint16 window,
			  lcrzo_data tcpdata,  
			  lcrzo_uint16 tcpdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip  hdrlip;
  lcrzo_hdrltcp hdrltcp;
  int ret;
  lcrzo_data ipopt;
  lcrzo_int32 ipoptsize;
  lcrzo_spoof spoof;

  /*initialisation de l'entete ETH*/
  lcrzo_er(lcrzo_hdrleth_initdefault(&hdrleth));
  memcpy(hdrleth.src, psock->here_etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrleth.dst, psock->there_etha, LCRZO_ETHA_MAXBYTES);
  /*initialisation de l'entete IP*/
  lcrzo_er(lcrzo_hdrlip_initdefault(&hdrlip));
  hdrlip.saddr=psock->here_ipl;
  hdrlip.daddr=psock->there_ipl;
  /*initialisation de l'entete TCP*/
  lcrzo_er(lcrzo_hdrltcp_initdefault(&hdrltcp));
  hdrltcp.sport=psock->here_port;
  hdrltcp.dport=psock->there_port;
  hdrltcp.seqnum=seqnum;
  hdrltcp.acknum=acknum;
  hdrltcp.window=window;
  hdrltcp.syn=(lcrzo_uint8)bitsyn;
  hdrltcp.ack=(lcrzo_uint8)bitack;
  hdrltcp.fin=(lcrzo_uint8)bitfin;
  hdrltcp.rst=(lcrzo_uint8)bitrst;

  /*mise a jour eventuelle des options IP*/
  ret=lcrzo_ipc_readm(&(psock->cioptions), LCRZO_FALSE, &ipopt, &ipoptsize);
  if ( ret==LCRZO_ERR_OK )
  { if (ipoptsize>LCRZO_IPOPT_MAXBYTES)
      return(lcrzo_ie("lcrzo_priv_sock_spiot",
		      "ipoptsize lu depuis l'IPC est > LCRZO_IPOPT_MAXBYTES",
		      "ipoptsize read from IPC is > LCRZO_IPOPT_MAXBYTES"));
    memcpy(psock->ipopt, ipopt, ipoptsize);
    psock->ipoptsize=(lcrzo_uint8)(ipoptsize);
    lcrzo_data_free(ipopt);
  }

  /*envoi d'un packet avec donnees*/
  lcrzo_er(lcrzo_spoof_init(&spoof));
  lcrzo_er(lcrzo_spoof_ethipopttcpoptdata(&spoof, psock->here_device, hdrleth,
					  hdrlip, psock->ipopt,
					  psock->ipoptsize,
					  hdrltcp, NULL, 0,
					  tcpdata, tcpdatasize));
  lcrzo_er(lcrzo_spoof_close(&spoof));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_priv_sock_tcpstate_sniff(lcrzo_sock *psock, 
				   lcrzo_sniff *psniff,
				   lcrzo_bool knowclient,
				   lcrzo_bool *ppacketforus,
				   lcrzo_bool *psyn,
				   lcrzo_bool *pack,
				   lcrzo_bool *pfin,
				   lcrzo_bool *prst,
				   lcrzo_uint32 *pseqnum,
				   lcrzo_uint32 *packnum,
				   lcrzo_uint16 *pwindow,
				   lcrzo_data *pdata,
				   lcrzo_uint16 *pdatasize);
int lcrzo_priv_sock_tcpstate_sniff(lcrzo_sock *psock, 
				   lcrzo_sniff *psniff,
				   lcrzo_bool knowclient,
				   lcrzo_bool *ppacketforus,
				   lcrzo_bool *psyn,
				   lcrzo_bool *pack,
				   lcrzo_bool *pfin,
				   lcrzo_bool *prst,
				   lcrzo_uint32 *pseqnum,
				   lcrzo_uint32 *packnum,
				   lcrzo_uint16 *pwindow,
				   lcrzo_data *pdata,
				   lcrzo_uint16 *pdatasize)
{ lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_data ethdata;
  lcrzo_int32 ethdatasize;
  int ret;

  /* sniff a packet */
  ret=lcrzo_sniff_nextm(psniff, LCRZO_FALSE, LCRZO_SNIFF_TYPE_TCPREOR,
			&ethdata, &ethdatasize);
  if ( ret==LCRZO_ERR_OKTEMPDATAEND )
  { *ppacketforus=LCRZO_FALSE;
    return(LCRZO_ERR_OK);
  }
  else if ( ret!=LCRZO_ERR_OK )
  { return(ret);
  }

  /* anwser for the virtual computer */
  if (lcrzo_global.sock_virt_answeralive)
  { lcrzo_er(lcrzo_virtual_answer_alive(ethdata, ethdatasize,
					psock->here_device, 
					psock->here_etha,
					psock->here_ipl));
  }

  /* try to decode this packet as tcp */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(ethdata, ethdatasize,
					      &hdrleth, 
					      &hdrlip, NULL, NULL,
					      &hdrltcp, NULL, NULL, 
					      pdata, pdatasize);
  lcrzo_data_free(ethdata);
  *ppacketforus=LCRZO_FALSE;
  if ( ret==LCRZO_ERR_OK) 
  { /* set values if the sniffed packet is for us */
    if ( lcrzo_etha_equal(hdrleth.dst, psock->here_etha) &&
	 hdrlip.daddr == psock->here_ipl &&
	 hdrltcp.dport == psock->here_port &&
	 ( !knowclient ||
	   ( lcrzo_etha_equal(hdrleth.src, psock->there_etha) &&
	     hdrlip.saddr == psock->there_ipl &&
	     hdrltcp.sport == psock->there_port 
	   )
	 )
       )
    { *ppacketforus=LCRZO_TRUE;
      *psyn=hdrltcp.syn;	  
      *pack=hdrltcp.ack;	  
      *pfin=hdrltcp.fin;	  
      *prst=hdrltcp.rst;	  
      *pseqnum=hdrltcp.seqnum;	  
      *packnum=hdrltcp.acknum;
      *pwindow=hdrltcp.window;
      /* if we do not know client, we set its value */
      if ( !knowclient )
      { memcpy(psock->there_etha, hdrleth.src, LCRZO_ETHA_MAXBYTES);
        psock->there_ipl=hdrlip.saddr;
	psock->there_port=hdrltcp.sport;
      }
    }
    else
    { /* if the packet wasn't for us, we have to free pdata */
      lcrzo_data_free(*pdata);
    }
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
#define LCRZO_DEBUG_TCPSTATE 0
int lcrzo_priv_sock_tcpstate(lcrzo_sock *psock, lcrzo_sniff *psniff);
int lcrzo_priv_sock_tcpstate(lcrzo_sock *psock, lcrzo_sniff *psniff)
{ lcrzo_bool sn, sp, sn_packetforus, sn_continue;
  lcrzo_bool sn_syn, sn_ack, sn_fin, sn_rst,  sp_syn, sp_ack, sp_fin, sp_rst;
  lcrzo_uint32 sn_seqnum, sn_acknum,  sp_seqnum, sp_acknum;
  lcrzo_uint32 sn_currentseqnum, sp_currentseqnum;
  lcrzo_data sn_data,  sp_data;
  lcrzo_uint16 sn_datasize, sn_window,  sp_datasize, sp_window;
  lcrzo_int32 sp_datasizeint32;
  lcrzo_time timeouttimer;
  lcrzo_uint32 usec, isn;
  lcrzo_bool spoofedsomething, knowclient;
  int numberofsynsent, numesta, ret, counterforsniffregulation;

  /* initialize needed values */
  lcrzo_er(lcrzo_uint32_rand(0, 0xFFFFFFFFu, &isn));
  numberofsynsent=0;
  sp_window=1500;
  knowclient=LCRZO_TRUE;

  /* just for the compiler because "might be used uninitialized" */
  sn=0; sp=0; sp_syn=0; sp_ack=0; sp_fin=0; sp_rst=0; sp_seqnum=0; sp_acknum=0;
  sp_data=NULL; sp_datasize=0; sn_currentseqnum=0; sp_currentseqnum=0;
  numesta=0; counterforsniffregulation=0; sn_window=0; spoofedsomething=0;

  /* main loop */
  while (1) {
    lcrzo_time_sleep(2000);
    /* determine what to do in a given state */
    /*printf("Current state : %d\n", psock->tcpstate);*/

    switch(psock->tcpstate) {

    case LCRZO_PRIV_SOCK_TCPSTATE_SYNSENT_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_SYNSENT_INIT");
      #endif
      sn=LCRZO_FALSE;     /* no sniff */
      sp=LCRZO_TRUE;      /* spoof */
      sp_syn=LCRZO_TRUE;  /* syn */
      sp_ack=LCRZO_FALSE; /* no ack */
      sp_fin=LCRZO_FALSE; /* no fin */
      sp_rst=LCRZO_FALSE; /* no rst */
      sp_currentseqnum=isn;
      sp_seqnum=sp_currentseqnum;
      sp_acknum=0;
      sp_data=NULL;       /* no data */
      sp_datasize=0;      /* no data */
      lcrzo_er(lcrzo_time_init(&timeouttimer));
      numberofsynsent++;
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_SYNSENT;
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_SYNSENT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_SYNSENT");
      #endif
      /* change from spoof to sniff */
      if (sp) {
        sn=LCRZO_TRUE;      /* sniff */
	sp=LCRZO_FALSE;     /* no spoof */
	break;              /* change nothing else */
      }
      /* if no answer is received after 1 second...*/
      lcrzo_er(lcrzo_time_diff(&timeouttimer, NULL, &usec));
      if (usec>1000000)
      { if ( numberofsynsent<3 )
	{ /* we send a new syn */
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_SYNSENT_INIT;
	}
	else
	{ /* the fourth time : abort */
	  return(LCRZO_ERR_IESOCKTCPNOSA);
	}
	break;              /* continue */
      }
      /* the packet is not for us */
      if (!sn_packetforus) {
	break;              /* continue */
      }
      else
      { /* in this state, we do not need data */
	lcrzo_data_free(sn_data);
      }
      /* the server sent us a reset */
      if ( (!sn_syn) && sn_ack && (!sn_fin) && sn_rst && 
	   sn_seqnum==0 && sn_acknum==sp_currentseqnum+1 )
      { return(LCRZO_ERR_IESOCKTCPRST);
      }
      /* the server sent us a synack */
      else if ( sn_syn && sn_ack && (!sn_fin) && (!sn_rst) && 
		sn_acknum==sp_currentseqnum+1 )
      { /* save numbers*/
        sn_currentseqnum=sn_seqnum+1;
        sp_currentseqnum++;
	/* change state and sp */
	sn=LCRZO_FALSE;     /* no sniff */
	sp=LCRZO_TRUE;      /* spoof */
	sp_syn=LCRZO_FALSE;  /* no syn */
	sp_ack=LCRZO_TRUE; /* ack */
	sp_fin=LCRZO_FALSE; /* no fin */
	sp_rst=LCRZO_FALSE; /* no rst */
	sp_seqnum=sp_currentseqnum;
	sp_acknum=sn_currentseqnum;
	sp_data=NULL;       /* no data */
	sp_datasize=0;      /* no data */
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_ESTABLISHED_INIT;
	lcrzo_er(lcrzo_priv_sock_okchild(psock));
      }
      else
      { /* ignore other kinds */
      }
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_ESTABLISHED_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("ESTABLISHED_INIT");
      #endif
      /* this state permits to initialize variables */
      numesta=0;
      counterforsniffregulation=10;
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_ESTABLISHED;
      /*no break*/
    case LCRZO_PRIV_SOCK_TCPSTATE_ESTABLISHED:
      #if LCRZO_DEBUG_TCPSTATE
        printf("TCPSTATE_ESTABLISHED : numesta=%d\n", numesta);
      #endif
      sn=LCRZO_FALSE;     /* no sniff */
      sp=LCRZO_FALSE;     /* no spoof */
      /*-------*/
      if ( numesta==0 )
      { numesta++;
	/* we are going to sniff */
	sn=LCRZO_TRUE;     /* sniff */
	break;
      }
      /*-------*/
      else if ( numesta==1 )
      { /*++++++++*/
	/* This zone is particular : we have to sniff more often
	   than other steps, and this must be regulated. */
	{ /* normally, we stop to sniff when counterforsniffregulation==0 */
	  if ( (counterforsniffregulation--)<=0 )
	    sn_continue=LCRZO_FALSE;
	  else
	    sn_continue=LCRZO_TRUE;
	  /* first exception: when we are syncronized */
	  if ( sn_packetforus && (sn_acknum==sp_currentseqnum) )
	    sn_continue=LCRZO_FALSE;
	  /* second exception: we reached the distant half window size */
	  if ( sn_packetforus && ((sp_currentseqnum-sn_acknum)>(sn_window/2)) )
	    sn_continue=LCRZO_TRUE;
	  /* we've decided, so now we set variables */
	  if (sn_continue) {
	    numesta=0;
	  }
	  else {
	    numesta++;
	    counterforsniffregulation=10;
	  }
	}
	/*++++++++*/
	/* look at what we've sniffed */
	if (!sn_packetforus) {
	  /* the packet is not for us */
	  break;              /* continue */
	}
	/* packet contains a syn */
	if (sn_syn) {
	  lcrzo_data_free(sn_data);
	  break;              /* ignore */
	}
	/* packet contains a rst */
	if (sn_rst) {
	  lcrzo_data_free(sn_data);
	  if (sn_seqnum==sn_currentseqnum)
	  { psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	  }
	  break;
	}
	/* packet contains a fin */
	if (sn_fin) {
	  lcrzo_data_free(sn_data);
	  if (sn_seqnum==sn_currentseqnum) {
	    /* send an ack */
	    sp=LCRZO_TRUE;      /* spoof */
	    sp_syn=LCRZO_FALSE;  /* no syn */
	    sp_ack=LCRZO_TRUE; /* ack */
	    sp_fin=LCRZO_FALSE; /* no fin */
	    sp_rst=LCRZO_FALSE; /* no rst */
	    sp_seqnum=sp_currentseqnum;
	    sn_currentseqnum++;
	    sp_acknum=sn_currentseqnum;
	    sp_data=NULL;       /* no data */
	    sp_datasize=0;      /* no data */
	    psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSEWAIT;
	  }
	  break;
	}
	if ( (!sn_syn) && sn_ack && (!sn_fin) && (!sn_rst) )
        { sn_currentseqnum+=sn_datasize;
	  /* give data to the user */
	  lcrzo_er(lcrzo_ipc_write_msg(&(psock->cisniff), 
				       sn_data, sn_datasize));
	  lcrzo_data_free(sn_data);
	  /* we are going to spoof (ack for this packet) */
	  sp=LCRZO_TRUE;    /* spoof */
	  sp_syn=LCRZO_FALSE;  /* no syn */
	  sp_ack=LCRZO_TRUE; /* ack */
	  sp_fin=LCRZO_FALSE; /* no fin */
	  sp_rst=LCRZO_FALSE; /* no rst */
	  sp_seqnum=sp_currentseqnum;
	  sp_acknum=sn_currentseqnum;
	  sp_data=NULL;       /* no data */
	  sp_datasize=0;      /* no data */
	}
	else
        { lcrzo_data_free(sn_data);
	  break;  /* ignore this paquet (for example a NUL packet)*/
	}
	break;
      }
      /*-------*/
      else if ( numesta==2 )
      { numesta++;
        /* we spoof user's data */
        ret=lcrzo_ipc_readm(&(psock->cispoof), LCRZO_FALSE, 
			    &sp_data, &sp_datasizeint32);
	if (ret==LCRZO_ERR_OK)
	{ sp=LCRZO_TRUE;      /* spoof */
	  sp_syn=LCRZO_FALSE;  /* no syn */
	  sp_ack=LCRZO_TRUE; /* ack */
	  sp_fin=LCRZO_FALSE; /* no fin */
	  sp_rst=LCRZO_FALSE; /* no rst */
	  sp_seqnum=sp_currentseqnum;
	  sp_acknum=sn_currentseqnum;
	  sp_datasize=sp_datasizeint32;
	  sp_currentseqnum+=sp_datasize; /* if >0xFFFFFFFF, pass to 0*/
	  spoofedsomething=LCRZO_TRUE;
	}
	else
	{ spoofedsomething=LCRZO_FALSE;
	}
	break;
      }
      /*-------*/
      else if ( numesta==3 )
      { numesta++;
        /* we look if parent process want us to stop */
        /* we do not look, while we have data to spoof */
        if (spoofedsomething) break;
	/* see if parent sent us something */
        ret=lcrzo_ipc_read_uint8(&(psock->cicontrole2), LCRZO_FALSE, NULL);
	if ( ret==LCRZO_ERR_OK )
	{ /* send a fin */
	  sp=LCRZO_TRUE;      /* spoof */
	  sp_syn=LCRZO_FALSE; /* no syn */
	  sp_ack=LCRZO_TRUE; /* ack */
	  sp_fin=LCRZO_TRUE;  /* fin */
	  sp_rst=LCRZO_FALSE; /* no rst */
	  sp_seqnum=sp_currentseqnum;
	  sp_acknum=sn_currentseqnum;
	  sp_data=NULL;
	  sp_datasize=0;
	  sp_currentseqnum++; /* if >0xFFFFFFFF, pass to 0*/
          psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT1_INIT;
	}
	break;
      }
      /*-------*/
      else
      { 
#if defined LCRZODEF_SYSTEM_FreeBSD
	/* on FreeBSD, select doesn't work on psniff->pcapfd, and blocks.
	   We could specify a timeout, but on loaded network, we miss 
	   sniffed packets. So, on FreeBSD we do not wait
	*/
#else
        fd_set readfdset;
        int maxfd;

	FD_ZERO(&readfdset);
	/* watch sniff */
	FD_SET(psniff->pcapfd, &readfdset);
        maxfd=psniff->pcapfd;
	/* watch spoof */
	FD_SET(psock->cispoof.readpipe, &readfdset);
	if ( psock->cispoof.readpipe > maxfd ) 
	  maxfd=psock->cispoof.readpipe;
	/* watch control */
	FD_SET(psock->cicontrole2.readpipe, &readfdset);
	if ( psock->cicontrole2.readpipe > maxfd ) 
	  maxfd=psock->cicontrole2.readpipe;
	/* wait */
	maxfd++; /* because maxfd=max(fd1, fd2, fd3)+1 */
        #if LCRZO_DEBUG_TCPSTATE
          puts("TCPSTATE : wait with select");
        #endif
	ret=select(maxfd, &readfdset, NULL, NULL, NULL);
	if (ret<0) return(LCRZO_ERR_FUSELECT);
#endif
	/* after waiting, we restart */
	numesta=0;
	break;
      }
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_CLOSEWAIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_CLOSEWAIT");
      #endif
      ret=lcrzo_ipc_readm(&(psock->cispoof), LCRZO_FALSE, 
			  &sp_data, &sp_datasizeint32);
      if (ret==LCRZO_ERR_OK) { 
	/* continue to send user's data */
	sn=LCRZO_FALSE;      /* no sniff */
	sp=LCRZO_TRUE;      /* spoof */
	sp_syn=LCRZO_FALSE;  /* no syn */
	sp_ack=LCRZO_TRUE; /* ack */
	sp_fin=LCRZO_FALSE; /* no fin */
	sp_rst=LCRZO_FALSE; /* no rst */
	sp_seqnum=sp_currentseqnum;
	sp_acknum=sn_currentseqnum;
	sp_datasize=sp_datasizeint32;
	sp_currentseqnum+=sp_datasize; /* if >0xFFFFFFFF, pass to 0*/
      }
      else {
	/* send a fin */
	sn=LCRZO_FALSE;      /* no sniff */
	sp=LCRZO_TRUE;      /* spoof */
	sp_syn=LCRZO_FALSE;  /* no syn */
	sp_ack=LCRZO_TRUE;  /* ack */
	sp_fin=LCRZO_TRUE; /* fin */
	sp_rst=LCRZO_FALSE; /* no rst */
	sp_seqnum=sp_currentseqnum;
	sp_acknum=sn_currentseqnum;
	sp_data=NULL;
	sp_datasize=0;
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_LASTACK_INIT;
      }
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_LASTACK_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_LASTACK_INIT");
      #endif
      /* this state permits to initialize variables */
      sn=LCRZO_TRUE;      /* sniff */
      sp=LCRZO_FALSE;     /* no spoof */
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_LASTACK;
      lcrzo_er(lcrzo_time_init(&timeouttimer));
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_LASTACK:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_LASTACK");
      #endif
      /* we sniff to have the ack of our fin */
      /* if no answer is received after 1 second...*/
      lcrzo_er(lcrzo_time_diff(&timeouttimer, NULL, &usec));
      if (usec>1000000)
      { /* jump to closed state */
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	if (sn_packetforus) lcrzo_data_free(sn_data);
	break;
      }
      /* look at what we've sniffed */
      if (!sn_packetforus) {
	/* the packet is not for us */
	break;              /* continue */
      }
      /* packet contains a rst */
      if ( sn_rst ) {
	if (sn_seqnum==sn_currentseqnum) { 
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	  lcrzo_data_free(sn_data);
	  break;
	}
      }
      /* packet contains an ack */
      if ( sn_ack ) {
	if (sn_acknum==sp_currentseqnum+1) { 
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	  lcrzo_data_free(sn_data);
	  break;
	}
      }
      /* ignore other kinds of packets */
      lcrzo_data_free(sn_data);
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT1_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_FINWAIT1_INIT");
      #endif
      /* this state permits to initialize variables */
      sn=LCRZO_TRUE;      /* sniff */
      sp=LCRZO_FALSE;     /* no spoof */
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT1;
      lcrzo_er(lcrzo_time_init(&timeouttimer));
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT1:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_FINWAIT1");
      #endif
      /* if no answer is received after 1 second...*/
      lcrzo_er(lcrzo_time_diff(&timeouttimer, NULL, &usec));
      if (usec>1000000)
      { /* jump to closed state */
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	if (sn_packetforus) lcrzo_data_free(sn_data);
	break;
      }
      /* look at what we've sniffed */
      if (!sn_packetforus) {
	/* the packet is not for us */
	break;              /* continue */
      }
      /* packet contains a rst */
      if ( sn_rst ) {
	if (sn_seqnum==sn_currentseqnum) { 
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	  lcrzo_data_free(sn_data);
	  break;
	}
      }
      /* packet contains an ack */
      if ( sn_ack ) {
	if (sn_acknum==sp_currentseqnum) { 
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT2_INIT;
	  lcrzo_data_free(sn_data);
	  break;
	}
      }
      /* packet contains a fin */
      if ( sn_fin ) {
	lcrzo_data_free(sn_data);
	sn=LCRZO_FALSE;      /* no sniff */
	sp=LCRZO_TRUE;      /* spoof */
	sp_syn=LCRZO_FALSE;  /* no syn */
	sp_ack=LCRZO_TRUE;  /* ack */
	sp_fin=LCRZO_FALSE; /* no fin */
	sp_rst=LCRZO_FALSE; /* no rst */
	sp_seqnum=sp_currentseqnum;
	sn_currentseqnum=sn_seqnum; /* user asked to close so we do not
				     give him data between sn_currentseqnum
				     and sn_seqnum */
	sp_acknum=sn_currentseqnum+1;
	sp_data=NULL;
	sp_datasize=0;
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSING_INIT;
	break;
      }
      /* ignore other kinds of packets */
      lcrzo_data_free(sn_data);
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT2_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_FINWAIT2_INIT");
      #endif
      /* this state permits to initialize variables */
      sn=LCRZO_TRUE;      /* sniff */
      sp=LCRZO_FALSE;     /* no spoof */
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT2;
      lcrzo_er(lcrzo_time_init(&timeouttimer));
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_FINWAIT2 :
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_FINWAIT2");
      #endif
      /* if no answer is received after 1 second...*/
      lcrzo_er(lcrzo_time_diff(&timeouttimer, NULL, &usec));
      if (usec>1000000)
      { /* jump to closed state */
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	if (sn_packetforus) lcrzo_data_free(sn_data);
	break;
      }
      /* look at what we've sniffed */
      if (!sn_packetforus) {
	/* the packet is not for us */
	break;              /* continue */
      }
      /* packet contains a rst */
      if ( sn_rst ) {
	if (sn_seqnum==sn_currentseqnum) { 
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	  lcrzo_data_free(sn_data);
	  break;
	}
      }
      /* packet contains a fin */
      if ( sn_fin ) {
	lcrzo_data_free(sn_data);
	sn=LCRZO_FALSE;      /* no sniff */
	sp=LCRZO_TRUE;      /* spoof */
	sp_syn=LCRZO_FALSE;  /* no syn */
	sp_ack=LCRZO_TRUE;  /* ack */
	sp_fin=LCRZO_FALSE; /* no fin */
	sp_rst=LCRZO_FALSE; /* no rst */
	sp_seqnum=sp_currentseqnum;
	sn_currentseqnum=sn_seqnum; /* user asked to close so we do not
				     give him data between sn_currentseqnum
				     and sn_seqnum */
	sp_acknum=sn_currentseqnum+1;
	sp_data=NULL;
	sp_datasize=0;
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSING_INIT;
	break;
      }
      /* ignore other kinds of packets */
      lcrzo_data_free(sn_data);
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_CLOSING_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_CLOSING_INIT");
      #endif
      /* this state permits to initialize variables */
      sn=LCRZO_TRUE;      /* sniff */
      sp=LCRZO_FALSE;     /* no spoof */
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSING;
      lcrzo_er(lcrzo_time_init(&timeouttimer));
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_CLOSING:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_CLOSING");
      #endif
      /* if no answer is received after 1 second...*/
      lcrzo_er(lcrzo_time_diff(&timeouttimer, NULL, &usec));
      if (usec>1000000)
      { /* jump to closed state */
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	if (sn_packetforus) lcrzo_data_free(sn_data);
	break;
      }
      /* look at what we've sniffed */
      if (!sn_packetforus) {
	/* the packet is not for us */
	break;              /* continue */
      }
      /* packet contains a rst */
      if ( sn_rst ) {
	if (sn_seqnum==sn_currentseqnum) { 
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
	  lcrzo_data_free(sn_data);
	  break;
	}
      }
      /* packet contains an ack */
      if ( sn_ack ) {
	if (sn_acknum==sp_currentseqnum) { 
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_TIMEWAIT;
	  lcrzo_data_free(sn_data);
	  break;
	}
      }
      /* ignore other kinds of packets */
      lcrzo_data_free(sn_data);
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_TIMEWAIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_TIMEWAIT");
      #endif
      sn=LCRZO_FALSE;      /* no sniff */
      sp=LCRZO_FALSE;      /* no spoof */
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_CLOSED;
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_LISTEN_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("CPSTATE_LISTEN_INIT");
      #endif
      sn=LCRZO_TRUE;      /* sniff */
      sp=LCRZO_FALSE;      /* no spoof */
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_LISTEN;
      knowclient=LCRZO_FALSE;
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_LISTEN:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_LISTEN");
      #endif
      knowclient=LCRZO_FALSE;
      /* look at what we've sniffed */
      if (!sn_packetforus) {
	/* the packet is not for us */
	break;              /* continue */
      }
      /* packet contains a syn */
      if ( sn_syn && (!sn_ack) && (!sn_fin) && (!sn_rst) ) {
	lcrzo_data_free(sn_data);
	sn=LCRZO_FALSE;      /* no sniff */
	sp=LCRZO_TRUE;      /* spoof */
	sp_syn=LCRZO_TRUE;  /* syn */
	sp_ack=LCRZO_TRUE;  /* ack */
	sp_fin=LCRZO_FALSE; /* no fin */
	sp_rst=LCRZO_FALSE; /* no rst */
	sp_currentseqnum=isn;
	sn_currentseqnum=sn_seqnum+1;
	sp_seqnum=sp_currentseqnum;
	sp_acknum=sn_currentseqnum;
	sp_data=NULL;
	sp_datasize=0;
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_SYNRECEIVED_INIT;
	break;
      }
      /* ignore other kinds of packets */
      lcrzo_data_free(sn_data);
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_SYNRECEIVED_INIT:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_SYNRECEIVED_INIT");
      #endif
      sn=LCRZO_TRUE;      /* sniff */
      sp=LCRZO_FALSE;      /* no spoof */
      psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_SYNRECEIVED;
      lcrzo_er(lcrzo_time_init(&timeouttimer));
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_SYNRECEIVED :
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_SYNRECEIVED");
      #endif
      /* if no answer is received after 1 second...*/
      lcrzo_er(lcrzo_time_diff(&timeouttimer, NULL, &usec));
      if (usec>1000000)
      { /* jump to listen state */
	psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_LISTEN_INIT;
	if (sn_packetforus) lcrzo_data_free(sn_data);
	break;
      }
      /* look at what we've sniffed */
      if (!sn_packetforus) {
	/* the packet is not for us */
	break;              /* continue */
      }
      /* packet contains an ack */
      if ( (!sn_syn) && sn_ack && (!sn_fin) && (!sn_rst) ) {
	lcrzo_data_free(sn_data);
	if ( sn_acknum==sp_currentseqnum+1 )
	{ sn=LCRZO_FALSE;      /* no sniff */
	  sp=LCRZO_TRUE;      /* no spoof */
	  sp_currentseqnum++;
	  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_ESTABLISHED_INIT;
	  /* inform parent process */
	  lcrzo_er(lcrzo_priv_sock_okchild(psock));
	  lcrzo_er(lcrzo_ipc_write_ethaiplport(&(psock->cicontrole1),
					       psock->there_etha,
					       psock->there_ipl,
					       psock->there_port));
	}
	break;
      }
      /* ignore other kinds of packets */
      lcrzo_data_free(sn_data);
      break;

    case LCRZO_PRIV_SOCK_TCPSTATE_CLOSED:
      #if LCRZO_DEBUG_TCPSTATE
        puts("TCPSTATE_CLOSED");
      #endif
      return(LCRZO_ERR_OK);
      break;

    default:
      return(lcrzo_ieuv("lcrzo_priv_sock_tcpstate", "psock->tcpstate"));
    }

    /* check sn and sp */
    if (sn&&sp) {
      return(lcrzo_ie("lcrzo_priv_sock_tcpstate", 
		      "sn et sp sont affectes",
		      "sn and sp both set"));
    }
  
    /* sniff */
    if (sn) {  
      lcrzo_er(lcrzo_priv_sock_tcpstate_sniff(psock, psniff, knowclient,
					      &sn_packetforus,
					      &sn_syn,&sn_ack,&sn_fin,&sn_rst,
					      &sn_seqnum, &sn_acknum,
					      &sn_window,
					      &sn_data, &sn_datasize));
    }

    /* spoof */
    if (sp) { 
      /* spoof a packet */
      lcrzo_er(lcrzo_priv_sock_spiot(psock, sp_syn, sp_ack, sp_fin, sp_rst,
				     sp_seqnum, sp_acknum, sp_window,
				     sp_data, sp_datasize));
      if (sp_data!=NULL) lcrzo_data_free(sp_data);
    }

  }

  /* should never reach this point */
  return(lcrzo_ierp("lcrzo_priv_sock_tcpstate"));
}

/* Fin zone de fonctions privees */
/***************************************************************/

/*-------------------------------------------------------------*/
int lcrzo_sock_udpcli_real(lcrzo_ipl    ipl, 
			  lcrzo_uint16 port,
			  lcrzo_sock   *psock)
{ return(lcrzo_sock_udpcli_real2(0, ipl, 0, port, NULL, 0, psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpcli_virt(const lcrzo_device device,
			   const lcrzo_etha ethaclient,
			   const lcrzo_etha ethaserveur,
			   const lcrzo_ipl iplclient,
			   const lcrzo_ipl iplserveur, 
			   lcrzo_uint16 portclient,
			   lcrzo_uint16 portserveur,
			   lcrzo_sock *psock)
{ return(lcrzo_sock_udpcli_virt2(device, ethaclient, ethaserveur,
				 iplclient, iplserveur,
				 portclient, portserveur,
				 NULL, 0,
				 psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpcli_real2(lcrzo_ipl    iplclient,
			    lcrzo_ipl    iplserveur, 
			    lcrzo_uint16 portclient,
			    lcrzo_uint16 portserveur,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock   *psock)
{ struct sockaddr_in cli;
  int retour, sock;

  /*parameters verification*/
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);

  /*creation de la socket*/
  sock=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
  if (sock<0) return(LCRZO_ERR_FUSOCKET);

  /*ajout eventuel des options IP*/
  if (ipoptsize)
  { retour = setsockopt(sock, IPPROTO_IP, IP_OPTIONS, ipopt, ipoptsize);
    if (retour)
      return(LCRZO_ERR_FUSETSOCKOPT);
  }

  /*le client specifie des infos locales*/
  if ( iplclient || portclient )
  { memset(&cli, 0, sizeof(cli));
    cli.sin_family = AF_INET;
    cli.sin_addr.s_addr=lcrzo_htonl(iplclient);
    cli.sin_port = lcrzo_htons(portclient);
    retour=bind(sock, (struct sockaddr *)&cli, sizeof(cli));
    if (retour<0) return(LCRZO_ERR_FUBIND);
  }

  /*connexion sur le serveur*/
  memset(&cli, 0, sizeof(cli));
  cli.sin_family = AF_INET;
  cli.sin_addr.s_addr=lcrzo_htonl(iplserveur);
  cli.sin_port = lcrzo_htons(portserveur);
  retour=connect(sock, (struct sockaddr *)&cli, sizeof(cli));
  if (retour<0) return(LCRZO_ERR_FUCONNECT);

  /*if the user only want to test, its ok, so we close*/
  if (psock==NULL)
  { retour=close(sock);
    if ( retour == -1 ) return(LCRZO_ERR_FUCLOSE);
    return(LCRZO_ERR_OK);
  }

  /*remplissage des caracteristiques de la socket*/
  psock->type=LCRZO_PRIV_TYPESOCK_REALCLIUDP;
  psock->socknum=sock;
  psock->beblocking=1;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;

  /*remplissage des informations connues et initialisation des autres*/
  lcrzo_er(lcrzo_device_initdefault(psock->here_device));
  lcrzo_er(lcrzo_etha_initdefault(psock->here_etha));
  lcrzo_er(lcrzo_etha_initdefault(psock->there_etha));
  psock->there_ipl=iplserveur;
  psock->there_port=portserveur;
  psock->here_ipl=iplclient;
  psock->here_port=portclient;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpcli_virt2(const lcrzo_device device,
			    const lcrzo_etha ethaclient,
			    const lcrzo_etha ethaserveur,
			    const lcrzo_ipl iplclient,
			    const lcrzo_ipl iplserveur, 
			    lcrzo_uint16 portclient,
			    lcrzo_uint16 portserveur,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock *psock)
{ 
  /*parameters verification*/
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ethaserveur==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ethaclient==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  /*initialisations*/
  psock->type=LCRZO_PRIV_TYPESOCK_VIRTCLIUDP;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;
  lcrzo_er(lcrzo_priv_sock_majinfos(device, ethaclient, iplclient, portclient,
				    ethaserveur, iplserveur, portserveur,
				    psock));
  lcrzo_er(lcrzo_priv_sock_ipc_init(psock));
  lcrzo_efr(lcrzo_sock_ipopt_set(psock, ipopt, ipoptsize),
	    lcrzo_priv_sock_ipc_close(psock));

  /* sniff process */
  lcrzo_er(lcrzo_priv_sock_fork(psock, &(psock->pidsniff)));
  if (psock->pidsniff==0) /* we are in the child */
  { /* close unnecessary IPC */
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole1)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cicontrole2)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cioptions)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_read(&(psock->cisniff)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cispoof)));
    /* sniff function */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_udp_sniffprocess(psock));
    /* end - should not be reatched */
    lcrzo_priv_sock_eechild(psock, LCRZO_ERR_IEINTERNALERROR);
    _exit(LCRZO_ERR_IEINTERNALERROR);
  }

  /* spoof process */
  lcrzo_er(lcrzo_priv_sock_fork(psock, &(psock->pidspoof)));
  if (psock->pidspoof==0) /* we are in the child */
  { /* close unnecessary IPC */
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole1)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cicontrole2)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cioptions)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cisniff)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cispoof)));
    /* spoof function */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_udp_spoofprocess(psock));
    /* end */
    lcrzo_priv_sock_okchild(psock);
    _exit(LCRZO_ERR_OK);
  }

  /* close unnecessary IPC */
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cicontrole1)));
  lcrzo_er(lcrzo_ipc_close(&(psock->cicontrole2)));
  lcrzo_er(lcrzo_ipc_close_read(&(psock->cioptions)));
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cisniff)));
  lcrzo_er(lcrzo_ipc_close_read(&(psock->cispoof)));
  
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_sock_tcpcli_real(lcrzo_ipl    ipl,
			  lcrzo_uint16 port, 
			  lcrzo_sock   *psock)
{ return(lcrzo_sock_tcpcli_real2(0, ipl, 0, port, NULL, 0, psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpcli_virt(const lcrzo_device device,
			   const lcrzo_etha ethaclient,
			   const lcrzo_etha ethaserveur,
			   const lcrzo_ipl iplclient,
			   const lcrzo_ipl iplserveur, 
			   lcrzo_uint16 portclient,
			   lcrzo_uint16 portserveur,
			   lcrzo_sock *psock)
{ return(lcrzo_sock_tcpcli_virt2(device, ethaclient, ethaserveur,
				 iplclient, iplserveur, 
				 portclient, portserveur,
				 NULL, 0,
				 psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpcli_real2(lcrzo_ipl    iplclient,
			    lcrzo_ipl    iplserveur, 
			    lcrzo_uint16 portclient,
			    lcrzo_uint16 portserveur,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock   *psock)
{ struct sockaddr_in cli;
  int retour, sock;

  /*parameters verification*/
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);

  /*creation de la socket*/
  sock=socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
  if (sock<0) return(LCRZO_ERR_FUSOCKET);

  /*ajout eventuel des options IP*/
  if (ipoptsize)
  { retour = setsockopt(sock, IPPROTO_IP, IP_OPTIONS, ipopt, ipoptsize);
    if (retour)
      return(LCRZO_ERR_FUSETSOCKOPT);
  }

  /*le client specifie des infos locales*/
  if ( iplclient || portclient )
  { memset(&cli, 0, sizeof(cli));
    cli.sin_family = AF_INET;
    cli.sin_addr.s_addr=lcrzo_htonl(iplclient);
    cli.sin_port = lcrzo_htons(portclient);
    retour=bind(sock, (struct sockaddr *)&cli, sizeof(cli));
    if (retour<0) return(LCRZO_ERR_FUBIND);
  }

  /*connexion sur le serveur*/
  memset(&cli, 0, sizeof(cli));
  cli.sin_family = AF_INET;
  cli.sin_addr.s_addr=lcrzo_htonl(iplserveur);
  cli.sin_port = lcrzo_htons(portserveur);
  retour=connect(sock, (struct sockaddr *)&cli, sizeof(cli));
  if (retour<0) return(LCRZO_ERR_FUCONNECT);

  /*if the user only want to test, its ok, so we close*/
  if (psock==NULL)
  { retour=close(sock);
    if ( retour == -1 ) return(LCRZO_ERR_FUCLOSE);
    return(LCRZO_ERR_OK);
  }

  /*remplissage des caracteristiques de la socket*/
  psock->type=LCRZO_PRIV_TYPESOCK_REALCLITCP;
  psock->socknum=sock;
  psock->beblocking=1;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;

  /*remplissage des informations connues et initialisation des autres*/
  lcrzo_er(lcrzo_device_initdefault(psock->here_device));
  lcrzo_er(lcrzo_etha_initdefault(psock->here_etha));
  lcrzo_er(lcrzo_etha_initdefault(psock->there_etha));
  psock->there_ipl=iplserveur;
  psock->there_port=portserveur;
  psock->here_ipl=iplclient;
  psock->here_port=portclient;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpcli_virt2(const lcrzo_device device,
			    const lcrzo_etha ethaclient,
			    const lcrzo_etha ethaserveur,
			    const lcrzo_ipl iplclient,
			    const lcrzo_ipl iplserveur, 
			    lcrzo_uint16 portclient,
			    lcrzo_uint16 portserveur,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock *psock)
{ lcrzo_sniff sniff;

  /* parameters verification */
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ethaserveur==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ethaclient==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  /* initializations */
  psock->type=LCRZO_PRIV_TYPESOCK_VIRTCLITCP;
  psock->connectionclosed=LCRZO_FALSE;
  psock->storeddatasize=0;
  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_SYNSENT_INIT;
  lcrzo_er(lcrzo_priv_sock_majinfos(device, ethaclient, iplclient, portclient,
                                    ethaserveur, iplserveur, portserveur,
                                    psock));
  lcrzo_er(lcrzo_priv_sock_ipc_init(psock));
  lcrzo_efr(lcrzo_sock_ipopt_set(psock, ipopt, ipoptsize),
	    lcrzo_priv_sock_ipc_close(psock));

  /* we use one process : we use pidsniff to store it*/
  lcrzo_er(lcrzo_priv_sock_fork(psock, &(psock->pidsniff)));
  if (psock->pidsniff==0) /* we are in the child */
  { /* close unnecessary IPC */
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole1)));
    lcrzo_priv_sock_eechild(psock, 
			    lcrzo_ipc_close_write(&(psock->cicontrole2)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cioptions)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_read(&(psock->cisniff)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cispoof)));
     /* initialize sniff */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_sniff_init_dev(device, &sniff));
    /* run the state machine */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_sock_tcpstate(psock, &sniff));
    /* close sniff */
    lcrzo_priv_sock_eechild(psock, lcrzo_sniff_close(&sniff));
    /* end */
    lcrzo_priv_sock_okchild(psock);
    _exit(LCRZO_ERR_OK);
  }
  else
  { /* close unnecessary IPC */
    lcrzo_er(lcrzo_ipc_close_write(&(psock->cicontrole1)));
    lcrzo_er(lcrzo_ipc_close_read(&(psock->cicontrole2)));
    lcrzo_er(lcrzo_ipc_close_read(&(psock->cioptions)));
    lcrzo_er(lcrzo_ipc_close_write(&(psock->cisniff)));
    lcrzo_er(lcrzo_ipc_close_read(&(psock->cispoof)));
  }

  /* wait the end of the handshake */
  lcrzo_er(lcrzo_priv_sock_waitendhandshake(psock));
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_sock_udpser_real(lcrzo_uint16 port,
			  lcrzo_sock *psock)
{ return(lcrzo_sock_udpser_real2(0, port, NULL, 0, psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpser_virt(const lcrzo_device device,
			  const lcrzo_etha ethaserveur,
			  const lcrzo_ipl iplserveur, 
			  lcrzo_uint16 portserveur,
			  lcrzo_sock *psock)
{ return(lcrzo_sock_udpser_virt2(device, ethaserveur,
				 iplserveur, portserveur,
				 NULL, 0, psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpser_real2(lcrzo_ipl ipl,
			    lcrzo_uint16 port,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock *psock)
{ struct sockaddr_in sain;
  int retour, sock;

  /*parameters verification*/
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);

  /*creation de la socket*/
  sock=socket(AF_INET, SOCK_DGRAM, 0);
  if (sock<0) return(LCRZO_ERR_FUSOCKET);

  /*ajout eventuel des options IP*/
  if (ipoptsize)
  { retour = setsockopt(sock, IPPROTO_IP, IP_OPTIONS, ipopt, ipoptsize);
    if (retour)
      return(LCRZO_ERR_FUSETSOCKOPT);
  }

  /*ecoute sur le port*/
  memset(&sain, 0, sizeof(sain));
  sain.sin_family = AF_INET;
  if ( ipl==0 )
    sain.sin_addr.s_addr=lcrzo_htonl(INADDR_ANY);
  else
    sain.sin_addr.s_addr=lcrzo_htonl(ipl);
  sain.sin_port = lcrzo_htons(port);
  retour=bind(sock, (struct sockaddr *)&sain, sizeof(sain));
  if (retour<0) return(LCRZO_ERR_FUBIND);

  /*if the user only want to test, its ok, so we close*/
  if (psock==NULL)
  { retour=close(sock);
    if ( retour == -1 ) return(LCRZO_ERR_FUCLOSE);
    return(LCRZO_ERR_OK);
  }

  /*remplissage des caracteristiques de la socket*/
  psock->type=LCRZO_PRIV_TYPESOCK_REALSERUDP;
  psock->socknum=sock;
  psock->canwrite=0;
  psock->beblocking=1;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;

  /*remplissage des informations connues et initialisation des autres*/
  lcrzo_er(lcrzo_device_initdefault(psock->here_device));
  lcrzo_er(lcrzo_etha_initdefault(psock->here_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->here_ipl)));
  psock->here_port=port;
  lcrzo_er(lcrzo_etha_initdefault(psock->there_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->there_ipl)));
  psock->there_port=0;
   return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpser_virt2(const lcrzo_device device,
			    const lcrzo_etha ethaserveur,
			    const lcrzo_ipl iplserveur, 
			    lcrzo_uint16 portserveur,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock *psock)
{ 
  /*parameters verification*/
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ethaserveur==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  /*initialisations*/
  psock->type=LCRZO_PRIV_TYPESOCK_VIRTSERUDP;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;
  lcrzo_er(lcrzo_device_init(device, psock->here_device));
  memcpy(psock->here_etha, ethaserveur, LCRZO_ETHA_MAXBYTES);
  psock->here_ipl=iplserveur;
  psock->here_port=portserveur;
  lcrzo_er(lcrzo_etha_initdefault(psock->there_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->there_ipl)));
  psock->there_port=0;
  psock->canwrite=0;
  lcrzo_er(lcrzo_priv_sock_ipc_init(psock));
  lcrzo_efr(lcrzo_sock_ipopt_set(psock, ipopt, ipoptsize),
	    lcrzo_priv_sock_ipc_close(psock));

  /* sniff process */
  lcrzo_er(lcrzo_priv_sock_fork(psock, &(psock->pidsniff)));
  if (psock->pidsniff==0) /* we are in the child */
  { /* close unnecessary IPC */
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole1)));
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole2)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cioptions)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_read(&(psock->cisniff)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cispoof)));
    /* sniff function */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_udp_sniffprocess(psock));
    /* end - should not be reatched */
    lcrzo_priv_sock_eechild(psock, LCRZO_ERR_IEINTERNALERROR);
    _exit(LCRZO_ERR_IEINTERNALERROR);
  }

  /* spoof process */
  lcrzo_er(lcrzo_priv_sock_fork(psock, &(psock->pidspoof)));
  if (psock->pidspoof==0) /* we are in the child */
  { /* close unnecessary IPC */
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole1)));
    lcrzo_priv_sock_eechild(psock,
			    lcrzo_ipc_close_write(&(psock->cicontrole2)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cioptions)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cisniff)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cispoof)));
    /* spoof function */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_udp_spoofprocess(psock));
    /* end */
    lcrzo_priv_sock_okchild(psock);
    _exit(LCRZO_ERR_OK);
  }

  /* close unnecessary IPC */
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cicontrole1)));
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cicontrole2)));
  lcrzo_er(lcrzo_ipc_close_read(&(psock->cioptions)));
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cisniff)));
  lcrzo_er(lcrzo_ipc_close_read(&(psock->cispoof)));

  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_sock_udpmulser_real(lcrzo_uint16 port,
			      lcrzo_sock *psock)
{ return(lcrzo_sock_udpmulser_real2(0, port, psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpmulser_virt(const lcrzo_device device,
			      const lcrzo_etha ethaserveur,
			      const lcrzo_ipl iplserveur, 
			      lcrzo_uint16 portserveur,
			      lcrzo_sock *psock)
{ 
  /*parameters verification*/
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ethaserveur==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  /*initialisations*/
  psock->type=LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;
  lcrzo_er(lcrzo_device_init(device, psock->here_device));
  memcpy(psock->here_etha, ethaserveur, LCRZO_ETHA_MAXBYTES);
  psock->here_ipl=iplserveur;
  psock->here_port=portserveur;
  lcrzo_er(lcrzo_etha_initdefault(psock->there_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->there_ipl)));
  psock->there_port=0;
  lcrzo_er(lcrzo_priv_sock_ipc_init(psock));

  /* sniff process */
  lcrzo_er(lcrzo_priv_sock_fork(psock, &(psock->pidsniff)));
  if (psock->pidsniff==0) /* we are in the child */
  { /* close unnecessary IPC */
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole1)));
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole2)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cioptions)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_read(&(psock->cisniff)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close(&(psock->cispoof)));
    /* sniff function */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_udp_sniffprocess(psock));
    /* end - should not be reatched */
    lcrzo_priv_sock_eechild(psock, LCRZO_ERR_IEINTERNALERROR);
    _exit(LCRZO_ERR_IEINTERNALERROR);
  }

  /* close unnecessary IPC */
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cicontrole1)));
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cicontrole2)));
  lcrzo_er(lcrzo_ipc_close_read(&(psock->cioptions)));
  lcrzo_er(lcrzo_ipc_close_write(&(psock->cisniff)));
  lcrzo_er(lcrzo_ipc_close(&(psock->cispoof)));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_udpmulser_real2(lcrzo_ipl ipl,
			       lcrzo_uint16 port,
			       lcrzo_sock *psock)
{ struct sockaddr_in sain;
  int retour, sock;

  /*creation de la socket*/
  sock=socket(AF_INET, SOCK_DGRAM, 0);
  if (sock<0) return(LCRZO_ERR_FUSOCKET);

  memset(&sain, 0, sizeof(sain));
  sain.sin_family = AF_INET;
  if ( ipl==0 )
    sain.sin_addr.s_addr=lcrzo_htonl(INADDR_ANY);
  else
    sain.sin_addr.s_addr=lcrzo_htonl(ipl);
  sain.sin_port = lcrzo_htons(port);
  retour=bind(sock, (struct sockaddr *)&sain, sizeof(sain));
  if (retour<0) return(LCRZO_ERR_FUBIND);

  /*if the user only want to test, its ok, so we close*/
  if (psock==NULL)
  { retour=close(sock);
    if ( retour == -1 ) return(LCRZO_ERR_FUCLOSE);
    return(LCRZO_ERR_OK);
  }

  /*remplissage des caracteristiques de la socket*/
  psock->type=LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI;
  psock->socknum=sock;
  psock->beblocking=1;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;

  /*remplissage des informations connues et initialisation des autres*/
  lcrzo_er(lcrzo_device_initdefault(psock->here_device));
  lcrzo_er(lcrzo_etha_initdefault(psock->here_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->here_ipl)));
  psock->here_port=port;
  lcrzo_er(lcrzo_etha_initdefault(psock->there_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->there_ipl)));
  psock->there_port=0;
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_sock_tcpser_real(lcrzo_uint16 port,
			  lcrzo_sock *psock)
{ return(lcrzo_sock_tcpser_real2(0, port, NULL, 0, psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpser_virt(const lcrzo_device device,
			  const lcrzo_etha ethaserveur,
			  const lcrzo_ipl iplserveur, 
			  lcrzo_uint16 portserveur,
			  lcrzo_sock *psock)
{ return(lcrzo_sock_tcpser_virt2(device, ethaserveur, iplserveur,
				 portserveur, NULL, 0, psock));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpser_real2(lcrzo_ipl ipl,
			    lcrzo_uint16 port,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock *psock)
{ struct sockaddr_in sain, sain2;
  int retour, soc, sizeof_sain2, un, sock;

  /*parameters verification*/
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);

  /*creation de la socket*/
  soc=socket(AF_INET, SOCK_STREAM, 0);
  if (soc<0) return(LCRZO_ERR_FUSOCKET);

  /*pour que l'adresse puisse etre immediatemment reutilisee*/
  un=1;
  retour=setsockopt(soc, SOL_SOCKET, SO_REUSEADDR, (void *)&un, sizeof(un));
  if (retour) return(LCRZO_ERR_FUSETSOCKOPT);

  /*ajout eventuel des options IP*/
  if (ipoptsize)
  { retour = setsockopt(soc, IPPROTO_IP, IP_OPTIONS, ipopt, ipoptsize);
    if (retour)
      return(LCRZO_ERR_FUSETSOCKOPT);
  }

  /*ecoute sur le port*/
  memset(&sain, 0, sizeof(sain));
  sain.sin_family = AF_INET;
  if ( ipl==0 )
    sain.sin_addr.s_addr=lcrzo_htonl(INADDR_ANY);
  else
    sain.sin_addr.s_addr=lcrzo_htonl(ipl);
  sain.sin_port = lcrzo_htons(port);
  retour=bind(soc, (struct sockaddr *)&sain, sizeof(sain));
  if (retour<0) return(LCRZO_ERR_FUBIND);

  retour=listen(soc, 1); /*maximum 1 connexion en parallele*/
  if (retour<0) return(LCRZO_ERR_FULISTEN);

  sizeof_sain2=sizeof(sain2);
  memset(&sain2, 0, sizeof_sain2);
  sock=accept(soc,  (struct sockaddr *)&sain2, &sizeof_sain2);
  if (sock<0) return(LCRZO_ERR_FUACCEPT);

  retour=close(soc);
  if (retour==-1) return(LCRZO_ERR_FUCLOSE);

  /*ajout eventuel des options IP*/
  if (ipoptsize)
  { retour = setsockopt(sock, IPPROTO_IP, IP_OPTIONS, ipopt, ipoptsize);
    if (retour)
      return(LCRZO_ERR_FUSETSOCKOPT);
  }

  /*if the user only want to test, its ok, so we close*/
  if (psock==NULL)
  { retour=close(sock);
    if ( retour == -1 ) return(LCRZO_ERR_FUCLOSE);
    return(LCRZO_ERR_OK);
  }

  /*remplissage des caracteristiques de la socket*/
  psock->type=LCRZO_PRIV_TYPESOCK_REALSERTCP;
  psock->socknum=sock;
  psock->beblocking=1;
  psock->storeddatasize=0;
  psock->connectionclosed=LCRZO_FALSE;

  /*remplissage des informations connues et initialisation des autres*/
  lcrzo_er(lcrzo_device_initdefault(psock->here_device));
  lcrzo_er(lcrzo_etha_initdefault(psock->here_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->here_ipl)));
  psock->here_port=port;
  lcrzo_er(lcrzo_etha_initdefault(psock->there_etha));
  lcrzo_er(lcrzo_ipl_initdefault(&(psock->there_ipl)));
  psock->there_port=0;
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpser_virt2(const lcrzo_device device,
			    const lcrzo_etha ethaserveur,
			    const lcrzo_ipl iplserveur, 
			    lcrzo_uint16 portserveur,
			    const lcrzo_ipopt ipopt,
			    lcrzo_uint8 ipoptsize,
			    lcrzo_sock *psock)
{ lcrzo_sniff sniff;

  /* parameters verification */
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ethaserveur==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  /* initializations */
  psock->type=LCRZO_PRIV_TYPESOCK_VIRTSERTCP;
  psock->connectionclosed=LCRZO_FALSE;
  psock->storeddatasize=0;
  psock->tcpstate=LCRZO_PRIV_SOCK_TCPSTATE_LISTEN_INIT;
  lcrzo_er(lcrzo_priv_sock_majinfos(device, 
				    ethaserveur, iplserveur, portserveur,
				    NULL, 0, 0,
                                    psock));
  lcrzo_er(lcrzo_priv_sock_ipc_init(psock));
  lcrzo_efr(lcrzo_sock_ipopt_set(psock, ipopt, ipoptsize),
	    lcrzo_priv_sock_ipc_close(psock));

  /* we use one process : we use pidsniff to store it*/
  lcrzo_er(lcrzo_priv_sock_fork(psock, &(psock->pidsniff)));
  if (psock->pidsniff==0) /* we are in the child */
  { /* close unnecessary IPC */
    lcrzo_priv_sock_eechild(psock,lcrzo_ipc_close_read(&(psock->cicontrole1)));
    lcrzo_priv_sock_eechild(psock, 
			    lcrzo_ipc_close_write(&(psock->cicontrole2)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cioptions)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_read(&(psock->cisniff)));
    lcrzo_priv_sock_eechild(psock, lcrzo_ipc_close_write(&(psock->cispoof)));
    /* initialize sniff */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_sniff_init_dev(device, &sniff));
    /* run the state machine */
    lcrzo_priv_sock_eechild(psock, lcrzo_priv_sock_tcpstate(psock, &sniff));
    /* close sniff */
    lcrzo_priv_sock_eechild(psock, lcrzo_sniff_close(&sniff));
    /* end */
    lcrzo_priv_sock_okchild(psock);
    _exit(LCRZO_ERR_OK);
  }
  else
  { /* close unnecessary IPC */
    lcrzo_er(lcrzo_ipc_close_write(&(psock->cicontrole1)));
    lcrzo_er(lcrzo_ipc_close_read(&(psock->cicontrole2)));
    lcrzo_er(lcrzo_ipc_close_read(&(psock->cioptions)));
    lcrzo_er(lcrzo_ipc_close_write(&(psock->cisniff)));
    lcrzo_er(lcrzo_ipc_close_read(&(psock->cispoof)));
  }

  /* wait the end of the handshake */
  lcrzo_er(lcrzo_priv_sock_waitendhandshake(psock));
  /* get infos on the client */
  lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole1), LCRZO_TRUE,
				      psock->there_etha,
				      &(psock->there_ipl),
				      &(psock->there_port)));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpmulser_real(lcrzo_uint16 port,
			      int (*pfonc)(lcrzo_sock sock,
					   void *pinfos),
			      void *pinfos)
{ return(lcrzo_sock_tcpmulser_real2(0, port, NULL, 0, pfonc, pinfos));
}

/*-------------------------------------------------------------*/
int lcrzo_sock_tcpmulser_real2(lcrzo_ipl ipl,
			       lcrzo_uint16 port,
			       const lcrzo_ipopt ipopt,
			       lcrzo_uint8 ipoptsize,
			       int (*pfonc)(lcrzo_sock sock,
					    void *pinfos),
			       void *pinfos)
{ struct sockaddr_in sain, sain2;
  int retour, sockglo, sizeof_sain2, un, pid;
  lcrzo_sock sock;

  /*parameters verification*/
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (pfonc==NULL) return(LCRZO_ERR_PANULLPTR);

  /*creation de la socket*/
  sockglo=socket(AF_INET, SOCK_STREAM, 0);
  if (sockglo<0) return(LCRZO_ERR_FUSOCKET);

  /*pour que l'adresse puisse etre immediatemment reutilisee*/
  un=1;
  retour=setsockopt(sockglo, SOL_SOCKET, SO_REUSEADDR,
		    (void *)&un, sizeof(un));
  if (retour) return(LCRZO_ERR_FUSETSOCKOPT);

  /*ajout eventuel des options IP*/
  if (ipoptsize)
  { retour = setsockopt(sockglo, IPPROTO_IP, IP_OPTIONS, ipopt, ipoptsize);
    if (retour)
      return(LCRZO_ERR_FUSETSOCKOPT);
  }

  /*ecoute sur le port*/
  memset(&sain, 0, sizeof(sain));
  sain.sin_family = AF_INET;
  if ( ipl==0 )
    sain.sin_addr.s_addr=lcrzo_htonl(INADDR_ANY);
  else
    sain.sin_addr.s_addr=lcrzo_htonl(ipl);
  sain.sin_port = lcrzo_htons(port);
  retour=bind(sockglo, (struct sockaddr *)&sain, sizeof(sain));
  if (retour<0) return(LCRZO_ERR_FUBIND);

  retour=listen(sockglo, lcrzo_global.sock_tcpser_backlog);
  if (retour<0) return(LCRZO_ERR_FULISTEN);

  /*on ecoute indefiniment*/
  while (1)
  { sizeof_sain2=sizeof(sain2);
    memset(&sain2, 0, sizeof_sain2);
    sock.socknum=accept(sockglo, (struct sockaddr *)&sain2, &sizeof_sain2);
    if (sock.socknum<0) return(LCRZO_ERR_FUACCEPT);
    pid=fork();
    if (pid<0) return(LCRZO_ERR_FUFORK);
    if (pid==0) /*on est dans le fils*/
    { /*on ferme la socket du pere*/
      retour=close(sockglo);
      if (retour==-1) return(LCRZO_ERR_FUCLOSE);
      /*ajout eventuel des options IP*/
      if (ipoptsize)
      { retour = setsockopt(sock.socknum, IPPROTO_IP, IP_OPTIONS,
			    ipopt, ipoptsize);
        if (retour) return(LCRZO_ERR_FUSETSOCKOPT);
      }
      /*remplissage des caracteristiques de la socket*/
      sock.type=LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI;
      sock.storeddatasize=0;
      sock.beblocking=1;
      sock.connectionclosed=LCRZO_FALSE;
      /*remplissage des informations connues et initialisation des autres*/
      lcrzo_er(lcrzo_device_initdefault(sock.here_device));
      lcrzo_er(lcrzo_etha_initdefault(sock.here_etha));
      lcrzo_er(lcrzo_ipl_initdefault(&sock.here_ipl));
      sock.here_port=port;
      lcrzo_er(lcrzo_etha_initdefault(sock.there_etha));
      lcrzo_er(lcrzo_ipl_initdefault(&sock.there_ipl));
      sock.there_port=0;
      /*utilisation de sock*/
      retour=(*pfonc)(sock, pinfos);
      _exit(retour);
    }
    else /*on est dans le pere*/
    { retour=close(sock.socknum);
      if (retour==-1) return(LCRZO_ERR_FUCLOSE);
    }
  }
  /*jamais atteint, mais pour faire plaisir au compilateur*/
  return(lcrzo_ierp("lcrzo_sock_tcpmulser_real2"));
}


/*-------------------------------------------------------------*/
int lcrzo_sock_write(lcrzo_sock *psock,
		     lcrzo_constdata data, 
		     lcrzo_int32 datasize)
{ 
  /*parameters verification*/
  if (data==NULL && datasize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (datasize==0) return(LCRZO_ERR_OK);

  /* error cases */
  switch(psock->type)
  { case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      return(LCRZO_ERR_BUUDPSERMULTINOWRITE);
      break;
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if (psock->canwrite==0)
	return(LCRZO_ERR_BUUDPSERREADBEFWRITE);
      break;
  }

  /* normal cases */
  switch(psock->type)
  { case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      return(lcrzo_fd_write(psock->socknum, data, datasize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      /* look if we can still write */
      lcrzo_er(lcrzo_priv_sock_analyzeexitchild(psock, LCRZO_FALSE));
      if ( psock->connectionclosed ) return(LCRZO_ERR_BUWRITECLOSED);
      /* write */
      return(lcrzo_ipc_write(&(psock->cispoof), data, datasize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_write", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_write_msg(lcrzo_sock *psock,
			 lcrzo_constdata data, 
			 lcrzo_int32 datasize)
{ 
  /*parameters verification*/
  if (data==NULL && datasize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (datasize==0) return(LCRZO_ERR_OK);

  /* error cases */
  switch(psock->type)
  { case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      return(LCRZO_ERR_BUUDPSERMULTINOWRITE);
      break;
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if (psock->canwrite==0)
	return(LCRZO_ERR_BUUDPSERREADBEFWRITE);
      break;
  }

  /* normal cases */
  switch(psock->type)
  { case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      return(lcrzo_fd_write_msg(psock->socknum, data, datasize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      /* look if we can still write */
      lcrzo_er(lcrzo_priv_sock_analyzeexitchild(psock, LCRZO_FALSE));
      if ( psock->connectionclosed ) return(LCRZO_ERR_BUWRITECLOSED);
      return(lcrzo_ipc_write_msg(&(psock->cispoof), data, datasize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_write_msg", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_read(lcrzo_sock *psock,
		    lcrzo_bool beblocking,
		    lcrzo_int32 dataoutmaxsize,
		    lcrzo_data dataout,
		    lcrzo_int32 *pdataoutsize)
{         
  /*verification des parametres*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);

  /*lecture normale*/
  switch(psock->type)
  {
    case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_read(LCRZO_PRIV_MULTI_READ_SOCKR, psock,
				     beblocking, dataoutmaxsize,
				     dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      lcrzo_er(lcrzo_fd_read(psock->socknum, beblocking,
			     dataoutmaxsize, dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      lcrzo_er(lcrzo_ipc_read(&(psock->cisniff), beblocking,
			      dataoutmaxsize, dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if ( ! psock->canwrite )
      { /* the first time, we need to know the client */
        lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), beblocking,
					    psock->there_etha,
					    &(psock->there_ipl),
					    &(psock->there_port)));
        psock->canwrite=1;
      } 
      lcrzo_er(lcrzo_ipc_read(&(psock->cisniff), beblocking,
			      dataoutmaxsize, dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_read(LCRZO_PRIV_MULTI_READ_SOCKVSUM, psock,
				     beblocking, dataoutmaxsize,
				     dataout, pdataoutsize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_read", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_readm(lcrzo_sock *psock,
		     lcrzo_bool beblocking,
		     lcrzo_data *pdataout,
		     lcrzo_int32 *pdataoutsize)
{         
  /*verification des parametres*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);
 
  /*lecture normale*/
  switch(psock->type)
  {
    case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_readm(LCRZO_PRIV_MULTI_READ_SOCKR, psock,
				      beblocking, pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      lcrzo_er(lcrzo_fd_readm(psock->socknum, beblocking,
			      pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      lcrzo_er(lcrzo_ipc_readm(&(psock->cisniff), beblocking,
			       pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if ( ! psock->canwrite )
      { /* the first time, we need to know the client */
        lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), beblocking,
					    psock->there_etha,
					    &(psock->there_ipl),
					    &(psock->there_port)));
        psock->canwrite=1;
      } 
      lcrzo_er(lcrzo_ipc_readm(&(psock->cisniff), beblocking,
			       pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_readm(LCRZO_PRIV_MULTI_READ_SOCKVSUM, psock,
				      beblocking, pdataout, pdataoutsize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_readm", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_read_fixed(lcrzo_sock *psock,
			  lcrzo_bool beblocking,
			  lcrzo_int32 datasizetoread,
			  lcrzo_data dataout)
{         
  /*verification des parametres*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);

  /*lecture normale*/
  switch(psock->type)
  {
    case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_read_fixed(LCRZO_PRIV_MULTI_READ_SOCKR, psock,
					   beblocking, datasizetoread, 
					   dataout));
      break;
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      lcrzo_er(lcrzo_fd_read_fixed(psock->socknum, beblocking,
				   datasizetoread, dataout));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      lcrzo_er(lcrzo_ipc_read_fixed(&(psock->cisniff), beblocking,
				    datasizetoread, dataout));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if ( ! psock->canwrite )
      { /* the first time, we need to know the client */
        lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), beblocking,
					    psock->there_etha,
					    &(psock->there_ipl),
					    &(psock->there_port)));
        psock->canwrite=1;
      } 
      lcrzo_er(lcrzo_ipc_read_fixed(&(psock->cisniff), beblocking,
				    datasizetoread, dataout));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_read_fixed(LCRZO_PRIV_MULTI_READ_SOCKVSUM, 
					   psock, beblocking, 
					   datasizetoread, dataout));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_read_fixed", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_readm_fixed(lcrzo_sock *psock,
			   lcrzo_bool beblocking,
			   lcrzo_int32 datasizetoread,
			   lcrzo_data *pdataout)
{         
  /*verification des parametres*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);

  /*lecture normale*/
  switch(psock->type)
  {
    case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_readm_fixed(LCRZO_PRIV_MULTI_READ_SOCKR, psock,
					    beblocking, datasizetoread, 
					    pdataout));
      break;
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      lcrzo_er(lcrzo_fd_readm_fixed(psock->socknum, beblocking,
				    datasizetoread, pdataout));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      lcrzo_er(lcrzo_ipc_readm_fixed(&(psock->cisniff), beblocking,
				     datasizetoread, pdataout));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if ( ! psock->canwrite )
      { /* the first time, we need to know the client */
        lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), beblocking,
					    psock->there_etha,
					    &(psock->there_ipl),
					    &(psock->there_port)));
        psock->canwrite=1;
      } 
      lcrzo_er(lcrzo_ipc_readm_fixed(&(psock->cisniff), beblocking,
				     datasizetoread, pdataout));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_readm_fixed(LCRZO_PRIV_MULTI_READ_SOCKVSUM,
					    psock, beblocking, datasizetoread,
					    pdataout));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_readm_fixed", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_read_line(lcrzo_sock *psock,
			 lcrzo_bool beblocking,
			 lcrzo_int32 dataoutmaxsize,
			 lcrzo_data dataout,
			 lcrzo_int32 *pdataoutsize)
{         
  /*verification des parametres*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);

  /*lecture normale*/
  switch(psock->type)
  {
    case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_read_line(LCRZO_PRIV_MULTI_READ_SOCKR, psock,
					  beblocking, dataoutmaxsize, 
					  dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      lcrzo_er(lcrzo_fd_read_line(psock->socknum, beblocking,
				  dataoutmaxsize, dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      lcrzo_er(lcrzo_ipc_read_line(&(psock->cisniff), beblocking,
				   dataoutmaxsize, dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if ( ! psock->canwrite )
      { /* the first time, we need to know the client */
        lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), beblocking,
					    psock->there_etha,
					    &(psock->there_ipl),
					    &(psock->there_port)));
        psock->canwrite=1;
      } 
      lcrzo_er(lcrzo_ipc_read_line(&(psock->cisniff), beblocking,
				   dataoutmaxsize, dataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_read_line(LCRZO_PRIV_MULTI_READ_SOCKVSUM, 
					  psock, beblocking, dataoutmaxsize,
					  dataout, pdataoutsize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_read_line", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_readm_line(lcrzo_sock *psock,
			  lcrzo_bool beblocking,
			  lcrzo_data *pdataout,
			  lcrzo_int32 *pdataoutsize)
{         
  /*verification des parametres*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  lcrzo_bool_verifbof(beblocking);

  /*lecture normale*/
  switch(psock->type)
  {
    case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_readm_line(LCRZO_PRIV_MULTI_READ_SOCKR, psock,
					   beblocking, pdataout,
					   pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      lcrzo_er(lcrzo_fd_readm_line(psock->socknum, beblocking,
				   pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      lcrzo_er(lcrzo_ipc_readm_line(&(psock->cisniff), beblocking,
				    pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      if ( ! psock->canwrite )
      { /* the first time, we need to know the client */
        lcrzo_er(lcrzo_ipc_read_ethaiplport(&(psock->cicontrole2), beblocking,
					    psock->there_etha,
					    &(psock->there_ipl),
					    &(psock->there_port)));
        psock->canwrite=1;
      } 
      lcrzo_er(lcrzo_ipc_readm_line(&(psock->cisniff), beblocking,
				    pdataout, pdataoutsize));
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      lcrzo_er(lcrzo_priv_multi_readm_line(LCRZO_PRIV_MULTI_READ_SOCKVSUM,
					   psock, beblocking,
					   pdataout, pdataoutsize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_readm_line", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_close(lcrzo_sock *psock)
{ int ret;

  /* free stored data */
  if (psock->storeddatasize)
  { lcrzo_er(lcrzo_data_free(psock->storeddata));
    psock->storeddatasize=0;
  }
  
  /* close */
  switch(psock->type)
  { case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      ret=close(psock->socknum);
      if (ret==-1) return(LCRZO_ERR_FUCLOSE);
      return(LCRZO_ERR_OK);
      break;

    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      /* check if the connexion has just closed */
      if ( ! psock->connectionclosed ) {
	lcrzo_er(lcrzo_priv_sock_analyzeexitchild(psock, LCRZO_FALSE));
      }
      /* if connexion is still not closed */
      if ( ! psock->connectionclosed ) {
	/* inform child we want to stop */
	lcrzo_er(lcrzo_ipc_write_uint8(&(psock->cicontrole2), 1));
	/* wait till the end of the child */
	lcrzo_er(lcrzo_priv_sock_analyzeexitchild(psock, LCRZO_TRUE));
      }
      /* the child has now ended */
      /* kill and wait for the child */
      lcrzo_er(lcrzo_priv_killwait(psock->pidsniff));
      /* close IPC */
      lcrzo_er(lcrzo_priv_sock_ipc_close(psock));
      errno=0; /* because Solaris set it */
      return(LCRZO_ERR_OK);
      break;

    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
      /* kill the sniff process */
      lcrzo_er(lcrzo_priv_killwait(psock->pidsniff));
      /* close the IPC of the spoof process : force it to terminate
	 when it will have read everything */
      lcrzo_er(lcrzo_ipc_close(&(psock->cispoof)));
      /* wait till the end of the spoof process */
      lcrzo_er(lcrzo_priv_sock_analyzeexitchild(psock, LCRZO_TRUE));
      /* kill the spoof process */
      lcrzo_er(lcrzo_priv_killwait(psock->pidspoof));
      /* close IPC */
      lcrzo_er(lcrzo_priv_sock_ipc_close(psock));
      /* end */
      errno=0; /* because Solaris set it */
      return(LCRZO_ERR_OK);
      break;

    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      /* kill the sniff process */
      lcrzo_er(lcrzo_priv_killwait(psock->pidsniff));
      /* close IPC */
      lcrzo_er(lcrzo_priv_sock_ipc_close(psock));
      /* end */
      errno=0; /* because Solaris set it */
      return(LCRZO_ERR_OK);
      break;

    default:
      return(lcrzo_ieuv("lcrzo_sock_close", "psock->type"));
  }

  return(lcrzo_ierp("lcrzo_sock_close"));
}

/*---------------------------------------------------------------*/
int lcrzo_sock_get_infos(lcrzo_sock *psock,
			lcrzo_device here_device,
			lcrzo_etha here_etha,
			lcrzo_ipl *phere_ipl,
			lcrzo_uint16 *phere_port,
			lcrzo_etha there_etha,
			lcrzo_ipl *pthere_ipl,
			lcrzo_uint16 *pthere_port)
{ int retour, taillesa;
  struct sockaddr sa;
  struct sockaddr_in sai;

  switch(psock->type)
  { case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      if (here_device!=NULL)
	lcrzo_er(lcrzo_device_initdefault(here_device));
      if (here_etha!=NULL)
	lcrzo_er(lcrzo_etha_initdefault(here_etha));
      if (there_etha!=NULL)
	lcrzo_er(lcrzo_etha_initdefault(there_etha));
      taillesa=sizeof(struct sockaddr);
      retour=getsockname(psock->socknum, &sa, &taillesa);
      if (retour==-1) return(LCRZO_ERR_FUGETSOCKNAME);
      memcpy(&sai, &sa, sizeof(struct sockaddr_in));
      if (phere_ipl!=NULL)
	*phere_ipl=lcrzo_ntohl(sai.sin_addr.s_addr);
      if (phere_port!=NULL)
	*phere_port=lcrzo_ntohs(sai.sin_port);
      taillesa=sizeof(struct sockaddr);
      retour=getpeername(psock->socknum, &sa, &taillesa);
      if (retour==-1)
      { if (errno!=ENOTCONN) return(LCRZO_ERR_FUGETPEERNAME);
        /*personne n'est connecte*/
        errno=0;
        if (pthere_ipl!=NULL)
	  lcrzo_er(lcrzo_ipl_initdefault(pthere_ipl));
	if (pthere_port!=NULL)
	  *pthere_port=0;
      }
      else
      { memcpy(&sai, &sa, sizeof(struct sockaddr_in));
        if (pthere_ipl!=NULL)
	  *pthere_ipl=lcrzo_ntohl(sai.sin_addr.s_addr);
	if (pthere_port!=NULL)
	  *pthere_port=lcrzo_ntohs(sai.sin_port);
      }
      break;
    case LCRZO_PRIV_TYPESOCK_REALSERUDPMULTI:
      taillesa=sizeof(struct sockaddr);
      retour=getsockname(psock->socknum, &sa, &taillesa);
      if (retour==-1) return(LCRZO_ERR_FUGETSOCKNAME);
      memcpy(&sai, &sa, sizeof(struct sockaddr_in));
      if (phere_ipl!=NULL)
	*phere_ipl=lcrzo_ntohl(sai.sin_addr.s_addr);
      if (phere_port!=NULL)
	*phere_port=lcrzo_ntohs(sai.sin_port);
      if (here_device!=NULL)
	memcpy(here_device, psock->here_device, LCRZO_DEVICE_MAXBYTES);
      if (here_etha!=NULL)
	memcpy(here_etha, psock->here_etha, LCRZO_ETHA_MAXBYTES);
      if (there_etha!=NULL)
	memcpy(there_etha, psock->there_etha, LCRZO_ETHA_MAXBYTES);
      if (pthere_ipl!=NULL)
	*pthere_ipl=psock->there_ipl;
      if (pthere_port!=NULL)
	*pthere_port=psock->there_port;
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDPMULTI:
      if (here_device!=NULL)
	memcpy(here_device, psock->here_device, LCRZO_DEVICE_MAXBYTES);
      if (here_etha!=NULL)
	memcpy(here_etha, psock->here_etha, LCRZO_ETHA_MAXBYTES);
      if (phere_ipl!=NULL)
	*phere_ipl=psock->here_ipl;
      if (phere_port!=NULL)
	*phere_port=psock->here_port;
      if (there_etha!=NULL)
	memcpy(there_etha, psock->there_etha, LCRZO_ETHA_MAXBYTES);
      if (pthere_ipl!=NULL)
	*pthere_ipl=psock->there_ipl;
      if (pthere_port!=NULL)
	*pthere_port=psock->there_port;
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_get_infos", "psock->type"));
  }
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_fprint_infos(LCRZOFILE *pf,
			    lcrzo_sock *psock)
{ lcrzo_device here_device;
  lcrzo_etha here_etha;
  lcrzo_ipl here_ipl;
  lcrzo_uint16 here_port;
  lcrzo_etha there_etha;
  lcrzo_ipl there_ipl;
  lcrzo_uint16 there_port;

  /*parameters verification*/
  if (pf==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_sock_get_infos(psock, here_device, here_etha, &here_ipl, 
				&here_port,  there_etha, &there_ipl,
				&there_port));
  lcrzo_er(lcrzo_device_fprint(pf, "Local : device=", here_device, ", "));
  lcrzo_er(lcrzo_etha_fprint(pf, "eth=", here_etha, ", "));
  lcrzo_er(lcrzo_ipl_fprint(pf, "ip=", here_ipl, ", "));
  fprintf(pf, "port=%d\n", here_port);
  lcrzo_er(lcrzo_etha_fprint(pf, "Distant : eth=", there_etha, ", "));
  lcrzo_er(lcrzo_ipl_fprint(pf, "ip=", there_ipl, ", "));
  fprintf(pf, "port=%d\n", there_port);
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_ipopt_set(lcrzo_sock *psock,
			 const lcrzo_ipopt ipopt,
			 lcrzo_uint8 ipoptsize)
{ int ret;

  /* check parameters */
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);
  if (ipopt==NULL && ipoptsize) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (ipoptsize>LCRZO_IPOPT_MAXBYTES)
    return(LCRZO_ERR_SPOPTMAX10);

  /* set structure */
  psock->ipoptsize=ipoptsize;
  memcpy(psock->ipopt, ipopt, ipoptsize);
  
  /* inform socket */
  switch(psock->type)
  { case LCRZO_PRIV_TYPESOCK_REALCLIUDP:
    case LCRZO_PRIV_TYPESOCK_REALCLITCP:
    case LCRZO_PRIV_TYPESOCK_REALSERUDP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCP:
    case LCRZO_PRIV_TYPESOCK_REALSERTCPMULTI:
      ret = setsockopt(psock->socknum, IPPROTO_IP,IP_OPTIONS, ipopt,ipoptsize);
      if (ret) return(LCRZO_ERR_FUSETSOCKOPT);
      break;
    case LCRZO_PRIV_TYPESOCK_VIRTCLIUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTCLITCP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERUDP:
    case LCRZO_PRIV_TYPESOCK_VIRTSERTCP:
      lcrzo_er(lcrzo_ipc_write_msg(&(psock->cioptions), ipopt, ipoptsize));
      break;
    default:
      return(lcrzo_ieuv("lcrzo_sock_ipopt_set", "psock->type"));
  } 
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_sock_ipopt_get(lcrzo_sock *psock,
			 lcrzo_ipopt ipopt,
			 lcrzo_uint8 *pipoptsize)
{
  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  if (pipoptsize!=NULL) *pipoptsize=psock->ipoptsize;
  if (ipopt!=NULL) memcpy(ipopt, psock->ipopt, psock->ipoptsize);
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_ipopt_noop(lcrzo_sock *psock)
{ lcrzo_ipopt opt;
  lcrzo_uint8 ipoptsize;

  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_ipopt_init_noop(opt, &ipoptsize));
  lcrzo_er(lcrzo_sock_ipopt_set(psock, opt, ipoptsize));
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_ipopt_lsrr(lcrzo_sock *psock,
			 lcrzo_ipl ipl1,
			 lcrzo_ipl ipl2,
			 lcrzo_ipl ipl3,
			 lcrzo_ipl ipl4,
			 lcrzo_ipl ipl5,
			 lcrzo_ipl ipl6,
			 lcrzo_ipl ipl7,
			 lcrzo_ipl ipl8,
			 lcrzo_ipl ipl9,
			 lcrzo_int8 nbiplaffectes)
{ lcrzo_ipopt opt;
  lcrzo_uint8 ipoptsize;

  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_ipopt_init_lsrr(ipl1, ipl2, ipl3, ipl4, ipl5, ipl6,
				ipl7, ipl8, ipl9, nbiplaffectes,
				opt, &ipoptsize));
  lcrzo_er(lcrzo_sock_ipopt_set(psock, opt, ipoptsize));
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_ipopt_ssrr(lcrzo_sock *psock,
			 lcrzo_ipl ipl1,
			 lcrzo_ipl ipl2,
			 lcrzo_ipl ipl3,
			 lcrzo_ipl ipl4,
			 lcrzo_ipl ipl5,
			 lcrzo_ipl ipl6,
			 lcrzo_ipl ipl7,
			 lcrzo_ipl ipl8,
			 lcrzo_ipl ipl9,
			 lcrzo_int8 nbiplaffectes)
{ lcrzo_ipopt opt;
  lcrzo_uint8 ipoptsize;

  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_ipopt_init_ssrr(ipl1, ipl2, ipl3, ipl4, ipl5, ipl6,
				ipl7, ipl8, ipl9, nbiplaffectes,
				opt, &ipoptsize));
  lcrzo_er(lcrzo_sock_ipopt_set(psock, opt, ipoptsize));
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_ipopt_rr(lcrzo_sock *psock,
		       lcrzo_int8 nbroutesaenregistrer)
{ lcrzo_ipopt opt;
  lcrzo_uint8 ipoptsize;

  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_ipopt_init_rr(nbroutesaenregistrer, opt, &ipoptsize));
  lcrzo_er(lcrzo_sock_ipopt_set(psock, opt, ipoptsize));
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_ipopt_time(lcrzo_sock *psock,
			 lcrzo_int8 nbtimestampsaenregistrer)
{ lcrzo_ipopt opt;
  lcrzo_uint8 ipoptsize;

  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_ipopt_init_time(nbtimestampsaenregistrer, 
				 opt, &ipoptsize));
  lcrzo_er(lcrzo_sock_ipopt_set(psock, opt, ipoptsize));
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_ipopt_timeip(lcrzo_sock *psock,
			   lcrzo_int8 nbtimestampsaenregistrer)
{ lcrzo_ipopt opt;
  lcrzo_uint8 ipoptsize;

  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_ipopt_init_timeip(nbtimestampsaenregistrer, 
				   opt, &ipoptsize));
  lcrzo_er(lcrzo_sock_ipopt_set(psock, opt, ipoptsize));
  return(LCRZO_ERR_OK);
}

/*---------------------------------------------------------------*/
int lcrzo_sock_ipopt_timeipp(lcrzo_sock *psock,
			    lcrzo_ipl ipl1,
			    lcrzo_ipl ipl2,
			    lcrzo_ipl ipl3,
			    lcrzo_ipl ipl4,
			    lcrzo_int8 nbiplaffectes)
{ lcrzo_ipopt opt;
  lcrzo_uint8 ipoptsize;

  /*parameters verification*/
  if (psock==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_ipopt_init_timeipp(ipl1, ipl2, ipl3, ipl4, nbiplaffectes,
				    opt, &ipoptsize));
  lcrzo_er(lcrzo_sock_ipopt_set(psock, opt, ipoptsize));
  return(LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int lcrzo_virtual_answer_arp(lcrzo_constdata packet,
			    lcrzo_uint32 nboct,
			    const lcrzo_device device,
			    const lcrzo_etha etha,
			    const lcrzo_ipl ipl)
{ int retour;
  lcrzo_hdrleth hdrlethrecu, hdrlethenvoye;
  lcrzo_hdrlarp hdrlarprecu, hdrlarpenvoye;
  lcrzo_ipa ipa;
  lcrzo_spoof spoof;

  /*parameters verification*/
  if (packet==NULL && nboct) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (etha==NULL) return(LCRZO_ERR_PANULLPTR);

  /*tentative de decodage ARP*/
  retour=lcrzo_packet_decode_etharp(packet, nboct,
				   &hdrlethrecu, &hdrlarprecu);
  if (retour!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);

  /*on n'est interesse que par les requetes arp*/
  if ( hdrlethrecu.type != LCRZO_HDRLETH_TYPE_ARP ) 
    return(LCRZO_ERR_OK);
  if ( hdrlarprecu.op != 1 ) return(LCRZO_ERR_OK);
  
  /*on n'est interesse que par les requetes de ipa*/
  lcrzo_er(lcrzo_ipa_init_ipl(ipl, ipa));
  if (!lcrzo_ipa_equal(hdrlarprecu.prot_dst, ipa)) return(LCRZO_ERR_OK);

  /*cree le nouveau entete ethernet*/
  lcrzo_er(lcrzo_hdrleth_initdefault(&hdrlethenvoye));
  memcpy(hdrlethenvoye.src, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrlethenvoye.dst, hdrlarprecu.hw_src, LCRZO_ETHA_MAXBYTES);
  hdrlethenvoye.type=LCRZO_HDRLETH_TYPE_ARP;
    
  /*cree le nouveau entete arp*/
  lcrzo_er(lcrzo_hdrlarp_initdefault(&hdrlarpenvoye));
  hdrlarpenvoye.op=2;
  memcpy(hdrlarpenvoye.hw_src, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrlarpenvoye.hw_dst, hdrlarprecu.hw_src, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrlarpenvoye.prot_src, hdrlarprecu.prot_dst, LCRZO_IPA_MAXBYTES);
  memcpy(hdrlarpenvoye.prot_dst, hdrlarprecu.prot_src, LCRZO_IPA_MAXBYTES);
    
  /*envoi du packet*/
  lcrzo_er(lcrzo_spoof_init(&spoof));
  lcrzo_er(lcrzo_spoof_etharp(&spoof, device, hdrlethenvoye,
			      hdrlarpenvoye));
  lcrzo_er(lcrzo_spoof_close(&spoof));
  
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_virtual_answer_rarp(lcrzo_constdata packet,
			     lcrzo_uint32 nboct,
			     const lcrzo_device device,
			     const lcrzo_etha etha,
			     const lcrzo_ipl ipl)
{ int retour;
  lcrzo_hdrleth hdrlethrecu, hdrlethenvoye;
  lcrzo_hdrlarp hdrlarprecu, hdrlarpenvoye;
  lcrzo_ipa ipa;
  lcrzo_spoof spoof;

  /*parameters verification*/
  if (packet==NULL && nboct) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (etha==NULL) return(LCRZO_ERR_PANULLPTR);

  /*tentative de decodage ARP*/
  retour=lcrzo_packet_decode_etharp(packet, nboct,
				   &hdrlethrecu, &hdrlarprecu);
  if (retour!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);

  /*on n'est interesse que par les requetes rarp*/
  if ( hdrlethrecu.type != LCRZO_HDRLETH_TYPE_RARP )
    return(LCRZO_ERR_OK);
  if ( hdrlarprecu.op != 3 ) return(LCRZO_ERR_OK);
  
  /*on n'est interesse que par les requetes de etha*/
  if (!lcrzo_etha_equal(hdrlarprecu.hw_dst, etha)) return(LCRZO_ERR_OK);

  /*cree le nouveau entete ethernet*/
  lcrzo_er(lcrzo_hdrleth_initdefault(&hdrlethenvoye));
  memcpy(hdrlethenvoye.src, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrlethenvoye.dst, hdrlarprecu.hw_src, LCRZO_ETHA_MAXBYTES);
  hdrlethenvoye.type=LCRZO_HDRLETH_TYPE_RARP;
    
  /*cree le nouveau entete arp*/
  lcrzo_er(lcrzo_ipa_init_ipl(ipl, ipa));
  lcrzo_er(lcrzo_hdrlarp_initdefault(&hdrlarpenvoye));
  hdrlarpenvoye.op=4;
  memcpy(hdrlarpenvoye.hw_src, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrlarpenvoye.hw_dst, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrlarpenvoye.prot_src, ipa, LCRZO_IPA_MAXBYTES);
  memcpy(hdrlarpenvoye.prot_dst, ipa, LCRZO_IPA_MAXBYTES);
    
  /*envoi du packet*/
  lcrzo_er(lcrzo_spoof_init(&spoof));
  lcrzo_er(lcrzo_spoof_etharp(&spoof, device, hdrlethenvoye, hdrlarpenvoye));
  lcrzo_er(lcrzo_spoof_close(&spoof));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_virtual_answer_ping(lcrzo_constdata packet,
			     lcrzo_uint32 nboct,
			     const lcrzo_device device,
			     const lcrzo_etha etha,
			     const lcrzo_ipl ipl)
{ lcrzo_hdrleth hdrlethrecu, hdrlethenvoye;
  lcrzo_hdrlip  hdrliprecu, hdrlipenvoye;
  lcrzo_hdrlicmp hdrlicmprecu, hdrlicmpenvoye;
  lcrzo_data data;
  lcrzo_ipopt opt;
  lcrzo_uint16 nboctdata;
  lcrzo_uint8 nboctopt;
  lcrzo_spoof spoof;
  int retour;

  /*parameters verification*/
  if (packet==NULL && nboct) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (etha==NULL) return(LCRZO_ERR_PANULLPTR);

  /*tentative de decodage ICMP*/
  retour=lcrzo_packet_decodem_ethipopticmpdata(packet, nboct,
					       &hdrlethrecu, &hdrliprecu,
					       opt, &nboctopt, &hdrlicmprecu,
					       &data, &nboctdata);
  if (retour!=LCRZO_ERR_OK) return(LCRZO_ERR_OK);

  /*on n'est interesse que par les echo request*/
  if ( hdrlicmprecu.type != 8 ) 
  { lcrzo_data_free(data);
    return(LCRZO_ERR_OK); 
  }
  
  /*on n'est interesse que par les requetes de etha et ipa*/
  if (!lcrzo_etha_equal(hdrlethrecu.dst, etha)) 
  { lcrzo_data_free(data);
    return(LCRZO_ERR_OK);
  }
  if (!lcrzo_ipl_equal(hdrliprecu.daddr, ipl))
  { lcrzo_data_free(data); 
    return(LCRZO_ERR_OK);
  }

  /*cree le nouveau entete ethernet*/
  lcrzo_efr(lcrzo_hdrleth_initdefault(&hdrlethenvoye), lcrzo_data_free(data));
  memcpy(hdrlethenvoye.src, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(hdrlethenvoye.dst, hdrlethrecu.src, LCRZO_ETHA_MAXBYTES);
    
  /*cree le nouveau entete ip*/
  lcrzo_efr(lcrzo_hdrlip_initdefault(&hdrlipenvoye), lcrzo_data_free(data));
  hdrlipenvoye.saddr=hdrliprecu.daddr;
  hdrlipenvoye.daddr=hdrliprecu.saddr;

  /*cree le nouveau entete icmp*/
  lcrzo_efr(lcrzo_hdrlicmp_initdefault(&hdrlicmpenvoye),
	    lcrzo_data_free(data));
  hdrlicmpenvoye.type=0;

  /*envoi du packet*/
  lcrzo_efr(lcrzo_spoof_init(&spoof), lcrzo_data_free(data));
  lcrzo_efr(lcrzo_spoof_ethipopticmpdata(&spoof, device,
					 hdrlethenvoye,
					 hdrlipenvoye, opt, nboctopt,
					 hdrlicmpenvoye, 
					 data, nboctdata), 
	    lcrzo_data_free(data));
  lcrzo_data_free(data);
  lcrzo_er(lcrzo_spoof_close(&spoof));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzo_virtual_answer_alive(lcrzo_constdata packet,
				lcrzo_uint32 nboct,
				const lcrzo_device device,
				const lcrzo_etha etha,
				const lcrzo_ipl ipl)
{ 
  /*parameters verification*/
  if (packet==NULL && nboct) return(LCRZO_ERR_SPNULLPTRSIZE);
  if (device==NULL) return(LCRZO_ERR_PANULLPTR);
  if (etha==NULL) return(LCRZO_ERR_PANULLPTR);

  lcrzo_er(lcrzo_virtual_answer_arp(packet, nboct,
				    device, etha, ipl));
  lcrzo_er(lcrzo_virtual_answer_rarp(packet, nboct,
				     device, etha, ipl));
  lcrzo_er(lcrzo_virtual_answer_ping(packet, nboct,
				     device, etha, ipl));
  return(LCRZO_ERR_OK);
}
