
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_http.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000097=
{ /*reference*/     "lcrzoex", 97,
  /*version*/       1, 0,
  /*title*/         "real tcp client [web head] (local ip/port and IP option)",
  /*french title*/  "client tcp reel [web head] (ip/port local et option IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "client_name server_name client_port server_port url [username [passwd]]",
  /*french usage*/  "nom_client nom_serveur port_client port_serveur url [utilisateur [motdepasse]]",
  /*usage example*/ "1.2.3.5 1.2.3.4 5555 4444 /index.html joe hello",
  /*fr. usage ex.*/ "1.2.3.5 1.2.3.4 5555 4444 /index.html laurent bonjour",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000097_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000097_main(int argc, char *argv[])
{ lcrzo_ipl serveripl, clientipl;
  lcrzo_uint16 serverport, clientport;
  lcrzo_sock sock;
  lcrzo_ipopt ipopt;
  lcrzo_int8 ipoptsize;
  lcrzo_string user, passwd, file;
  lcrzo_int16 errornumber;
  lcrzo_string errormsg, servertype, contenttype;
  lcrzo_data header;
  lcrzo_int32 headersize, contentlength;
  int language;

  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000097, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* initialize the needed variables */
  lcrzo_epr(lcrzo_ipl_init_hs(argv[1], &clientipl));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[2], &serveripl));
  clientport=(lcrzo_uint16)atoi(argv[3]);
  serverport=(lcrzo_uint16)atoi(argv[4]);
  lcrzo_epr(lcrzo_string_initm_text(argv[5], &file));
  if ( argc<7 )
  { user=NULL;
    passwd=NULL;
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text(argv[6], &user));
    if ( argc<8 )
    { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &passwd));

    }
    else
    { lcrzo_epr(lcrzo_string_initm_text(argv[7], &passwd));
    }
  }
  lcrzo_global_get_language(&language);
  lcrzo_epr(lcrzo_ipopt_stdin((language==LCRZO_GLOBAL_FRLANG)?
                              "options IP":"IP options", "",
                              ipopt, &ipoptsize));

  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real2(clientipl, serveripl,
				    clientport, serverport,
				    ipopt, ipoptsize,
				    &sock));
    
  /* send a request */
  lcrzo_epr(shared_http_request_head(&sock, file, argv[2], serverport,
				     user, passwd));
  /* free variables */
  lcrzo_string_free(file);
  if (user!=NULL) lcrzo_string_free(user);
  if (passwd!=NULL) lcrzo_string_free(passwd);

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    &servertype,
					    &contenttype,
					    &contentlength,
					    &header, &headersize));
  printf("Return %d (%s)\n", errornumber, errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  printf("Content-length: %ld\n", contentlength);
  lcrzo_epr(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  /* free variables */
  lcrzo_string_free(errormsg);
  lcrzo_string_free(servertype);
  lcrzo_string_free(contenttype);
  lcrzo_data_free(header);

  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}
