
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_http.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000099=
{ /*reference*/     "lcrzoex", 99,
  /*version*/       1, 0,
  /*title*/         "virtual tcp client [web head] (IP option)",
  /*french title*/  "client tcp virtuel [web head] (option IP)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device clieth serveth cliname servname cliport servport url [username [passwd]]",
  /*french usage*/  "device ethcli ethserv nomcli nomserv portcli portserv url [utilisateur [motdepasse]]",
  /*usage example*/ "eth0 aa:bbcc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 4444 /index.html joe hello",
  /*fr. usage ex.*/ "eth0 aa:bbcc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 4444 /index.html laurent bonjour",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000099_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000099_main(int argc, char *argv[])
{ lcrzo_etha servereth, clienteth;
  lcrzo_ipl serveripl, clientipl;
  lcrzo_uint16 serverport, clientport;
  lcrzo_sock sock;
  lcrzo_string user, passwd, file;
  lcrzo_int16 errornumber;
  lcrzo_string errormsg, servertype, contenttype;
  lcrzo_data header;
  lcrzo_int32 headersize, contentlength;
  lcrzo_ipopt ipopt;
  lcrzo_int8 ipoptsize;
  int language;

  if ( argc<9 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000099, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* initialize the needed variables */
  lcrzo_epr(lcrzo_etha_init_eths(argv[2], clienteth));
  lcrzo_epr(lcrzo_etha_init_eths(argv[3], servereth));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[4], &clientipl));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[5], &serveripl));
  clientport=(lcrzo_uint16)atoi(argv[6]);
  serverport=(lcrzo_uint16)atoi(argv[7]);
  lcrzo_epr(lcrzo_string_initm_text(argv[8], &file));
  if ( argc<10 )
  { user=NULL;
    passwd=NULL;
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text(argv[9], &user));
    if ( argc<11 )
    { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &passwd));

    }
    else
    { lcrzo_epr(lcrzo_string_initm_text(argv[10], &passwd));
    }
  }
  /* initialize IP options */
  lcrzo_global_get_language(&language);
  lcrzo_epr(lcrzo_ipopt_stdin((language==LCRZO_GLOBAL_FRLANG)?
                              "options IP":"IP options", "",
                              ipopt, &ipoptsize));

  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_virt2(argv[1],
				    clienteth, servereth,
				    clientipl, serveripl, 
				    clientport, serverport,
                                    ipopt, ipoptsize,
				    &sock));
 
  /* send a request */
  lcrzo_epr(shared_http_request_head(&sock, file, argv[5], serverport, 
				     user, passwd));
  /* free variables */
  lcrzo_string_free(file);
  if (user!=NULL) lcrzo_string_free(user);
  if (passwd!=NULL) lcrzo_string_free(passwd);

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    &servertype,
					    &contenttype,
					    &contentlength,
					    &header, &headersize));
  printf("Return %d (%s)\n", errornumber, errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  printf("Content-length: %ld\n", contentlength);
  lcrzo_epr(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  /* free variables */
  lcrzo_string_free(errormsg);
  lcrzo_string_free(servertype);
  lcrzo_string_free(contenttype);
  lcrzo_data_free(header);

  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}
