
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_http.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000198=
{ /*reference*/     "lcrzoex", 198,
  /*version*/       1, 0,
  /*title*/         "real tcp client [web put]",
  /*french title*/  "client tcp reel [web put]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "server_name server_port url file [username [passwd]]",
  /*french usage*/  "nom_serveur port_serveur url file [utilisateur [motdepasse]]",
  /*usage example*/ "1.2.3.5 4444 /index.html /tmp/file joe hello",
  /*fr. usage ex.*/ "1.2.3.5 4444 /index.html /tmp/file laurent bonjour",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000198_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000198_main(int argc, char *argv[])
{ lcrzo_ipl ipl;
  lcrzo_sock sock;
  lcrzo_uint16 port;
  lcrzo_string user, passwd, url;
  lcrzo_int16 errornumber;
  lcrzo_string errormsg, servertype, contenttype;
  lcrzo_data header, body;
  lcrzo_int32 headersize, contentlength, bodysize;

  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000198, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* initialize the needed variables */
  lcrzo_epr(lcrzo_ipl_init_hs(argv[1], &ipl));
  port=(lcrzo_uint16)atoi(argv[2]);
  lcrzo_epr(lcrzo_string_initm_text(argv[3], &url));
  if ( argc<6 )
  { user=NULL;
    passwd=NULL;
  }
  else
  { lcrzo_epr(lcrzo_string_initm_text(argv[5], &user));
    if ( argc<7 )
    { lcrzo_epr(lcrzo_string_initm_text_passwd("Passwd", "", &passwd));
    }
    else
    { lcrzo_epr(lcrzo_string_initm_text(argv[6], &passwd));
    }
  }

  /* open the connection */
  lcrzo_epr(lcrzo_sock_tcpcli_real(ipl, port, &sock));
  
  /* send a request */
  lcrzo_epr(lcrzo_file_read(argv[4], &body, &bodysize));
  lcrzo_epr(shared_http_request_put(&sock, url, argv[1], port, user, passwd,
				    body, bodysize));
  /* free variables */
  lcrzo_data_free(body);
  lcrzo_string_free(url);
  if (user!=NULL) lcrzo_string_free(user);
  if (passwd!=NULL) lcrzo_string_free(passwd);

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    &servertype,
					    &contenttype,
					    &contentlength,
					    &header, &headersize));
  printf("Return %d (%s)\n", errornumber, errormsg);
  printf("Server: %s\n", servertype);
  printf("Content-Type: %s\n", contenttype);
  printf("Content-length: %ld\n", contentlength);
  lcrzo_epr(lcrzo_data_print(header, headersize, LCRZO_PRINTTYPE_DUMP));
  /* free variables */
  lcrzo_string_free(errormsg);
  lcrzo_string_free(servertype);
  lcrzo_string_free(contenttype);
  lcrzo_data_free(header);

  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}
