
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000206=
{ /*reference*/     "lcrzoex", 206,
  /*version*/       1, 0,
  /*title*/         "{ETH,IP,TCP} spoof specified by command line (with fin)",
  /*french title*/  "spoof {ETH,IP,TCP} specifie sur la ligne de commande (avec fin)",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device src_eth dst_eth src_ip dst_ip port_src port_dst syn ack fin rst seqnum acknum",
  /*french usage*/  "device eth_src eth_dst ip_src ip_dst src_port dst_port syn ack fin rst seqnum acknum",
  /*usage example*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 21  1 1 0 0  43223 493484",
  /*fr. usage ex.*/ "eth0 aa:bb:cc:dd:ee:ff 11:22:33:44:55:66 1.2.3.4 5.6.7.8 1234 21  1 1 0 0  43223 493484", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000206_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000206_main(int argc, char *argv[])
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_spoof spoof;
  int language;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_ipopt ipopt;
  lcrzo_tcpopt tcpopt;
  lcrzo_uint8 ipoptsize, tcpoptsize;
  lcrzo_device device;

  if ( argc<14 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000206, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* get language */
  lcrzo_epr(lcrzo_global_get_language(&language));

  /* initialize device */
  lcrzo_epr(lcrzo_device_init(argv[1], device));

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  lcrzo_epr(lcrzo_etha_init_eths(argv[2], hdrleth.src));
  lcrzo_epr(lcrzo_etha_init_eths(argv[3], hdrleth.dst));

  /* initialize IP header */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&hdrlip));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[4], &(hdrlip.saddr)));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[5], &(hdrlip.daddr)));

  /* initialize IP options */
  lcrzo_epr(lcrzo_ipopt_stdin((language==LCRZO_GLOBAL_FRLANG)?
			      "options IP":"IP options", "",
			      ipopt, &ipoptsize));

  /* initialize TCP header */
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&hdrltcp));
  hdrltcp.sport=(lcrzo_uint16)atoi(argv[6]);
  hdrltcp.dport=(lcrzo_uint16)atoi(argv[7]);
  hdrltcp.syn=(lcrzo_uint8)atoi(argv[8]);
  hdrltcp.ack=(lcrzo_uint8)atoi(argv[9]);
  hdrltcp.fin=(lcrzo_uint8)atoi(argv[10]);
  hdrltcp.rst=(lcrzo_uint8)atoi(argv[11]);
  lcrzo_epr(lcrzo_uint32_init_text(argv[12], &hdrltcp.seqnum));
  lcrzo_epr(lcrzo_uint32_init_text(argv[13], &hdrltcp.acknum));

  /* initialize TCP options */
  lcrzo_epr(lcrzo_tcpopt_stdin((language==LCRZO_GLOBAL_FRLANG)?
			       "options TCP":"TCP options", "",
			       tcpopt, &tcpoptsize));

  /* initialize data */
  lcrzo_epr(lcrzo_data_initm_mixed_stdin((language==LCRZO_GLOBAL_FRLANG)?
					 "donnees du paquet":"packet's data",
					 "'hello' 0D0A", &data, &datasize));

  /* spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_ethipopttcpoptdata(&spoof, device, hdrleth,
					   hdrlip, ipopt, ipoptsize,
					   hdrltcp, tcpopt, tcpoptsize,
					   data, datasize));
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  lcrzo_data_free(data);
  return(LCRZO_ERR_OK);
}
