
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000208=
{ /*reference*/     "lcrzoex", 208,
  /*version*/       1, 0,
  /*title*/         "ARP ping",
  /*french title*/  "ARP ping",
  /*description*/   "the IP address as to be on the LAN",
  /*french desc.*/  "l'adresse IP doit etre sur le LAN",
  /*usage*/         "ip_to_ping",
  /*french usage*/  "adresse_ip_a_pinguer",
  /*usage example*/ "1.2.3.4",
  /*fr. usage ex.*/ "1.2.3.4",
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declarations */
int lcrzoex_000208_main(int argc, char *argv[]);
int lcrzoex_000208_arp_ping(lcrzo_ipl ipl);
void lcrzoex_000208_alarm(int num);
int lcrzoex_000208_arp_ping_spoofsniff(lcrzo_device device,
				       lcrzo_ipl ipl,
				       lcrzo_bool *pgotanswer);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000208_main(int argc, char *argv[])
{ lcrzo_ipl ipl;

  if ( argc<2 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000208, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* initialize infos */
  lcrzo_epr(lcrzo_ipl_init_hs(argv[1], &ipl));

  /* try to ARP ping */
  lcrzo_epr(lcrzoex_000208_arp_ping(ipl));

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int lcrzoex_000208_arp_ping(lcrzo_ipl ipl)
{ lcrzo_device device;
  lcrzo_int32 count, i;
  lcrzo_bool gotanswer;

  /* we try on all devices */
  lcrzo_epr(lcrzo_device_count(LCRZO_DEVICE_TYPE_ONLYBOARDS, &count));
  for ( i=1 ; i<=count ; i++ )
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, i, device));
    printf("Ask on device=%s\n", device);
    lcrzo_epr(lcrzoex_000208_arp_ping_spoofsniff(device, ipl, &gotanswer));
    if (gotanswer) return(LCRZO_ERR_OK);
  }

  puts("No arp replies received");
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
void lcrzoex_000208_alarm(int num) 
{ /*puts("alarm");*/
  _exit(0);
}

/*-------------------------------------------------------------*/
int lcrzoex_000208_arp_ping_spoofsniff(lcrzo_device device,
				       lcrzo_ipl ipl,
				       lcrzo_bool *pgotanswer)
{ lcrzo_sniff sniff;
  lcrzo_ipc ipc;
  lcrzo_spoof spoof;
  lcrzo_uint32 mtu;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlarp hdrlarp;
  lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_etha etha;
  lcrzo_ipa ipa;
  int ret, sniffpid, status, i;

  /* initialize Ethernet header */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&hdrleth));
  for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
  { lcrzo_epr(lcrzo_uint8_rand(1, 0xFE, &(hdrleth.src[i])));
  }
  lcrzo_epr(lcrzo_etha_init(0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, hdrleth.dst));
  hdrleth.type=LCRZO_HDRLETH_TYPE_ARP;

  /* initialize ARP header */
  lcrzo_epr(lcrzo_hdrlarp_initdefault(&hdrlarp));
  hdrlarp.op = 1;
  memcpy(hdrlarp.hw_src, hdrleth.src, LCRZO_ETHA_MAXBYTES);
  for ( i=0 ; i<LCRZO_IPA_MAXBYTES ; i++ )
  { lcrzo_epr(lcrzo_uint8_rand(1, 0xFE, &(hdrlarp.prot_src[i])));
  }
  lcrzo_epr(lcrzo_etha_init(0,0,0,0,0,0, hdrlarp.hw_dst));
  lcrzo_epr(lcrzo_ipa_init_ipl(ipl, hdrlarp.prot_dst));

  /* initialize sniff */
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));
  lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));

  /* initialize IPC */
  ret=lcrzo_ipc_init(&ipc);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_epr(lcrzo_sniff_close(&sniff));
    return(ret);
  }

  /* initialize spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));

  /* send ARP request*/
  ret=lcrzo_spoof_etharp(&spoof, device, hdrleth, hdrlarp);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_epr(lcrzo_sniff_close(&sniff));
    lcrzo_epr(lcrzo_ipc_close(&ipc));
    lcrzo_epr(lcrzo_spoof_close(&spoof));
    return(ret);
  }

  /* send another ARP request, but with IP on the same network 
     because some devices only reply to arp on their network */
  /* set another Ethernet */
  for ( i=0 ; i<LCRZO_ETHA_MAXBYTES ; i++ )
  { lcrzo_epr(lcrzo_uint8_rand(1, 0xFE, &(hdrleth.src[i])));
  }
  memcpy(hdrlarp.hw_src, hdrleth.src, LCRZO_ETHA_MAXBYTES);
  /* set IP */
  lcrzo_epr(lcrzo_ipa_init_ipl(ipl, hdrlarp.prot_src));
  if ( hdrlarp.prot_src[3] & 1 )
    hdrlarp.prot_src[3] &= 0xFE;
  else
    hdrlarp.prot_src[3] |= 1;
  /* spoof */
  ret=lcrzo_spoof_etharp(&spoof, device, hdrleth, hdrlarp);
  if (ret!=LCRZO_ERR_OK)
  { lcrzo_epr(lcrzo_sniff_close(&sniff));
    lcrzo_epr(lcrzo_ipc_close(&ipc));
    lcrzo_epr(lcrzo_spoof_close(&spoof));
    return(ret);
  }

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  /* create sniff process */
  sniffpid=fork();
  if (sniffpid<0)
  { lcrzo_epr(lcrzo_sniff_close(&sniff));
    lcrzo_epr(lcrzo_ipc_close(&ipc));
    return(LCRZO_ERR_OKUNRESOLVED);
  }
  if (sniffpid==0)
  { /* we have one second to obtain the result */
    alarm(1);
    signal(SIGALRM, lcrzoex_000208_alarm);
    /* we compare ipa */
    lcrzo_epr(lcrzo_ipa_init_ipl(ipl, ipa));
    /* sniff */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_NORMAL,
			  &data, &datasize);
    while( ret==LCRZO_ERR_OK )
    { /* we look the answer */
      ret=lcrzo_packet_decode_etharp(data, datasize, NULL, &hdrlarp);
      if ( ret==LCRZO_ERR_OK && hdrlarp.op==2 )
      { if (lcrzo_ipa_equal(hdrlarp.prot_src, ipa))
        { /* found */
          ret=lcrzo_ipc_write_etha(&ipc, hdrlarp.hw_src);
          lcrzo_sniff_close(&sniff);
          lcrzo_ipc_close(&ipc);
          _exit(ret);
        }
      }
      lcrzo_data_free(data);
      ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_NORMAL,
			    &data, &datasize);
    }
    /* reatched on sniff error */
    lcrzo_sniff_close(&sniff);
    lcrzo_ipc_close(&ipc);
    _exit(ret);
  }

  /* wait the end of child */
  waitpid(sniffpid, &status, 0);

  /* see if there is an answer in the IPC */
  ret=lcrzo_ipc_read_etha(&ipc, LCRZO_FALSE, etha);
  if (ret==LCRZO_ERR_OK)
  { *pgotanswer=LCRZO_TRUE;
    lcrzo_ipl_print("The IP address ", ipl, " was found at ");
    lcrzo_etha_print("", etha, "\n");
  }
  else
  { *pgotanswer=LCRZO_FALSE;
  }

  /* close */
  lcrzo_sniff_close(&sniff);
  lcrzo_ipc_close(&ipc);
  return(LCRZO_ERR_OK);
}
