
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_sniff.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000211=
{ /*reference*/     "lcrzoex", 211,
  /*version*/       1, 0,
  /*title*/         "sniff and print open ports",
  /*french title*/  "sniffe et affiche les ports ouverts",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "[device [\"bpf_filter\"]]",
  /*french usage*/  "[device [\"filtre_bpf\"]]",
  /*usage example*/ "eth0 \"net 192.168.1.0/24\"",
  /*fr. usage ex.*/ "eth0 \"net 192.168.1.0/24\"", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000211_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000211_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_list list;
  int ret, language;
  
  /* init device and mtu */
  if (argc<=1)
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, 1, device));
  }
  else
  { lcrzo_epr(lcrzo_device_init(argv[1], device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));
  }
  else
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, argv[2], &sniff));
  }

  /* init the chained list */
  lcrzo_epr(lcrzo_list_init(&list, sizeof(shared_sniff_ipport), NULL));

  /* print message */
  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_FRLANG)
    puts("Attend que des paquets passent sur le reseau...");
  else
    puts("Waiting for packets on the network...");
  
  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_NORMAL, 
                        &sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /* analyze data in TCP headers */
    lcrzo_epr(shared_sniff_decodetcp_ipport(&list, sniffdata, sniffdatasize));
    /* analyze data in UDP headers */
    lcrzo_epr(shared_sniff_decodeudp_ipport(&list, sniffdata, sniffdatasize));
    lcrzo_data_free(sniffdata);
    /* get next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_NORMAL,
                          &sniffdata, &sniffdatasize);
  }

  /* close everything */
  lcrzo_epr(lcrzo_list_close(&list));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_err_print(ret);
  return (ret);
}



