
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_router.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000214=
{ /*reference*/     "lcrzoex", 214,
  /*version*/       1, 0,
  /*title*/         "router forwarding [fragmenting data]",
  /*french title*/  "routeur forwarding [fragmentant les donnees]",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "device ethins ipins ethmasq ipmasq ipgw [lan/mask] (see shared_router.h)",
  /*french usage*/  "device ethins ipins ethmasq ipmasq ipgw [lan/mask] (voir shared_router.h)",
  /*usage example*/ "eth0 a:a:a:a:a:a 192.168.1.3 b:b:b:b:b:b 192.168.2.4 192.168.2.254 192.168.2/24",
  /*fr. usage ex.*/ "eth0 a:a:a:a:a:a 192.168.1.3 b:b:b:b:b:b 192.168.2.4 192.168.2.254 192.168.2/24", 
  /*author*/        "Laurent"
};


/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000214_main(int argc, char *argv[]);


/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000214_main(int argc, char *argv[])
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_spoof spoofclient, spoofgateway;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_etha ethins, ethmasq;
  lcrzo_ipl ipins, ipmasq, ipgw, iplaninf, iplansup;
  int ret, language;
  
  if ( argc<7 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000214, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* init device and mtu */
  lcrzo_epr(lcrzo_device_init(argv[1], device));
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init addresses */
  lcrzo_epr(lcrzo_etha_init_eths(argv[2], ethins));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[3], &ipins));
  lcrzo_epr(lcrzo_etha_init_eths(argv[4], ethmasq));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[5], &ipmasq));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[6], &ipgw));
  if ( argc<8 )
  { iplaninf=0;
    iplansup=0;
  }
  else
  { lcrzo_epr(lcrzo_ipl_init_iprange(argv[7], &iplaninf, &iplansup));
  } 

  /* init sniff */
  lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoofclient));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoofclient, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_set_printprofile(&spoofclient,
					 LCRZO_PRINTPROFILE_SYNTHSYNTH));
  lcrzo_epr(lcrzo_spoof_init(&spoofgateway));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoofgateway, LCRZO_TRUE));
  lcrzo_epr(lcrzo_spoof_set_printprofile(&spoofgateway,
					 LCRZO_PRINTPROFILE_SYNTHSYNTH));

  /* we decide to fragment spoofed data : 2x8 bytes */
  lcrzo_epr(lcrzo_spoof_set_fragoctosize(&spoofgateway, 2));

  /* print message */
  lcrzo_epr(lcrzo_global_get_language(&language));
  if (language==LCRZO_GLOBAL_FRLANG)
  { puts("Attend que des paquets soient destines a ");
  }
  else
  { puts("Waiting for packets destinated to");
  }
  lcrzo_epr(lcrzo_ipl_print("", ipins, ""));
  lcrzo_epr(lcrzo_etha_print("[", ethins, "]\n"));

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_NORMAL, 
                        &sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /* analyze data */
    lcrzo_epr(shared_router_forward(sniffdata, sniffdatasize, 
				    &spoofclient, &spoofgateway, device,
				    ethins, ipins, ethmasq, ipmasq,
				    ipgw, iplaninf, iplansup));
    lcrzo_data_free(sniffdata);
    /* get next packet */
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, LCRZO_SNIFF_TYPE_NORMAL,
                          &sniffdata, &sniffdatasize);
  }

  /* close everything */
  lcrzo_epr(lcrzo_spoof_close(&spoofgateway));
  lcrzo_epr(lcrzo_spoof_close(&spoofclient));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_err_print(ret);
  return (ret);
}

