
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_record.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000224=
{ /*reference*/     "lcrzoex", 224,
  /*version*/       1, 0,
  /*title*/         "Eth record :  modify EthIPTCP fields, spoof",
  /*french title*/  "Eth record :  modifie champs EthIPTCP, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file device srceth dsteth srcip dstip sport dport [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier device ethsrc ethdst ipsrc ipdst sport dport [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1234 567 1 5",
  /*fr. usage ex.*/ "/tmp/fichier eth0 a:a:a:a:a:a b:b:b:b:b:b 1.2.3.4 5.6.7.8 1234 567 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000224_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000224_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_addmod sram;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth modhdreth;
  lcrzo_hdrlip modhdrip;
  lcrzo_hdrltcp modhdrtcp;

  if ( argc<9 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000224, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (argc>=10) start=atol(argv[9]); else start=+1;
  if (argc>=11) end=atol(argv[10]); else end=-1;

  /* init device */
  lcrzo_epr(lcrzo_device_init(argv[2], device));

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&modhdreth));
  lcrzo_epr(lcrzo_etha_init_eths(argv[3], modhdreth.src));
  lcrzo_epr(lcrzo_etha_init_eths(argv[4], modhdreth.dst));
  lcrzo_epr(lcrzo_hdrlip_initdefault(&modhdrip));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[5], &(modhdrip.saddr)));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[6], &(modhdrip.daddr)));
  lcrzo_epr(lcrzo_hdrltcp_initdefault(&modhdrtcp));
  lcrzo_epr(lcrzo_uint16_init_text(argv[7], &(modhdrtcp.sport)));
  lcrzo_epr(lcrzo_uint16_init_text(argv[8], &(modhdrtcp.dport)));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sram */
  lcrzo_epr(shared_record_addmod_initdefault(&sram));
  sram.pspoof=&spoof;
  sram.pspoofdev=&device;
  sram.pmodhdreth=&modhdreth;
  sram.pmodhdrip=&modhdrip;
  sram.pmodhdrtcp=&modhdrtcp;

  /* loop trough each packet saved in the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_eth_spoof, &sram));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  return(LCRZO_ERR_OK);
}
