
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_record.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000230=
{ /*reference*/     "lcrzoex", 230,
  /*version*/       1, 0,
  /*title*/         "IP record :   add Eth header, spoof",
  /*french title*/  "IP record :   ajoute entete Eth, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file device srceth dsteth [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier device ethsrc ethdst [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file eth0 a:a:a:a:a:a b:b:b:b:b:b 1 5",
  /*fr. usage ex.*/ "/tmp/fichier eth0 a:a:a:a:a:a b:b:b:b:b:b 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000230_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000230_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_addmod sram;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrleth addhdreth;

  if ( argc<5 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000230, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (argc>=6) start=atol(argv[5]); else start=+1;
  if (argc>=7) end=atol(argv[6]); else end=-1;

  /* init device */
  lcrzo_epr(lcrzo_device_init(argv[2], device));

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrleth_initdefault(&addhdreth));
  lcrzo_epr(lcrzo_etha_init_eths(argv[3], addhdreth.src));
  lcrzo_epr(lcrzo_etha_init_eths(argv[4], addhdreth.dst));
  addhdreth.type=LCRZO_HDRLETH_TYPE_IP;

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sram */
  lcrzo_epr(shared_record_addmod_initdefault(&sram));
  sram.pspoof=&spoof;
  sram.pspoofdev=&device;
  sram.paddhdreth=&addhdreth;
  
  /* loop trough the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_ip_spoof, &sram));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  return(LCRZO_ERR_OK);
}


