
/* include the library headers */
#include <lcrzo.h>
#include "lcrzoex.h"
#include "shared/shared_record.h"

/*-------------------------------------------------------------*/
/* set information about this example */
lcrzoex_infos lcrzoex_000235=
{ /*reference*/     "lcrzoex", 235,
  /*version*/       1, 0,
  /*title*/         "UDP record :  add IP header, modify UDP fields, spoof",
  /*french title*/  "UDP record :  ajoute entete IP, modifie champs UDP, spoofe",
  /*description*/   "",
  /*french desc.*/  "",
  /*usage*/         "file srcip dstip sport dport [startpacket [endpacket]]",
  /*french usage*/  "nom_fichier ipsrc ipdst sport dport [packet_debut [packet_fin]]",
  /*usage example*/ "/tmp/file 1.2.3.4 5.6.7.8 1234 567 1 5",
  /*fr. usage ex.*/ "/tmp/fichier 1.2.3.4 5.6.7.8 1234 567 1 5", 
  /*author*/        "Laurent"
};

/*-------------------------------------------------------------*/
/* function declaration */
int lcrzoex_000235_main(int argc, char *argv[]);

/*-------------------------------------------------------------*/
/* main function : you can rename it to "main", and then this file
   can be compiled alone. */
int lcrzoex_000235_main(int argc, char *argv[])
{ lcrzo_int32 start, end;
  lcrzo_record record;
  shared_record_addmod sram;
  lcrzo_spoof spoof;
  lcrzo_device device;
  lcrzo_hdrlip addhdrip;
  lcrzo_hdrludp modhdrudp;

  if ( argc<6 )
  { /* print usage */
    lcrzo_epr(lcrzoex_infos_print(lcrzoex_000235, argv[0]));
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (argc>=7) start=atol(argv[6]); else start=+1;
  if (argc>=8) end=atol(argv[7]); else end=-1;

  /* init new addresses */
  lcrzo_epr(lcrzo_hdrlip_initdefault(&addhdrip));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[2], &(addhdrip.saddr)));
  lcrzo_epr(lcrzo_ipl_init_hs(argv[3], &(addhdrip.daddr)));
  addhdrip.protocol=LCRZO_HDRLIP_PROTOCOL_UDP;
  lcrzo_epr(lcrzo_hdrludp_initdefault(&modhdrudp));
  lcrzo_epr(lcrzo_uint16_init_text(argv[4], &(modhdrudp.sport)));
  lcrzo_epr(lcrzo_uint16_init_text(argv[5], &(modhdrudp.dport)));

  /* init spoof */
  lcrzo_epr(lcrzo_spoof_init(&spoof));
  lcrzo_epr(lcrzo_spoof_set_printbeforesending(&spoof, LCRZO_TRUE));

  /* initialize the sram */
  lcrzo_epr(shared_record_addmod_initdefault(&sram));
  sram.pspoof=&spoof;
  sram.pspoofdev=&device;
  sram.paddhdrip=&addhdrip;
  sram.pmodhdrudp=&modhdrudp;
  
  /* loop trough the record */
  lcrzo_epr(lcrzo_record_open(&record, argv[1], LCRZO_RECORD_OPEN_READ));
  lcrzo_epr(lcrzo_record_loop_range(&record, start, end,
				    &shared_record_udp_spoof, &sram));
  lcrzo_epr(lcrzo_record_close(&record));

  /* close spoof */
  lcrzo_epr(lcrzo_spoof_close(&spoof));

  return(LCRZO_ERR_OK);
}
