#include <lcrzo.h>
#include "../lcrzoex.h"
#include "shared_ftp.h"

/*-------------------------------------------------------------*/
void shared_ftp_acti_data_sig_received(int i);
lcrzo_sock *shared_ftp_acti_data_psock;
void shared_ftp_acti_data(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* prepare for Control-C */
  signal(SIGINT, shared_ftp_acti_data_sig_received);
  shared_ftp_acti_data_psock=psock;

  while(1)
  { ret=lcrzo_sock_readm(shared_ftp_acti_data_psock, LCRZO_TRUE,
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { lcrzo_err_print(ret);
      _exit(ret);
    }
    lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT);
    lcrzo_data_free(data);
  }
  /* never reatched */
}
void shared_ftp_acti_data_sig_received(int i)
{ lcrzo_sock_close(shared_ftp_acti_data_psock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
int shared_ftp_acti_cont(lcrzo_sock *psock,
			 lcrzo_ipl clientipl, 
			 lcrzo_uint16 clientdataport,
			 char *file)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_ipa clientipa; 

  /* first, we read the banner */
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'USER' */
  sleep(1);
  lcrzo_epr(lcrzo_data_initm_text("USER anonymous\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'PASS' */
  sleep(1);
  lcrzo_epr(lcrzo_data_initm_text("PASS toto@dom.ex\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'PORT' */
  sleep(1);
  lcrzo_epr(lcrzo_ipa_init_ipl(clientipl, clientipa));
  lcrzo_epr(lcrzo_data_initm_text("PORT ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(clientipa[0], "%d", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(clientipa[1], "%d", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(clientipa[2], "%d", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(clientipa[3], "%d", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(clientdataport>>8, "%d", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(',', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(clientdataport&0xFF, "%d", datasize,
				   &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'RETR' */
  sleep(1);
  lcrzo_epr(lcrzo_data_initm_text("RETR ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(file, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'QUIT' */
  sleep(5);
  lcrzo_epr(lcrzo_data_initm_text("QUIT\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(psock));
  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_ftp_pass_contbegin(lcrzo_sock *psock,
			      lcrzo_ipl *pserveripl, 
			      lcrzo_uint16 *pserverdataport)
{ lcrzo_data data;
  lcrzo_string string;
  lcrzo_int32 datasize;
  lcrzo_regexp re;
  lcrzo_uint8 portvalues[6];
  int i;

  /* first, we read the banner */
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'USER' */
  sleep(1);
  lcrzo_epr(lcrzo_data_initm_text("USER anonymous\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'PASS' */
  sleep(1);
  lcrzo_epr(lcrzo_data_initm_text("PASS toto@dom.ex\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'PASV' */
  sleep(1);
  lcrzo_epr(lcrzo_data_initm_text("PASV\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_epr(lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				     "227 Entering Passive Mode \\("
				     "([0-9]+),([0-9]+),([0-9]+),"
				     "([0-9]+),([0-9]+),([0-9]+)\\)", re));
  data[datasize]='\0';
  for ( i=0 ; i<6 ; i++ )
  { lcrzo_epr(lcrzo_string_initm_range(data, re[i+1].startindex_positive,
				       re[i+1].endindex_positive, &string));
    portvalues[i]=(lcrzo_uint8)atoi(string);
    lcrzo_string_free(string);
  }
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_ipl_init_ipa(portvalues, pserveripl));
  *pserverdataport=(lcrzo_uint16)((portvalues[4]<<8)|portvalues[5]);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
void shared_ftp_pass_data_sig_received(int i);
lcrzo_sock *shared_ftp_pass_data_psock;
void shared_ftp_pass_data(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret;

  /* prepare for Control-C */
  signal(SIGINT, shared_ftp_pass_data_sig_received);
  shared_ftp_pass_data_psock=psock;

  while(1)
  { ret=lcrzo_sock_readm(shared_ftp_pass_data_psock, LCRZO_TRUE,
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { lcrzo_err_print(ret);
      _exit(ret);
    }
    lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT);
    lcrzo_data_free(data);
  }
  /* never reatched */
}
void shared_ftp_pass_data_sig_received(int i)
{ lcrzo_sock_close(shared_ftp_pass_data_psock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
int shared_ftp_pass_contend(lcrzo_sock *psock,
			    char *file)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* we send a 'RETR' */
  sleep(1);
  lcrzo_epr(lcrzo_data_initm_text("RETR ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(file, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'QUIT' */
  sleep(5);
  lcrzo_epr(lcrzo_data_initm_text("QUIT\r\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(psock));
  return (LCRZO_ERR_OK);
}
