#include <lcrzo.h>
#include "../lcrzoex.h"
#include "shared_http.h"

/*-------------------------------------------------------------*/
int shared_http_initm_methurlhttpver(lcrzo_conststring method,
				     lcrzo_conststring url,
				     lcrzo_conststring version,
				     lcrzo_data *pdata,
				     lcrzo_int32 *pdatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;

  /* create "METHOD url HTTP/version" */
  lcrzo_epr(lcrzo_data_initm_text(method, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(' ', 1, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(url, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(" HTTP/", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(version, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;
  if (pdata!=NULL)
  { *pdata=data;
  }
  else
  { lcrzo_data_free(data);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_hostport(lcrzo_conststring hostname,
				 lcrzo_uint16 port,
				 lcrzo_int32 datacurrentsize,
				 lcrzo_data *pdata,
				 lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize;

  /* append "Host: hostname:port" */
  lcrzo_epr(lcrzo_data_appendm_text("Host: ", datacurrentsize, 
				    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(hostname, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char(':', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(port, "%d", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_ipport(lcrzo_ipl ip,
			       lcrzo_uint16 port,
			       lcrzo_int32 datacurrentsize,
			       lcrzo_data *pdata,
			       lcrzo_int32 *pdatasize)
{ lcrzo_ips ips;

  lcrzo_epr(lcrzo_ips_init_ipl(ip, ips));
  lcrzo_epr(shared_http_appendm_hostport(ips, port, datacurrentsize, 
					 pdata, pdatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_appendm_authbasiclogpass(lcrzo_conststring login,
					 lcrzo_conststring passwd,
					 lcrzo_int32 datacurrentsize,
					 lcrzo_data *pdata,
					 lcrzo_int32 *pdatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_string logpass;
 
  /* first, we generate the base64 of login:passwd */
  lcrzo_epr(lcrzo_data_initm_text(login, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(":", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(passwd, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_base64_initm_data(data, datasize, &logpass));
  lcrzo_data_free(data);

  /* append "Authorization: Basic base64_of_login:passwd" */
  lcrzo_epr(lcrzo_data_appendm_text("Authorization: Basic ", datacurrentsize,
                                    pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(logpass, datasize, pdata, &datasize));
  lcrzo_string_free(logpass);
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_percenttext_initm_data(lcrzo_constdata data,
				       lcrzo_int32 datasize,
				       lcrzo_string *ptextwithpercent)
{ lcrzo_string textwithpercent;
  char c;
  int i;

  lcrzo_epr(lcrzo_string_initm_text("", &textwithpercent));
  for ( i=0 ; i<datasize ; i++ ) 
  { c=data[i];
    if ( ( c>='0' && c<='9' ) || ( c>='a' && c<='z') || ( c>='A' && c<='Z' ) )
    { lcrzo_epr(lcrzo_string_appendm_char(c, 1, &textwithpercent));
    }
    else
    { lcrzo_epr(lcrzo_string_appendm_char('%', 1, &textwithpercent));
      lcrzo_epr(lcrzo_string_appendm_int(c, "%02X", &textwithpercent));
    }
  }
  
  if (ptextwithpercent==NULL) { lcrzo_string_free(textwithpercent); }
  else { *ptextwithpercent=textwithpercent; }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_data_initm_percenttext(lcrzo_conststring textwithpercent,
				       lcrzo_data *pdata,
				       lcrzo_int32 *pdatasize)
{ lcrzo_int32 datasize, textsize;
  char c, c1, c2;
  int i, number;

  lcrzo_epr(lcrzo_data_initm_text("", pdata, &datasize));
  textsize=strlen(textwithpercent);
  for ( i=0 ; i<textsize ; i++ ) 
  { c=textwithpercent[i];
    if ( c != '%' )
    { lcrzo_epr(lcrzo_data_appendm_char(c, 1, datasize, pdata, &datasize));
    }
    else
    { if ( i+2 >= textsize ) return(LCRZOEX_ERR_HTTP_MISSCHARAFTERPERCENT);
      c1=textwithpercent[++i];
      c2=textwithpercent[++i];
      if ( c1>='0' && c1<='9' )
      { number=c1-'0';
      }
      else if ( c1>='a' && c1<='f') 
      { number=10+c1-'a';
      }
      else if ( c1>='A' && c1<='F' )
      { number=10+c1-'A';
      }
      else
      { return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
      }
      number=number<<4;
      if ( c2>='0' && c2<='9' )
      { number+=c2-'0';
      }
      else if ( c2>='a' && c2<='f') 
      { number+=10+c2-'a';
      }
      else if ( c2>='A' && c2<='F' )
      { number+=10+c2-'A';
      }
      else
      { return(LCRZOEX_ERR_HTTP_BADCHARAFTERPERCENT);
      }
      lcrzo_epr(lcrzo_data_appendm_char(number, 1, datasize, pdata,&datasize));
    }      
  }

  /* set the output values */
  if (pdatasize!=NULL) *pdatasize=datasize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_get(lcrzo_sock *psock,
			    lcrzo_conststring url,
			    lcrzo_conststring hostname,
			    lcrzo_uint16 port,
			    lcrzo_conststring login,
			    lcrzo_conststring passwd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("GET", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( login!=NULL && passwd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(login, passwd, datasize,
						   &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_post(lcrzo_sock *psock,
			     lcrzo_conststring url,
			     lcrzo_conststring hostname,
			     lcrzo_uint16 port,
			     lcrzo_conststring login,
			     lcrzo_conststring passwd,
			     lcrzo_conststring parameters)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("POST", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( login!=NULL && passwd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(login, passwd, datasize,
						   &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("Content-Type: application/"
				    "x-www-form-urlencoded\r\n",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Content-Length: ",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(strlen(parameters), "%d", 
				   datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  /* now the parameters without an ending line */
  lcrzo_epr(lcrzo_data_appendm_text(parameters, datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_head(lcrzo_sock *psock,
			     lcrzo_conststring url,
			     lcrzo_conststring hostname,
			     lcrzo_uint16 port,
			     lcrzo_conststring login,
			     lcrzo_conststring passwd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("HEAD", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( login!=NULL && passwd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(login, passwd, datasize,
						   &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_put(lcrzo_sock *psock,
			    lcrzo_conststring url,
			    lcrzo_conststring hostname,
			    lcrzo_uint16 port,
			    lcrzo_conststring login,
			    lcrzo_conststring passwd,
			    lcrzo_data filedata,
			    lcrzo_int32 filedatasize)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("PUT", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( login!=NULL && passwd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(login, passwd, datasize,
						   &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("Content-Length: ",
				    datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_int(filedatasize, "%d", 
				   datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  /* write the file to the socket */
  /*lcrzo_epr(lcrzo_data_print(filedata, filedatasize,LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, filedata, filedatasize));

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_request_delete(lcrzo_sock *psock,
			       lcrzo_conststring url,
			       lcrzo_conststring hostname,
			       lcrzo_uint16 port,
			       lcrzo_conststring login,
			       lcrzo_conststring passwd)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  
  /* construct the date to send*/
  lcrzo_epr(shared_http_initm_methurlhttpver("DELETE", url, "1.0",
					     &data, &datasize));
  lcrzo_epr(shared_http_appendm_hostport(hostname, port, datasize,
					 &data, &datasize));
  if ( login!=NULL && passwd!=NULL )
  { lcrzo_epr(shared_http_appendm_authbasiclogpass(login, passwd, datasize,
						   &data, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));

  /* write data to the socket */
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));*/
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  return (LCRZO_ERR_OK);
}


/*-------------------------------------------------------------*/
int shared_http_response_readheader(lcrzo_sock *psock,
				    lcrzo_int16 *perrornumber,
				    lcrzo_string *perrormsg,
				    lcrzo_string *pservertype,
				    lcrzo_string *pcontenttype,
				    lcrzo_int32 *pcontentlength,
				    lcrzo_data *pheader,
				    lcrzo_int32 *pheadersize)
{ lcrzo_data data;
  lcrzo_int32 datasize, headersize;
  lcrzo_string integerstring;
  lcrzo_regexp re;
  lcrzo_bool servertypefound, contenttypefound, contentlengthfound;
  int ret;

  /* on the first line, there is the error code */
  lcrzo_epr(lcrzo_sock_readm_line(psock, LCRZO_TRUE, &data, &datasize));
  /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
  ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
                                 "HTTP/... ([0-9]+) (.+)", re);
  data[datasize]='\0';
  if ( ret==LCRZO_ERR_OK )
  { lcrzo_epr(lcrzo_string_initm_range(data, re[1].startindex_positive,
				       re[1].endindex_positive, 
				       &integerstring));
    if (perrornumber!=NULL) *perrornumber=atoi(integerstring);
    lcrzo_string_free(integerstring);
    lcrzo_epr(lcrzo_string_initm_range(data, re[2].startindex_positive,
				       re[2].endindex_positive, 
				       perrormsg));
  }
  lcrzo_epr(lcrzo_data_initm_data(data, datasize, pheader, &headersize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize, pheader, &headersize));
  lcrzo_data_free(data);

  /* to default values in case they are not set in the header */
  servertypefound=LCRZO_FALSE;
  contenttypefound=LCRZO_FALSE;
  contentlengthfound=LCRZO_FALSE;

  /* we read the header till the empty line */
  while (datasize)
  { /* read the line */
    lcrzo_epr(lcrzo_sock_readm_line(psock, LCRZO_TRUE, &data, &datasize));
    /* add it to the returned header */     
    lcrzo_epr(lcrzo_data_appendm_data(data, datasize, headersize, 
				      pheader, &headersize));
    lcrzo_epr(lcrzo_data_appendm_text("\r\n", headersize,pheader,&headersize));
    /*lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXTN));*/
    if ( !servertypefound )
    { /* search Server */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Server: (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range(data, re[1].startindex_positive,
					   re[1].endindex_positive, 
					   pservertype));
        servertypefound=LCRZO_TRUE;
      }
    }
    if ( !contenttypefound )
    { /* search Content-Type */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Type: (.+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range(data, re[1].startindex_positive,
					   re[1].endindex_positive, 
					   pcontenttype));
        contenttypefound=LCRZO_TRUE;
      }
    }
    if ( !contentlengthfound )
    { /* search Content-Length */
      ret=lcrzo_data_search_regexp(data, datasize, LCRZO_FALSE, +1, -1,
				   "^Content-Length: ([0-9]+)$", re);
      data[datasize]='\0';
      if ( ret==LCRZO_ERR_OK )
      { lcrzo_epr(lcrzo_string_initm_range(data, re[1].startindex_positive,
					   re[1].endindex_positive, 
					   &integerstring));
        if ( pcontentlength!=NULL ) *pcontentlength=atoi(integerstring);
	lcrzo_string_free(integerstring);
	contentlengthfound=LCRZO_TRUE;
      }
    }
    /* we do not need data */
    lcrzo_data_free(data);
  }

  /* set ouput values */
  if ( !servertypefound && pservertype!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", pservertype));
  }
  if ( !contenttypefound && pcontenttype!=NULL )
  { lcrzo_epr(lcrzo_string_initm_text("", pcontenttype));
  }
  if ( !contentlengthfound && pcontentlength!=NULL )
  { *pcontentlength=-1; /* this value indicates that the header 
                           didn't contain the Content-Length line.
			   In this case (rfc2068), we have to read
			   till the server closes the connection :(
			*/
  }
  if ( pheadersize!=NULL ) *pheadersize=headersize;

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_http_response_readbody(lcrzo_sock *psock,
				  lcrzo_int32 contentlength,
				  lcrzo_data *pbody,
				  lcrzo_int32 *pbodysize)
{ lcrzo_data data;
  lcrzo_int32 datasize, bodysize;
  lcrzo_bool continuetoread;
  int ret;

  if ( contentlength != -1 )
  { lcrzo_er(lcrzo_sock_readm_fixed(psock, LCRZO_TRUE, contentlength, pbody));
    if ( pbodysize != NULL ) *pbodysize=contentlength;
  }
  else
  { /* -1 indicates that the header didn't contain the Content-Length line.
       In this case (rfc2068), we have to read till the server closes
       the connection :(
    */
    lcrzo_epr(lcrzo_data_initm_text("", pbody, &bodysize));
    continuetoread=LCRZO_TRUE;
    while (continuetoread)
    { ret=lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize);
      if (ret==LCRZO_ERR_OK)
      { lcrzo_epr(lcrzo_data_appendm_data(data, datasize, bodysize, 
					  pbody, &bodysize));
        lcrzo_epr(lcrzo_data_free(data));
      }
      else if (ret==LCRZO_ERR_OKDATAEND)
      { if ( pbodysize != NULL ) *pbodysize=bodysize;
        continuetoread=LCRZO_FALSE;
      }
      else
	return(ret);
    }
  }
  return (LCRZO_ERR_OK);
}









