#include <lcrzo.h>
#include "../lcrzoex.h"
#include "shared_record.h"

/*-------------------------------------------------------------*/
int shared_record_addmod_initdefault(shared_record_addmod *psram)
{
  psram->pspoof=NULL;
  psram->pspoofdev=NULL;
  psram->paddhdreth=NULL;
  psram->paddhdrip=NULL;
  psram->pmodhdreth=NULL;
  psram->pmodhdrip=NULL;
  psram->pmodhdrudp=NULL;
  psram->pmodhdrtcp=NULL;
  psram->pmodhdricmp=NULL;
  psram->pmodhdrarp=NULL;
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_record_eth_spoof(lcrzo_constdata data,
			    lcrzo_int32 datasize,
			    void *pinfos)
{ shared_record_addmod *psram=pinfos;
  lcrzo_data ethdata, ipdata, protodata, spdata;
  lcrzo_int32 spdatasize;
  lcrzo_uint16 ethdatasize, ipdatasize, protodatasize;
  lcrzo_bool haseth, hasip, hasudp, hastcp, hasicmp, hasarp; 
  lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize, tcpoptsize;
  lcrzo_hdrludp hdrudp;
  lcrzo_hdrltcp hdrtcp;
  lcrzo_tcpopt tcpopt;
  lcrzo_hdrlicmp hdricmp;
  lcrzo_hdrlarp hdrarp;
  int ret;

  if (psram==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pspoof==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pspoofdev==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->paddhdreth!=NULL)
  { puts("psram->paddhdreth should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (psram->paddhdrip!=NULL)
  { puts("psram->paddhdrip should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  
  /* by default, has no header */
  haseth=LCRZO_FALSE;
  hasip=LCRZO_FALSE;
  hasudp=LCRZO_FALSE;
  hastcp=LCRZO_FALSE;
  hasicmp=LCRZO_FALSE;
  hasarp=LCRZO_FALSE;

  /* try to decompose the Ethernet packet */
  ret=lcrzo_packet_decodem_ethdata(data, datasize,
				   &hdreth, &ethdata, &ethdatasize);
  if (ret!=LCRZO_ERR_OK)
  { puts("This packet doesn't start with an Ethernet header");
    return (LCRZO_ERR_OK); /*ignore error, and continue*/
  }
  haseth=LCRZO_TRUE;

  /* try to decode the ARP packet */  
  if (hdreth.type==LCRZO_HDRLETH_TYPE_ARP)
  { ret=lcrzo_packet_decode_arp(ethdata, ethdatasize, &hdrarp);
    hasarp=LCRZO_TRUE;
  } 

  /* try to decode the IP packet */  
  if (hdreth.type==LCRZO_HDRLETH_TYPE_IP)
  { ret=lcrzo_packet_decodem_ipoptdata(ethdata, ethdatasize,
				       &hdrip, ipopt, &ipoptsize, 
				       &ipdata, &ipdatasize);
    if (ret==LCRZO_ERR_OK)
    { hasip=LCRZO_TRUE;
      /* try to decode UDP packet */
      if ( hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_UDP )
      { ret=lcrzo_packet_decodem_udpdata(ipdata, ipdatasize,
					 &hdrudp, &protodata, &protodatasize);
        if (ret==LCRZO_ERR_OK)
        { hasudp=LCRZO_TRUE;
	}
      }
      /* try to decode TCP packet */
      else if ( hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_TCP )
      { ret=lcrzo_packet_decodem_tcpoptdata(ipdata, ipdatasize,
					    &hdrtcp, tcpopt, &tcpoptsize, 
					    &protodata, &protodatasize);
        if (ret==LCRZO_ERR_OK)
        { hastcp=LCRZO_TRUE;
	}
      }
      /* try to decode ICMP packet */
      else if ( hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_ICMP )
      { ret=lcrzo_packet_decodem_icmpdata(ipdata, ipdatasize,
					  &hdricmp, &protodata,&protodatasize);
        if (ret==LCRZO_ERR_OK)
        { hasicmp=LCRZO_TRUE;
	}
      }
    }
  }

  /* modify values */
  if (psram->pmodhdreth!=NULL)
    lcrzo_epr(lcrzo_hdrleth_update(*(psram->pmodhdreth), &hdreth));
  if (psram->pmodhdrip!=NULL)
  { lcrzo_epr(lcrzo_hdrlip_update(*(psram->pmodhdrip), &hdrip));
    hdrudp.check=LCRZO_HDRLUDP_CHECK_DEFVAL;
    hdrtcp.check=LCRZO_HDRLTCP_CHECK_DEFVAL;
    hdricmp.check=LCRZO_HDRLICMP_CHECK_DEFVAL;
  }
  if (psram->pmodhdrudp!=NULL)
    lcrzo_epr(lcrzo_hdrludp_update(*(psram->pmodhdrudp), &hdrudp));
  if (psram->pmodhdrtcp!=NULL)
    lcrzo_epr(lcrzo_hdrltcp_update(*(psram->pmodhdrtcp), &hdrtcp));
  if (psram->pmodhdricmp!=NULL)
    lcrzo_epr(lcrzo_hdrlicmp_update(*(psram->pmodhdricmp), &hdricmp));
  if (psram->pmodhdrarp!=NULL)
    lcrzo_epr(lcrzo_hdrlarp_update(*(psram->pmodhdrarp), &hdrarp));

  /* reconstruct packets */
  if ( hasip )
  { if (hasudp)
    { lcrzo_epr(lcrzo_packet_initm_ethipoptudpdata(hdreth, hdrip,
						   ipopt, ipoptsize,
						   hdrudp,
						   protodata, protodatasize,
						   &spdata, &spdatasize));
    }
    else if (hastcp)
    { lcrzo_epr(lcrzo_packet_initm_ethipopttcpoptdata(hdreth, hdrip,
						      ipopt, ipoptsize,
						      hdrtcp,
						      tcpopt, tcpoptsize,
						      protodata, protodatasize,
						      &spdata, &spdatasize));
    }
    else if (hasicmp)
    { lcrzo_epr(lcrzo_packet_initm_ethipopticmpdata(hdreth, hdrip,
						    ipopt, ipoptsize,
						    hdricmp,
						    protodata, protodatasize,
						    &spdata, &spdatasize));
    }
    else
    { lcrzo_epr(lcrzo_packet_initm_ethipoptdata(hdreth, hdrip,
						ipopt, ipoptsize,
						ipdata, ipdatasize,
						&spdata, &spdatasize));
    }
  }
  else if ( hasarp )
  { lcrzo_epr(lcrzo_packet_initm_etharp(hdreth, hdrarp,
					&spdata, &spdatasize));
  }
  else
  { lcrzo_epr(lcrzo_packet_initm_ethdata(hdreth, ethdata, ethdatasize,
					 &spdata, &spdatasize));
  }

  /* free data */
  if (haseth) lcrzo_data_free(ethdata);
  if (hasip) lcrzo_data_free(ipdata);
  if (hasudp||hastcp||hasicmp) lcrzo_data_free(protodata);

  /*spoof*/
  lcrzo_epr(lcrzo_spoof_eth(psram->pspoof, *(psram->pspoofdev),
			    spdata, spdatasize));
  lcrzo_data_free(spdata);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_record_ip_spoof(lcrzo_constdata data,
			   lcrzo_int32 datasize,
			   void *pinfos)
{ shared_record_addmod *psram=pinfos;
  lcrzo_data ipdata, protodata, spdata;
  lcrzo_int32 spdatasize;
  lcrzo_uint16 ipdatasize, protodatasize;
  lcrzo_bool hasip, hasudp, hastcp, hasicmp; 
  lcrzo_hdrlip hdrip;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize, tcpoptsize;
  lcrzo_hdrludp hdrudp;
  lcrzo_hdrltcp hdrtcp;
  lcrzo_tcpopt tcpopt;
  lcrzo_hdrlicmp hdricmp;
  int ret;

  if (psram==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pspoof==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->paddhdrip!=NULL)
  { puts("psram->paddhdrip should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (psram->pmodhdreth!=NULL)
  { puts("psram->pmodhdreth should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  
  /* by default, has no header */
  hasip=LCRZO_FALSE;
  hasudp=LCRZO_FALSE;
  hastcp=LCRZO_FALSE;
  hasicmp=LCRZO_FALSE;

  /* try to decompose the IP packet */
  ret=lcrzo_packet_decodem_ipoptdata(data, datasize,
				     &hdrip, ipopt, &ipoptsize, 
				     &ipdata, &ipdatasize);
  if (ret!=LCRZO_ERR_OK)
  { puts("This packet doesn't start with an IP header");
    return (LCRZO_ERR_OK); /*ignore error, and continue*/
  }
  hasip=LCRZO_TRUE;

  /* try to decode UDP packet */
  if ( hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_UDP )
  { ret=lcrzo_packet_decodem_udpdata(ipdata, ipdatasize,
				     &hdrudp, &protodata, &protodatasize);
    if (ret==LCRZO_ERR_OK)
    { hasudp=LCRZO_TRUE;
    }
  }
  /* try to decode TCP packet */
  else if ( hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_TCP )
  { ret=lcrzo_packet_decodem_tcpoptdata(ipdata, ipdatasize,
					&hdrtcp, tcpopt, &tcpoptsize, 
					&protodata, &protodatasize);
    if (ret==LCRZO_ERR_OK)
    { hastcp=LCRZO_TRUE;
    }
  }
  /* try to decode ICMP packet */
  else if ( hdrip.protocol==LCRZO_HDRLIP_PROTOCOL_ICMP )
  { ret=lcrzo_packet_decodem_icmpdata(ipdata, ipdatasize,
				      &hdricmp, &protodata,&protodatasize);
    if (ret==LCRZO_ERR_OK)
    { hasicmp=LCRZO_TRUE;
    }
  }

  /* modify values */
  if (psram->pmodhdrip!=NULL)
  { lcrzo_epr(lcrzo_hdrlip_update(*(psram->pmodhdrip), &hdrip));
    hdrudp.check=LCRZO_HDRLUDP_CHECK_DEFVAL;
    hdrtcp.check=LCRZO_HDRLTCP_CHECK_DEFVAL;
    hdricmp.check=LCRZO_HDRLICMP_CHECK_DEFVAL;
  }
  if (psram->pmodhdrudp!=NULL)
    lcrzo_epr(lcrzo_hdrludp_update(*(psram->pmodhdrudp), &hdrudp));
  if (psram->pmodhdrtcp!=NULL)
    lcrzo_epr(lcrzo_hdrltcp_update(*(psram->pmodhdrtcp), &hdrtcp));
  if (psram->pmodhdricmp!=NULL)
    lcrzo_epr(lcrzo_hdrlicmp_update(*(psram->pmodhdricmp), &hdricmp));

  /* reconstruct packets */
  if (hasudp)
  { lcrzo_epr(lcrzo_packet_initm_ipoptudpdata(hdrip, ipopt, ipoptsize,
					      hdrudp,
					      protodata, protodatasize,
					      &spdata, &spdatasize));
  }
  else if (hastcp)
  { lcrzo_epr(lcrzo_packet_initm_ipopttcpoptdata(hdrip, ipopt, ipoptsize,
						 hdrtcp,
						 tcpopt, tcpoptsize,
						 protodata, protodatasize,
						 &spdata, &spdatasize));
  }
  else if (hasicmp)
  { lcrzo_epr(lcrzo_packet_initm_ipopticmpdata(hdrip, ipopt, ipoptsize,
					       hdricmp,
					       protodata, protodatasize,
					       &spdata, &spdatasize));
  }
  else
  { lcrzo_epr(lcrzo_packet_initm_ipoptdata(hdrip, ipopt, ipoptsize,
					   ipdata, ipdatasize,
					   &spdata, &spdatasize));
  }

  /* free data */
  if (hasip) lcrzo_data_free(ipdata);
  if (hasudp||hastcp||hasicmp) lcrzo_data_free(protodata);

  /*spoof*/
  if ( psram->paddhdreth!=NULL && psram->pspoofdev!=NULL )
  { lcrzo_epr(lcrzo_spoof_ethdata(psram->pspoof, *(psram->pspoofdev),
				  *(psram->paddhdreth), 
				  spdata, spdatasize));
  }
  else
  { lcrzo_epr(lcrzo_spoof_ip(psram->pspoof, spdata, spdatasize));
  }

  lcrzo_data_free(spdata);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_record_udp_spoof(lcrzo_constdata data,
			    lcrzo_int32 datasize,
			    void *pinfos)
{ shared_record_addmod *psram=pinfos;
  lcrzo_data protodata, spdata;
  lcrzo_int32 spdatasize;
  lcrzo_uint16 protodatasize;
  lcrzo_hdrludp hdrudp;
  int ret;

  if (psram==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pspoof==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->paddhdrip==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pmodhdreth!=NULL)
  { puts("psram->pmodhdreth should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (psram->pmodhdrip!=NULL)
  { puts("psram->pmodhdrip should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* try to decompose the UDP packet */
  ret=lcrzo_packet_decodem_udpdata(data, datasize,
				   &hdrudp, &protodata, &protodatasize);
  if (ret!=LCRZO_ERR_OK)
  { puts("This packet doesn't start with an UDP header");
    return (LCRZO_ERR_OK); /*ignore error, and continue*/
  }

  /* modify header and force checksum to be recomputed */
  if (psram->pmodhdrudp!=NULL)
    lcrzo_epr(lcrzo_hdrludp_update(*(psram->pmodhdrudp), &hdrudp));
  hdrudp.check=LCRZO_HDRLUDP_CHECK_DEFVAL;

  /* reconstruct packets */
  lcrzo_epr(lcrzo_packet_initm_ipoptudpdata(*(psram->paddhdrip), NULL, 0,
					    hdrudp, protodata, protodatasize,
					    &spdata, &spdatasize));

  /* free data */
  lcrzo_data_free(protodata);

  /*spoof*/
  if ( psram->paddhdreth!=NULL && psram->pspoofdev!=NULL )
  { lcrzo_epr(lcrzo_spoof_ethdata(psram->pspoof, *(psram->pspoofdev),
				  *(psram->paddhdreth), 
				  spdata, spdatasize));
  }
  else
  { lcrzo_epr(lcrzo_spoof_ip(psram->pspoof, spdata, spdatasize));
  }

  lcrzo_data_free(spdata);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_record_tcp_spoof(lcrzo_constdata data,
			    lcrzo_int32 datasize,
			    void *pinfos)
{ shared_record_addmod *psram=pinfos;
  lcrzo_data protodata, spdata;
  lcrzo_int32 spdatasize;
  lcrzo_uint16 protodatasize;
  lcrzo_hdrltcp hdrtcp;
  lcrzo_tcpopt tcpopt;
  lcrzo_uint8 tcpoptsize;
  int ret;

  if (psram==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pspoof==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->paddhdrip==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pmodhdreth!=NULL)
  { puts("psram->pmodhdreth should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (psram->pmodhdrip!=NULL)
  { puts("psram->pmodhdrip should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* try to decompose the TCP packet */
  ret=lcrzo_packet_decodem_tcpoptdata(data, datasize,
				      &hdrtcp, tcpopt, &tcpoptsize,
				      &protodata, &protodatasize);
  if (ret!=LCRZO_ERR_OK)
  { puts("This packet doesn't start with an TCP header");
    return (LCRZO_ERR_OK); /*ignore error, and continue*/
  }

  /* modify header and force checksum to be recomputed */
  if (psram->pmodhdrtcp!=NULL)
    lcrzo_epr(lcrzo_hdrltcp_update(*(psram->pmodhdrtcp), &hdrtcp));
  hdrtcp.check=LCRZO_HDRLTCP_CHECK_DEFVAL;

  /* reconstruct packets */
  lcrzo_epr(lcrzo_packet_initm_ipopttcpoptdata(*(psram->paddhdrip), NULL, 0,
					       hdrtcp, tcpopt, tcpoptsize,
					       protodata, protodatasize,
					       &spdata, &spdatasize));

  /* free data */
  lcrzo_data_free(protodata);

  /*spoof*/
  if ( psram->paddhdreth!=NULL && psram->pspoofdev!=NULL )
  { lcrzo_epr(lcrzo_spoof_ethdata(psram->pspoof, *(psram->pspoofdev),
				  *(psram->paddhdreth), 
				  spdata, spdatasize));
  }
  else
  { lcrzo_epr(lcrzo_spoof_ip(psram->pspoof, spdata, spdatasize));
  }

  lcrzo_data_free(spdata);
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_record_icmp_spoof(lcrzo_constdata data,
			    lcrzo_int32 datasize,
			    void *pinfos)
{ shared_record_addmod *psram=pinfos;
  lcrzo_data protodata, spdata;
  lcrzo_int32 spdatasize;
  lcrzo_uint16 protodatasize;
  lcrzo_hdrlicmp hdricmp;
  int ret;

  if (psram==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pspoof==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->paddhdrip==NULL) return(LCRZO_ERR_PANULLPTR);
  if (psram->pmodhdreth!=NULL)
  { puts("psram->pmodhdreth should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }
  if (psram->pmodhdrip!=NULL)
  { puts("psram->pmodhdrip should be NULL");
    return(LCRZO_ERR_BULCRZOEXBADUSAGE);
  }

  /* try to decompose the ICMP packet */
  ret=lcrzo_packet_decodem_icmpdata(data, datasize,
				   &hdricmp, &protodata, &protodatasize);
  if (ret!=LCRZO_ERR_OK)
  { puts("This packet doesn't start with an ICMP header");
    return (LCRZO_ERR_OK); /*ignore error, and continue*/
  }

  /* modify header and force checksum to be recomputed */
  if (psram->pmodhdricmp!=NULL)
    lcrzo_epr(lcrzo_hdrlicmp_update(*(psram->pmodhdricmp), &hdricmp));
  hdricmp.check=LCRZO_HDRLICMP_CHECK_DEFVAL;

  /* reconstruct packets */
  lcrzo_epr(lcrzo_packet_initm_ipopticmpdata(*(psram->paddhdrip), NULL, 0,
					    hdricmp, protodata, protodatasize,
					    &spdata, &spdatasize));

  /* free data */
  lcrzo_data_free(protodata);

  /*spoof*/
  if ( psram->paddhdreth!=NULL && psram->pspoofdev!=NULL )
  { lcrzo_epr(lcrzo_spoof_ethdata(psram->pspoof, *(psram->pspoofdev),
				  *(psram->paddhdreth), 
				  spdata, spdatasize));
  }
  else
  { lcrzo_epr(lcrzo_spoof_ip(psram->pspoof, spdata, spdatasize));
  }

  lcrzo_data_free(spdata);
  return (LCRZO_ERR_OK);
}







