#include <lcrzo.h>
#include "../lcrzoex.h"
#include "shared_misc.h"
#include "shared_smtp.h"

/*-------------------------------------------------------------*/
/*-------------------------------------------------------------*/
#define LCRZO_ERR_EXBADEMAILAD  12001
int shared_smtp_sendamail(lcrzo_ipl serverip, lcrzo_uint16 serverport,
			  lcrzo_conststring mailfrom, 
			  lcrzo_conststring rcptto,
			  lcrzo_conststring from, 
			  lcrzo_conststring replyto, 
			  lcrzo_conststring to,
			  lcrzo_conststring subject, 
			  lcrzo_conststring bodyfilename)
{ lcrzo_sock sock;
  lcrzo_data data;
  lcrzo_int32 datasize, foundpositive;
  lcrzo_string domainname;
  int ret;

  /* initialize the client domain name */
  ret=lcrzo_string_search_char(mailfrom, LCRZO_FALSE,
			       +1, -1, '@',
			       NULL, &foundpositive);
  if ( ret!=LCRZO_ERR_OK )
  { printf("I didn't found '@' in %s\n", mailfrom); 
    return(LCRZO_ERR_EXBADEMAILAD);
  }
  lcrzo_epr(lcrzo_string_initm_range(mailfrom, foundpositive+1, -1,
				     &domainname));

  /* initialize the client */
  lcrzo_epr(lcrzo_sock_tcpcli_real(serverip, serverport, &sock));

  /* first, we read the banner */
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 220));

  /* then, we send a 'HELO' */
  lcrzo_epr(lcrzo_data_initm_text("HELO ", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(domainname, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_string_free(domainname);
  lcrzo_data_free(data);
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'MAIL FROM' */
  lcrzo_epr(lcrzo_data_initm_text("Mail From: <", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(mailfrom, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'RCPT TO' */
  lcrzo_epr(lcrzo_data_initm_text("Rcpt to: <", &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(rcptto, datasize, &data, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'DATA' */
  lcrzo_epr(lcrzo_data_initm_text("DATA\r\n",
				  &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 354));

  /* then, we send data */
  lcrzo_epr(shared_smtp_generateheaderbody(from, replyto, to, 
					   subject, bodyfilename,
					   &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(lcrzo_data_initm_text(".\r\n",
				  &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 250));

  /* then, we send a 'QUIT' */
  lcrzo_epr(lcrzo_data_initm_text("QUIT\r\n",
				  &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(&sock, data, datasize));
  lcrzo_data_free(data);
  lcrzo_epr(shared_misc_verifyresponse1(&sock, 221));

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(&sock));
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_smtp_generateheaderbody(lcrzo_conststring from,
				   lcrzo_conststring replyto,
				   lcrzo_conststring to,
				   lcrzo_conststring subject, 
				   lcrzo_conststring bodyfilename,
				   lcrzo_data *pdata,
				   lcrzo_int32 *pdatasize)
{ lcrzo_uint32 r;
  lcrzo_int32 datasize, linesize;
  lcrzo_data line;
  char thedate[80];
  time_t thetime;
  int ret, language;
  FILE *stream;

  /* open the file */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_epr(lcrzo_stream_open_read(bodyfilename, (LCRZOFILE**)&stream));

  /* generate the header */
  lcrzo_epr(lcrzo_data_initm_text("Message-ID: <", pdata, &datasize));
  lcrzo_epr(lcrzo_uint32_rand(0, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_data_appendm_int(r, "%08X", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_char('.', 1, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_uint32_rand(0, 0xFFFFFFFFu, &r));
  lcrzo_epr(lcrzo_data_appendm_int(r, "%08X", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("@domaine.fr>\r\n",
				    datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Date: ", datasize, pdata, &datasize));
  thetime=time(NULL);
  strftime(thedate, 79, "%a, %d %b %Y %H:%M:%S %z", localtime(&thetime));
  lcrzo_epr(lcrzo_data_appendm_text(thedate, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("From: <", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(from, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, pdata, &datasize));
  if ( strcmp(from, replyto) )
  { lcrzo_epr(lcrzo_data_appendm_text("Reply-To: <", datasize,
				      pdata, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text(replyto, datasize, pdata, &datasize));
    lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, pdata, &datasize));
  }
  lcrzo_epr(lcrzo_data_appendm_text("Organization: Laurent\r\n",
				    datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("X-Mailer: Laurent Mailer\r\n",
				    datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("X-Accept-Language: en\r\n",
				    datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("MIME-Version: 1.0\r\n",
				    datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("To: <", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(to, datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text(">\r\n", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Subject: ", datasize, pdata, &datasize));
  
  if ( strlen(subject) )
  { lcrzo_epr(lcrzo_data_appendm_text(subject, datasize, pdata, &datasize));
  }
  else
  { /* the subject is on the first line of the file */
    lcrzo_epr(lcrzo_stream_readm_line(stream, LCRZO_TRUE, &line, &linesize));
    lcrzo_epr(lcrzo_data_appendm_data(line, linesize, 
				      datasize, pdata, &datasize));
  }

  lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Content-Type: text/plain;"
				    " charset=iso-8859-1\r\n",
				    datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("Content-Transfer-Encoding: 8bit\r\n",
				    datasize, pdata, &datasize));
  lcrzo_epr(lcrzo_data_appendm_text("\r\n",
				    datasize, pdata, &datasize));

  /* generate the body */
  while(1)
  { ret=lcrzo_stream_readm_line(stream, LCRZO_TRUE, &line, &linesize);
    if (ret==LCRZO_ERR_OKDATAEND)
    { break;
    }
    else if (ret==LCRZO_ERR_OK)
    { if ( linesize>254 )
      { if ( language==LCRZO_GLOBAL_FRLANG )
          printf("La ligne semble trop longue: %ld caracteres\n",linesize); 
        else
          printf("Line seems too long: %ld characters\n", linesize); 
      }
      if ( linesize>0 && line[0]=='.' )
      { lcrzo_epr(lcrzo_data_appendm_char('.', 1, datasize, pdata, &datasize));
      }
      lcrzo_epr(lcrzo_data_appendm_data(line, linesize,
					datasize, pdata, &datasize));
      lcrzo_data_free(line);
      lcrzo_epr(lcrzo_data_appendm_text("\r\n", datasize, pdata, &datasize));
    }
    else
    { lcrzo_stream_close(stream);
      return(ret);
    }
  }
  lcrzo_epr(lcrzo_stream_close(stream));

  /* end */
  if (pdatasize!=NULL) *pdatasize=datasize;
  return(LCRZO_ERR_OK);
}
