#include <lcrzo.h>
#include "../lcrzoex.h"
#include "shared_sniff.h"

/*-------------------------------------------------------------*/
int shared_sniff_dump(int argc, char *argv[], int snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_printprofile printprofile;
  int ret, language;
  
  /* init device and mtu */
  if (argc<=1)
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, 1, device));
  }
  else
  { lcrzo_epr(lcrzo_device_init(argv[1], device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* get printprofile */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_printprofile_stdin((language==LCRZO_GLOBAL_FRLANG)?
			   "Choisissez le profil d'affichage":
			   "Choose the print profile",
			   LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile);

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));
  }
  else
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, argv[2], &sniff));
  }

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { lcrzo_packet_print_eth(sniffdata, sniffdatasize, printprofile);
    lcrzo_data_free(sniffdata);
    if (printprofile.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype,
			  &sniffdata, &sniffdatasize);
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_err_print(ret);
  return (ret);
}

/*-------------------------------------------------------------*/
void shared_sniff_dumpsaveeth_sig_received(int i);
lcrzo_record shared_sniff_dumpsaveeth_record;
int shared_sniff_dumpsaveeth(int argc, char *argv[], int snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_printprofile printprofile1, printprofile2;
  int ret, language;
  
  /* open the record */
  lcrzo_epr(lcrzo_record_open(&shared_sniff_dumpsaveeth_record, argv[1],
			      LCRZO_RECORD_OPEN_WRITE));

  /* prepare for Control-C */
  signal(SIGINT, shared_sniff_dumpsaveeth_sig_received);

  /* init device and mtu */
  if (argc<=2)
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, 1, device));
  }
  else
  { lcrzo_epr(lcrzo_device_init(argv[2], device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* get print profile */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_printprofile_stdin((language==LCRZO_GLOBAL_FRLANG)?
			   "Choisissez le profil d'affichage pour l'ecran":
			   "Choose the screen print profile",
			   LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile1);
  lcrzo_printprofile_stdin_record((language==LCRZO_GLOBAL_FRLANG)?
			   "Choisissez le profil d'affichage pour le record":
			   "Choose the record print profile",
			   LCRZO_PRINTPROFILE_HEXANCHMS, &printprofile2);

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));
  }
  else
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, argv[3], &sniff));
  }

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype, 
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { lcrzo_packet_print_eth(sniffdata, sniffdatasize, printprofile1);
    if (printprofile1.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
    lcrzo_epr(lcrzo_record_write_eth(&shared_sniff_dumpsaveeth_record,
				     sniffdata, sniffdatasize,
				     printprofile2));
    lcrzo_data_free(sniffdata);
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype, 
			  &sniffdata, &sniffdatasize);
  }

  /* close everything */
  lcrzo_epr(lcrzo_record_close(&shared_sniff_dumpsaveeth_record));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_err_print(ret);
  return (ret);
}

void shared_sniff_dumpsaveeth_sig_received(int i)
{ int language;

  /* close the record, and exit */
  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture du fichier (recu : SIG%d)\n", i);
  else
    printf("Closing file (received : SIG%d)\n", i);
  lcrzo_record_close(&shared_sniff_dumpsaveeth_record);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
void shared_sniff_dumpsaveip_sig_received(int i);
lcrzo_record shared_sniff_dumpsaveip_record;
int shared_sniff_dumpsaveip(int argc, char *argv[], int snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata, ipdata;
  lcrzo_int32 sniffdatasize;
  lcrzo_uint16 ipdatasize;
  lcrzo_ipopt ipopt;
  lcrzo_uint8 ipoptsize;
  lcrzo_printprofile printprofile1, printprofile2;
  lcrzo_hdrlip hdrlip;
  int ret, language;
  
  /* open the record */
  lcrzo_epr(lcrzo_record_open(&shared_sniff_dumpsaveip_record, argv[1],
			      LCRZO_RECORD_OPEN_WRITE));

  /* prepare for Control-C */
  signal(SIGINT, shared_sniff_dumpsaveip_sig_received);

  /* init device and mtu */
  if (argc<=2)
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, 1, device));
  }
  else
  { lcrzo_epr(lcrzo_device_init(argv[2], device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* get print profile */
  lcrzo_epr(lcrzo_global_get_language(&language));
  lcrzo_printprofile_stdin((language==LCRZO_GLOBAL_FRLANG)?
			   "Choisissez le profil d'affichage pour l'ecran":
			   "Choose the screen print profile",
			   LCRZO_PRINTPROFILE_ARRAYDUMP, &printprofile1);
  lcrzo_printprofile_stdin_record((language==LCRZO_GLOBAL_FRLANG)?
			   "Choisissez le profil d'affichage pour le record":
			   "Choose the record print profile",
			   LCRZO_PRINTPROFILE_HEXANCHMS, &printprofile2);

  /* init sniff */
  if (argc<=3)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));
  }
  else
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, argv[3], &sniff));
  }

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype,
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { /* only print if it's IP */
    ret=lcrzo_packet_decodem_ethipoptdata(sniffdata, sniffdatasize,
					  NULL, &hdrlip,
					  ipopt, &ipoptsize,
					  &ipdata, &ipdatasize);
    lcrzo_data_free(sniffdata);
    if ( ret==LCRZO_ERR_OK )
    { /* reconstruct the packet to print */
      lcrzo_packet_initm_ipoptdata(hdrlip, ipopt, ipoptsize,
				   ipdata, ipdatasize,
				   &sniffdata, &sniffdatasize);
      lcrzo_data_free(ipdata);
      lcrzo_packet_print_ip(sniffdata, sniffdatasize, printprofile1);
      if (printprofile1.hdr!=LCRZO_PRINTTYPE_SYNTH) printf("\n");
      lcrzo_record_write_ip(&shared_sniff_dumpsaveip_record,
			    sniffdata, sniffdatasize,
			    printprofile2);
      lcrzo_data_free(sniffdata);
    }
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype,
			  &sniffdata, &sniffdatasize);
  }

  /* close everything */
  lcrzo_epr(lcrzo_record_close(&shared_sniff_dumpsaveip_record));
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_err_print(ret);
  return (ret);
}

void shared_sniff_dumpsaveip_sig_received(int i)
{ int language;

  /* close the record, and exit */
  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture du fichier (recu : SIG%d)\n", i);
  else
    printf("Closing file (received : SIG%d)\n", i);
  lcrzo_record_close(&shared_sniff_dumpsaveip_record);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
int shared_sniff_check(int argc, char *argv[], int snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  lcrzo_data sniffdata;
  lcrzo_int32 sniffdatasize;
  int ret;
  
  /* init device and mtu */
  if (argc<=1)
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, 1, device));
  }
  else
  { lcrzo_epr(lcrzo_device_init(argv[1], device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));
  }
  else
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, argv[2], &sniff));
  }

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype,
			&sniffdata, &sniffdatasize);
  while (ret==LCRZO_ERR_OK)
  { shared_sniff_check_verif(sniffdata, sniffdatasize);
    lcrzo_data_free(sniffdata);
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype,
			  &sniffdata, &sniffdatasize);
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_err_print(ret);
  return (ret);
}

/*-------------------------------------------------------------*/
int shared_sniff_check_verif(lcrzo_constdata packet,
			     lcrzo_int32 packetsize)
{ int ret;
  lcrzo_hdrleth hdrleth;
  lcrzo_hdrlip hdrlip;
  lcrzo_hdrludp hdrludp;
  lcrzo_hdrltcp hdrltcp;
  lcrzo_hdrlicmp hdrlicmp;
  lcrzo_array2k data, temp;
  lcrzo_int32 tempsize;
  lcrzo_ipopt ipopt;
  lcrzo_tcpopt tcpopt;
  lcrzo_uint8 ipoptsize, tcpoptsize;
  lcrzo_uint16 check1, check2, datasize;

  /* if it's IP+UDP */
  ret=lcrzo_packet_decode_ethipoptudpdata(packet, packetsize,
					  &hdrleth, 
					  &hdrlip, ipopt, &ipoptsize,
					  &hdrludp, LCRZO_ARRAY2K_MAXBYTES,
					  data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\tUDP %04X\t\t", hdrlip.check, hdrludp.check);
    /* we save network values to compare them */
    check1=hdrlip.check;
    check2=hdrludp.check;
    /* the function has to compute checksums */
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    hdrludp.check=LCRZO_HDRLUDP_CHECK_DEFVAL; 
    lcrzo_packet_init_ipoptudpdata(hdrlip, ipopt, ipoptsize,
				   hdrludp, data, datasize,
				   LCRZO_ARRAY2K_MAXBYTES,
				   temp, &tempsize);
    lcrzo_packet_decode_ipoptudpdata(temp, tempsize,
				     &hdrlip, NULL, NULL,
				     &hdrludp, LCRZO_ARRAY2K_MAXBYTES,
				     NULL, NULL);
    /* compare */
    if ( check1==hdrlip.check &&
	 ((check2==hdrludp.check)||(hdrlip.offsetfrag|hdrlip.morefrag)) )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\tUDP %04X\n", hdrlip.check, hdrludp.check);
      lcrzo_packet_print_eth(packet, packetsize, LCRZO_PRINTPROFILE_ARRAYDUMP);
    }
    return (LCRZO_ERR_OK);
  }

  /* if it's IP+TCP */
  ret=lcrzo_packet_decode_ethipopttcpoptdata(packet, packetsize,
					     &hdrleth, 
					     &hdrlip, ipopt, &ipoptsize,
					     &hdrltcp, 
					     tcpopt, &tcpoptsize,
					     LCRZO_ARRAY2K_MAXBYTES,
					     data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\tTCP %04X\t\t", hdrlip.check, hdrltcp.check);
    check1=hdrlip.check;
    check2=hdrltcp.check;
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    hdrltcp.check=LCRZO_HDRLTCP_CHECK_DEFVAL; 
    lcrzo_packet_init_ipopttcpoptdata(hdrlip, ipopt, ipoptsize,
				      hdrltcp, tcpopt, tcpoptsize,
				      data, datasize, LCRZO_ARRAY2K_MAXBYTES,
				      temp, &tempsize);
    lcrzo_packet_decode_ipopttcpoptdata(temp, tempsize,
					&hdrlip, NULL, NULL,
					&hdrltcp, NULL, NULL, 
					LCRZO_ARRAY2K_MAXBYTES, NULL, NULL);
    if ( check1==hdrlip.check && 
	 ((check2==hdrltcp.check)||(hdrlip.offsetfrag|hdrlip.morefrag)) )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\tTCP %04X\n", hdrlip.check, hdrltcp.check);
      lcrzo_packet_print_eth(packet, packetsize, LCRZO_PRINTPROFILE_ARRAYDUMP);
    }
    return (LCRZO_ERR_OK);
  }

  /* if it's IP+ICMP */
  ret=lcrzo_packet_decode_ethipopticmpdata(packet, packetsize,
					   &hdrleth, 
					   &hdrlip, ipopt, &ipoptsize,
					   &hdrlicmp, LCRZO_ARRAY2K_MAXBYTES,
					   data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\tICMP %04X\t\t", hdrlip.check, hdrlicmp.check);
    check1=hdrlip.check;
    check2=hdrlicmp.check;
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    hdrlicmp.check=LCRZO_HDRLICMP_CHECK_DEFVAL; 
    lcrzo_packet_init_ipopticmpdata(hdrlip, ipopt, ipoptsize,
				    hdrlicmp, data, datasize,
				    LCRZO_ARRAY2K_MAXBYTES,
				    temp, &tempsize);
    lcrzo_packet_decode_ipopticmpdata(temp, tempsize,
				      &hdrlip, NULL, NULL,
				      &hdrlicmp,
				      LCRZO_ARRAY2K_MAXBYTES,
				      NULL, NULL);
    if ( check1==hdrlip.check &&
	 ((check2==hdrlicmp.check)||(hdrlip.offsetfrag|hdrlip.morefrag)) )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\tICMP %04X\n", hdrlip.check, hdrlicmp.check);
      lcrzo_packet_print_eth(packet, packetsize, LCRZO_PRINTPROFILE_ARRAYDUMP);
    }
    return (LCRZO_ERR_OK);
  }

  /* if it's IP only */
  ret=lcrzo_packet_decode_ethipoptdata(packet, packetsize,
				       &hdrleth, 
				       &hdrlip, ipopt, &ipoptsize,
				       LCRZO_ARRAY2K_MAXBYTES,
				       data, &datasize);
  if (ret==LCRZO_ERR_OK)
  { printf("IP %04X\t\t\t", hdrlip.check);
    check1=hdrlip.check;
    hdrlip.check=LCRZO_HDRLIP_CHECK_DEFVAL;
    lcrzo_packet_init_ipoptdata(hdrlip, ipopt, ipoptsize,
				data, datasize,
				LCRZO_ARRAY2K_MAXBYTES,
				temp, &tempsize);
    lcrzo_packet_decode_ipoptdata(temp, tempsize,
				  &hdrlip, NULL, NULL,
				  LCRZO_ARRAY2K_MAXBYTES,
				  NULL, NULL);
    if ( check1==hdrlip.check )
    { printf("ok\n");
    }
    else
    { printf("IP %04X\n", hdrlip.check);
      lcrzo_packet_print_eth(packet, packetsize, LCRZO_PRINTPROFILE_ARRAYDUMP);
    }
    return (LCRZO_ERR_OK);
  }

  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_count(int argc, char *argv[], int snifftype)
{ lcrzo_device device;
  lcrzo_uint32 mtu;
  lcrzo_sniff sniff;
  int ret, i, language;
  
  /* init device and mtu */
  if (argc<=1)
  { lcrzo_epr(lcrzo_device_value_pos(LCRZO_DEVICE_TYPE_ONLYBOARDS, 1, device));
  }
  else
  { lcrzo_epr(lcrzo_device_init(argv[1], device));
  }
  lcrzo_epr(lcrzo_mtu_init_device(device, &mtu));

  /* init sniff */
  if (argc<=2)
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, "", &sniff));
  }
  else
  { lcrzo_epr(lcrzo_sniff_init(device, mtu, argv[2], &sniff));
  }

  lcrzo_epr(lcrzo_global_get_language(&language));

  /* main loop */
  ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype, NULL, NULL);
  i=0;
  while (ret==LCRZO_ERR_OK)
  { i++; 
    if (!(i%100))
    { printf((language==LCRZO_GLOBAL_FRLANG)?"Paquet %d\n":"Packet %d\n", i);
    }
    ret=lcrzo_sniff_nextm(&sniff, LCRZO_TRUE, snifftype, NULL, NULL);
  }

  /* close everything */
  lcrzo_epr(lcrzo_sniff_close(&sniff));
  lcrzo_err_print(ret);
  return (ret);
}


/*-------------------------------------------------------------*/
int shared_sniff_printnewethip(lcrzo_list *plist,
			       const lcrzo_etha etha,
			       const lcrzo_ipa ipa)
{ shared_sniff_ethip ethip;
  int ret;

  /* put data in structure */
  memcpy(ethip.eth, etha, LCRZO_ETHA_MAXBYTES);
  memcpy(ethip.ip, ipa, LCRZO_IPA_MAXBYTES);

  /* search structure in list */
  ret=lcrzo_list_searchelem_all(plist, &ethip, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
  { /* element wasn't found : print it and add it */ 
    lcrzo_etha_print("", etha, "\t");
    lcrzo_ipa_print("", ipa, "\n");
    lcrzo_epr(lcrzo_list_add_first(plist, &ethip));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodearp_ethip(lcrzo_list *plist, 
				 lcrzo_constdata ethdata,
				 lcrzo_int32 ethdatasize)
{ lcrzo_hdrleth hdreth;
  lcrzo_hdrlarp hdrarp;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode arp */
  ret=lcrzo_packet_decode_etharp(ethdata, ethdatasize, &hdreth, &hdrarp);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);
  if ( hdreth.type != LCRZO_HDRLETH_TYPE_ARP &&
       hdreth.type != LCRZO_HDRLETH_TYPE_RARP )
    return(LCRZO_ERR_OK);

  /* analyze an ARP request */
  if ( hdrarp.op==1 )
  { shared_sniff_printnewethip(plist, hdrarp.hw_src, hdrarp.prot_src);
  }

  /* analyze an ARP reply */
  if ( hdrarp.op==2 )
  { shared_sniff_printnewethip(plist, hdrarp.hw_src, hdrarp.prot_src);
    shared_sniff_printnewethip(plist, hdrarp.hw_dst, hdrarp.prot_dst);
  }

  /* analyze a RARP reply */
  if ( hdrarp.op==4 )
  { shared_sniff_printnewethip(plist, hdrarp.hw_src, hdrarp.prot_src);
    shared_sniff_printnewethip(plist, hdrarp.hw_dst, hdrarp.prot_dst);
  }
  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodeip_ethip(lcrzo_list *plist, 
				lcrzo_constdata ethdata,
				lcrzo_int32 ethdatasize)
{ lcrzo_hdrleth hdreth;
  lcrzo_hdrlip hdrip;
  lcrzo_ipa ipa;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode ip */
  ret=lcrzo_packet_decodem_ethipoptdata(ethdata, ethdatasize, 
					&hdreth, &hdrip, NULL, NULL,
					NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* eventually print new addresses */
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.saddr, ipa));
  shared_sniff_printnewethip(plist, hdreth.src, ipa);
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.daddr, ipa));
  shared_sniff_printnewethip(plist, hdreth.dst, ipa);

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_printnewipport(lcrzo_list *plist,
				lcrzo_uint8 type,
				const lcrzo_ipa ipa,
				lcrzo_uint16 port)
{ shared_sniff_ipport ipport;
  int ret;

  /* put data in structure */
  ipport.type=type;
  memcpy(ipport.ip, ipa, LCRZO_IPA_MAXBYTES);
  ipport.port=port;

  /* search structure in list */
  ret=lcrzo_list_searchelem_all(plist, &ipport, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
  { /* element wasn't found : print it and add it */
    if ( type==1 )
      printf("UDP\t");
    else if ( type==2 )
      printf("TCP\t");
    else
      printf("???\t");
    lcrzo_ipa_print("", ipa, "\t");
    printf("%d\n", port);
    lcrzo_epr(lcrzo_list_add_first(plist, &ipport));
  }

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodeudp_ipport(lcrzo_list *plist, 
				  lcrzo_constdata ethdata,
				  lcrzo_int32 ethdatasize)
{ lcrzo_hdrlip hdrip;
  lcrzo_hdrludp hdrudp;
  lcrzo_ipa ipa;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipoptudpdata(ethdata, ethdatasize, 
					   NULL, &hdrip, NULL, NULL,
					   &hdrudp, NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* eventually print new addresses */
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.daddr, ipa));
  shared_sniff_printnewipport(plist, 2, ipa, hdrudp.dport);

  return(LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
int shared_sniff_decodetcp_ipport(lcrzo_list *plist, 
				  lcrzo_constdata ethdata,
				  lcrzo_int32 ethdatasize)
{ lcrzo_hdrlip hdrip;
  lcrzo_hdrltcp hdrtcp;
  lcrzo_ipa ipa;
  int ret;

  /*lcrzo_epr(lcrzo_packet_print_eth(ethdata, ethdatasize, 
              LCRZO_PRINTPROFILE_ARRAYDUMP));*/
  
  /* try to decode it */
  ret=lcrzo_packet_decodem_ethipopttcpoptdata(ethdata, ethdatasize, 
					      NULL, &hdrip, NULL, NULL,
					      &hdrtcp, NULL, NULL,
					      NULL, NULL);
  if ( ret!=LCRZO_ERR_OK )
    return(LCRZO_ERR_OK);

  /* only count SYNACK */
  if ( ! ( hdrtcp.syn && hdrtcp.ack && !hdrtcp.fin && !hdrtcp.rst ) )
    return(LCRZO_ERR_OK);

  /* eventually print new addresses */
  lcrzo_epr(lcrzo_ipa_init_ipl(hdrip.saddr, ipa));
  shared_sniff_printnewipport(plist, 2, ipa, hdrtcp.sport);

  return(LCRZO_ERR_OK);
}

