#include <lcrzo.h>
#include "../lcrzoex.h"
#include "shared_sock.h"

/*-------------------------------------------------------------*/
int shared_sock_banner(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
 
  /* first, we read the banner */
  lcrzo_epr(lcrzo_sock_readm(psock, LCRZO_TRUE, &data, &datasize));
  lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
  lcrzo_data_free(data);

  /* then, we send a 'quit' */
  lcrzo_epr(lcrzo_data_initm_text("QUIT\n", &data, &datasize));
  lcrzo_epr(lcrzo_sock_write(psock, data, datasize));
  lcrzo_data_free(data);

  /* close the socket */
  lcrzo_epr(lcrzo_sock_close(psock));
  return (LCRZO_ERR_OK);
}

/*-------------------------------------------------------------*/
void shared_sock_stdin_sig_received(int i);
int shared_sock_stdin_pid;
lcrzo_sock shared_sock_stdin_sock;

int shared_sock_stdin(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret, language;

  lcrzo_global_get_language(&language);

  /* in case we receive a signal */
  signal(SIGINT, shared_sock_stdin_sig_received);
  shared_sock_stdin_sock=*psock;

  shared_sock_stdin_pid=fork();
  if (shared_sock_stdin_pid<0) return(LCRZO_ERR_FUFORK);
  if (shared_sock_stdin_pid==0)
  { /* this process prints read data (child) */
    while(1)
    { ret=lcrzo_sock_readm(&shared_sock_stdin_sock, LCRZO_TRUE, 
			   &data, &datasize);
      if ( ret!=LCRZO_ERR_OK )
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("\nLa lecture sur la socket n'est plus possible.");
        else
	  puts("\nWe cannot read from the socket anymore.");
        kill(getppid(), SIGINT);
	_exit(ret);
      }
      if ( datasize!=0 )
      { printf("\n");
        lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP);
      }
      lcrzo_data_free(data);
    }
    /* never reatched */
  }
  else
  { /* this process writes data */
    while(1)
    { lcrzo_epr(lcrzo_data_initm_mixed_stdin((language==LCRZO_GLOBAL_FRLANG)?
					     "Vos donnees mixtes":
					     "Your mixed data",
					     "'hello' 0D0A", 
					     &data, &datasize));
      ret=lcrzo_sock_write(&shared_sock_stdin_sock, data, datasize);
      lcrzo_data_free(data);
      if (ret!=LCRZO_ERR_OK)
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("\nL'ecriture sur la socket n'est plus possible.");
        else 
	  puts("\nWe cannot write to the socket anymore.");
        raise(SIGINT);
      }
    }
    /* never reatched */
  }
  /* never reatched */
  return (LCRZO_ERR_IEINTERNALERROR);
}

void shared_sock_stdin_sig_received(int i)
{ int status, language;

  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture de la socket (recu : SIG%d)\n", i);
  else
    printf("Closing socket (received : SIG%d)\n", i);
  kill(shared_sock_stdin_pid, SIGTERM);
  waitpid(shared_sock_stdin_pid, &status, 0);
  lcrzo_sock_close(&shared_sock_stdin_sock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
void shared_sockudpser_stdin_sig_received(int i);
int shared_sockudpser_stdin_pid;
lcrzo_sock shared_sockudpser_stdin_sock;

int shared_sockudpser_stdin(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret, language;

  lcrzo_global_get_language(&language);

  /* in case we receive a signal */
  signal(SIGINT, shared_sockudpser_stdin_sig_received);
  shared_sockudpser_stdin_sock=*psock;

  /* we cannot write before reading */
  if ( language==LCRZO_GLOBAL_FRLANG )
    puts("\nNous attendons que le client ecrive.");
  else
    puts("\nWe wait for the client to write.");
  ret=lcrzo_sock_readm(&shared_sockudpser_stdin_sock, LCRZO_TRUE,
		       &data, &datasize);
  
  shared_sockudpser_stdin_pid=fork();
  if (shared_sockudpser_stdin_pid<0) return(LCRZO_ERR_FUFORK);
  if (shared_sockudpser_stdin_pid==0)
  { /* this process prints read data (child) */
    while(1)
    { if ( ret!=LCRZO_ERR_OK )
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("\nLa lecture sur la socket n'est plus possible.");
        else
	  puts("\nWe cannot read from the socket anymore.");
        kill(getppid(), SIGINT);
	_exit(ret);
      }
      if ( datasize!=0 )
      { printf("\n");
        lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP);
      }
      lcrzo_data_free(data);
      ret=lcrzo_sock_readm(&shared_sockudpser_stdin_sock, LCRZO_TRUE, 
			   &data, &datasize);
    }
    /* never reatched */
  }
  else
  { /* this process writes data */
    while(1)
    { lcrzo_epr(lcrzo_data_initm_mixed_stdin((language==LCRZO_GLOBAL_FRLANG)?
					     "Vos donnees mixtes":
					     "Your mixed data",
					     "'hello' 0D0A", 
					     &data, &datasize));
      ret=lcrzo_sock_write(&shared_sockudpser_stdin_sock, data, datasize);
      lcrzo_data_free(data);
      if (ret!=LCRZO_ERR_OK)
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("\nL'ecriture sur la socket n'est plus possible.");
        else 
	  puts("\nWe cannot write to the socket anymore.");
        raise(SIGINT);
      }
    }
    /* never reatched */
  }
  /* never reatched */
  return (LCRZO_ERR_IEINTERNALERROR);
}

void shared_sockudpser_stdin_sig_received(int i)
{ int status, language;

  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture de la socket (recu : SIG%d)\n", i);
  else
    printf("Closing socket (received : SIG%d)\n", i);
  kill(shared_sockudpser_stdin_pid, SIGTERM);
  waitpid(shared_sockudpser_stdin_pid, &status, 0);
  lcrzo_sock_close(&shared_sockudpser_stdin_sock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
void shared_sock_pipe_sig_received(int i);
int shared_sock_pipe_pid;
lcrzo_sock shared_sock_pipe_sock;

int shared_sock_pipe(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret, language;

  lcrzo_global_get_language(&language);

  /* in case we receive a signal */
  signal(SIGINT, shared_sock_pipe_sig_received);
  shared_sock_pipe_sock=*psock;

  shared_sock_pipe_pid=fork();
  if (shared_sock_pipe_pid<0) return(LCRZO_ERR_FUFORK);
  if (shared_sock_pipe_pid==0)
  { /* this process prints read data (child) */
    while(1)
    { ret=lcrzo_sock_readm(&shared_sock_pipe_sock, LCRZO_TRUE, 
			   &data, &datasize);
      if ( ret!=LCRZO_ERR_OK )
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("La lecture sur la socket n'est plus possible.");
        else
	  puts("We cannot read from the socket anymore.");
        kill(getppid(), SIGINT);
	_exit(ret);
      }
      if ( datasize!=0 )
      { lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT);
      }
      lcrzo_data_free(data);
    }
    /* never reatched */
  }
  else
  { /* this process writes data */
    while(1)
    { lcrzo_epr(lcrzo_fd_readm(0, LCRZO_TRUE, &data, &datasize));
      ret=lcrzo_sock_write(&shared_sock_pipe_sock, data, datasize);
      lcrzo_data_free(data);
      if (ret!=LCRZO_ERR_OK)
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("L'ecriture sur la socket n'est plus possible.");
        else 
	  puts("We cannot write to the socket anymore.");
        raise(SIGINT);
      }
    }
    /* never reatched */
  }
  /* never reatched */
  return (LCRZO_ERR_IEINTERNALERROR);
}

void shared_sock_pipe_sig_received(int i)
{ int status, language;

  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture de la socket (recu : SIG%d)\n", i);
  else
    printf("Closing socket (received : SIG%d)\n", i);
  kill(shared_sock_pipe_pid, SIGTERM);
  waitpid(shared_sock_pipe_pid, &status, 0);
  lcrzo_sock_close(&shared_sock_pipe_sock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
void shared_sockudpser_pipe_sig_received(int i);
int shared_sockudpser_pipe_pid;
lcrzo_sock shared_sockudpser_pipe_sock;

int shared_sockudpser_pipe(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret, language;

  lcrzo_global_get_language(&language);

  /* in case we receive a signal */
  signal(SIGINT, shared_sockudpser_pipe_sig_received);
  shared_sockudpser_pipe_sock=*psock;

  /* we cannot write before reading */
  if ( language==LCRZO_GLOBAL_FRLANG )
    puts("\nNous attendons que le client ecrive.");
  else
    puts("\nWe wait for the client to write.");
  ret=lcrzo_sock_readm(&shared_sockudpser_pipe_sock, LCRZO_TRUE, &data, &datasize);
  
  shared_sockudpser_pipe_pid=fork();
  if (shared_sockudpser_pipe_pid<0) return(LCRZO_ERR_FUFORK);
  if (shared_sockudpser_pipe_pid==0)
  { /* this process prints read data (child) */
    while(1)
    { if ( ret!=LCRZO_ERR_OK )
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("\nLa lecture sur la socket n'est plus possible.");
        else
	  puts("\nWe cannot read from the socket anymore.");
        kill(getppid(), SIGINT);
	_exit(ret);
      }
      if ( datasize!=0 )
      { lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_TEXT);
      }
      lcrzo_data_free(data);
      ret=lcrzo_sock_readm(&shared_sockudpser_pipe_sock, LCRZO_TRUE, 
			   &data, &datasize);
    }
    /* never reatched */
  }
  else
  { /* this process writes data */
    while(1)
    { lcrzo_epr(lcrzo_fd_readm(0, LCRZO_TRUE, &data, &datasize));
      ret=lcrzo_sock_write(&shared_sockudpser_pipe_sock, data, datasize);
      lcrzo_data_free(data);
      if (ret!=LCRZO_ERR_OK)
      { if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("\nL'ecriture sur la socket n'est plus possible.");
        else 
	  puts("\nWe cannot write to the socket anymore.");
        raise(SIGINT);
      }
    }
    /* never reatched */
  }
  /* never reatched */
  return (LCRZO_ERR_IEINTERNALERROR);
}

void shared_sockudpser_pipe_sig_received(int i)
{ int status, language;

  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture de la socket (recu : SIG%d)\n", i);
  else
    printf("Closing socket (received : SIG%d)\n", i);
  kill(shared_sockudpser_pipe_pid, SIGTERM);
  waitpid(shared_sockudpser_pipe_pid, &status, 0);
  lcrzo_sock_close(&shared_sockudpser_pipe_sock);
  raise(SIGTERM);
}


/*-------------------------------------------------------------*/
void shared_sock_echo_sig_received(int i);
lcrzo_sock shared_sock_echo_sock;

int shared_sock_echo(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  int ret, language;

  lcrzo_global_get_language(&language);

  /* in case we receive a signal */
  signal(SIGINT, shared_sock_echo_sig_received);
  shared_sock_echo_sock=*psock;

  while(1)
  { ret=lcrzo_sock_readm(&shared_sock_echo_sock, LCRZO_TRUE, 
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { if ( language==LCRZO_GLOBAL_FRLANG )
        puts("\nLa lecture sur la socket n'est plus possible.");
      else
	puts("\nWe cannot read from the socket anymore.");
      kill(getpid(), SIGINT);
    }
    if ( datasize!=0 )
    { if ( language==LCRZO_GLOBAL_FRLANG )
	puts("On vient de lire :");
      else
        puts("We read :");
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
      lcrzo_epr(lcrzo_sock_write(&shared_sock_echo_sock, data, datasize));
    }
    lcrzo_data_free(data);
  }
  /* never reatched */
  return(LCRZO_ERR_OK);
}

void shared_sock_echo_sig_received(int i)
{ int language;

  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture de la socket (recu : SIG%d)\n", i);
  else
    printf("Closing socket (received : SIG%d)\n", i);
  lcrzo_sock_close(&shared_sock_echo_sock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
void shared_socktcpmulser_echo_sig_received(int i);
lcrzo_sock shared_socktcpmulser_echo_sock;

int shared_socktcpmulser_echo(lcrzo_sock sock, void *pinfos)
{ lcrzo_data data;
  lcrzo_int32 datasize, pid;
  lcrzo_etha clienteth;
  lcrzo_ipl clientip;
  lcrzo_uint16 clientport;
  int ret, language;

  lcrzo_global_get_language(&language);

  /* in case we receive a signal */
  signal(SIGINT, shared_socktcpmulser_echo_sig_received);
  shared_socktcpmulser_echo_sock=sock;

  while(1)
  { ret=lcrzo_sock_readm(&shared_socktcpmulser_echo_sock, LCRZO_TRUE, 
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { if ( language==LCRZO_GLOBAL_FRLANG )
        puts("\nLa lecture sur la socket n'est plus possible.");
      else
	puts("\nWe cannot read from the socket anymore.");
      kill(getpid(), SIGINT);
    }
    if ( datasize!=0 )
    { /* print information about the client */
      lcrzo_epr(lcrzo_sock_get_infos(&shared_socktcpmulser_echo_sock, NULL,
				     NULL, NULL, NULL,
				     clienteth, &clientip, &clientport));
      pid=getpid();
      printf("[%ld] ", pid);
      if ( language==LCRZO_GLOBAL_FRLANG )
	printf("On a lu");
      else
        printf("We read");
      lcrzo_epr(lcrzo_etha_print(" (client ", clienteth, "-"));
      lcrzo_epr(lcrzo_ipl_print("", clientip, "-"));
      printf("%d) :\n", clientport);
      /* print the read data */
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
      /* write back a copy to the socket */
      lcrzo_epr(lcrzo_sock_write(&shared_socktcpmulser_echo_sock, 
				 data, datasize));
      /* if we read 'QUIT', we exit */
      if ( lcrzo_data_search_text(data, datasize, LCRZO_FALSE,
				  +1, -1, "QUIT",
				  NULL, NULL, NULL, NULL)
	   == LCRZO_ERR_OK )
      { pid=getpid();
        printf("[%ld] ", pid);
        if ( language==LCRZO_GLOBAL_FRLANG )
	  puts("Le client quitte");
        else
	  puts("The client sent a QUIT");
        raise(SIGINT);
      }
    }
    lcrzo_data_free(data);
  }
  /* never reatched */
  return(LCRZO_ERR_OK);
}

void shared_socktcpmulser_echo_sig_received(int i)
{ int language;

  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture de la socket (recu : SIG%d)\n", i);
  else
    printf("Closing socket (received : SIG%d)\n", i);
  lcrzo_sock_close(&shared_socktcpmulser_echo_sock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
void shared_sock_display_sig_received(int i);
lcrzo_sock shared_sock_display_sock;

int shared_sock_display(lcrzo_sock *psock)
{ lcrzo_data data;
  lcrzo_int32 datasize;
  lcrzo_etha clienteth;
  lcrzo_ipl clientip;
  lcrzo_uint16 clientport;
  int ret, language;

  lcrzo_global_get_language(&language);

  /* in case we receive a signal */
  signal(SIGINT, shared_sock_display_sig_received);
  shared_sock_display_sock=*psock;

  while(1)
  { ret=lcrzo_sock_readm(&shared_sock_display_sock, LCRZO_TRUE, 
			 &data, &datasize);
    if ( ret!=LCRZO_ERR_OK )
    { if ( language==LCRZO_GLOBAL_FRLANG )
        puts("\nLa lecture sur la socket n'est plus possible.");
      else
	puts("\nWe cannot read from the socket anymore.");
      kill(getpid(), SIGINT);
    }
    if ( datasize!=0 )
    { lcrzo_epr(lcrzo_sock_get_infos(&shared_sock_display_sock, NULL,
				     NULL, NULL, NULL,
				     clienteth, &clientip, &clientport));
      if ( language==LCRZO_GLOBAL_FRLANG )
	printf("On vient de lire");
      else
        printf("We read");
      lcrzo_epr(lcrzo_etha_print(" (client ", clienteth, "-"));
      lcrzo_epr(lcrzo_ipl_print("", clientip, "-"));
      printf("%d) :\n", clientport);
      lcrzo_epr(lcrzo_data_print(data, datasize, LCRZO_PRINTTYPE_DUMP));
    }
    lcrzo_data_free(data);
  }
  /* never reatched */
  return(LCRZO_ERR_OK);
}

void shared_sock_display_sig_received(int i)
{ int language;

  lcrzo_global_get_language(&language);
  if ( language==LCRZO_GLOBAL_FRLANG )
    printf("Fermeture de la socket (recu : SIG%d)\n", i);
  else
    printf("Closing socket (received : SIG%d)\n", i);
  lcrzo_sock_close(&shared_sock_display_sock);
  raise(SIGTERM);
}

/*-------------------------------------------------------------*/
